//------------------------------------------------------------------------------
// File: PrimoAV.h
// (c) Copyright 2003-2011  Primo Software Corp. All rights reserved.      
//------------------------------------------------------------------------------
#ifndef ___PRIMO_AV_H___
#define ___PRIMO_AV_H___

/** @file */

#include "PrimoReference.h"
#include "PrimoError.h"

#undef min

namespace primo
{
    /**
    Shared namespace for error definitions
    */
    namespace error
    {
        /**
        Common error facilities
        */
        namespace ErrorFacility
        {
            /**
            Common errors for encoders, decoders, muxers, demuxers and other A/V components.

            @see primo::codecs::CodecError::Enum
            */
            const int32_t Codec = 9;
        }
    }

    /**
    Common audio/video types and interfaces
    */
    namespace codecs {

        // TODO: Add images from https://en.wikipedia.org/wiki/Alpha_compositing
        /**
        Defines the way a foreground image will be combined with background video
        */
        namespace AlphaCompositingMode
        {
            enum Enum
            {
                /**
                Foreground over Background - in effect, normal painting operation.

                @remarks
                If Fa is foreground alpha, Frgb is foreground RGB, Ba is background alpha, Brgb is background RGB, then Rrgb is the compositing result:

                Rrgb = (Fa * Frgb) + (1 - Fa ) * (Ba * Brgb)

                Normally, alpha blending implies a background alpha of 1.0 (Ba = 1.0), so the compositing result Rrgb becomes:

                Rrgb = (Fa * Frgb)  + (1 - Fa) * Brgb

                Fa and Ba are alpha density value with range from 0.0 to 1.0
                */
                Over = 1,

                /**
                Foreground in Background - the alpha compositing equivalent of clipping / itersection.

                @remarks
                If Fa is foreground alpha, Frgb is foreground RGB, Ba is background alpha, Brgb is background RGB, then Rrgb is the compositing result:

                Rrgb = (Fa * Frgb) * Ba

                Fa and Ba are alpha density value with range from 0.0 to 1.0
                */
                In = 2,

                /**
                Foreground outside Background.

                @remarks
                If Fa is foreground alpha, Frgb is foreground RGB, Ba is background alpha, Brgb is background RGB, then Rrgb is the compositing result:

                Rrgb = (Fa * Frgb) * (1 - Ba)

                Fa and Ba are alpha density value with range from 0.0 to 1.0
                */
                Out = 3,

                /**
                Foreground atop Background.

                @remarks
                If Fa is foreground alpha, Frgb is foreground RGB, Ba is background alpha, Brgb is background RGB, then Rrgb is the compositing result:

                Rrgb = (Fa * Frgb) * Ba + (1 - Fa) * (Ba * Brgb)

                Fa and Ba are alpha density value with range from 0.0 to 1.0
                */
                Atop = 4,

                /**
                Foreground xor Background.

                @remarks
                If Fa is foreground alpha, Frgb is foreground RGB, Ba is background alpha, Brgb is background RGB, then Rrgb is the compositing result:

                Rrgb = (Fa * Frgb) * (1 - Ba) + (1 - Fa) * (Ba * Brgb)

                Fa and Ba are alpha density value with range from 0.0 to 1.0
                */
                Xor = 5,

                /**
                Foreground plus Background.

                @remarks
                If Fa is foreground alpha, Frgb is foreground RGB, Ba is background alpha, Brgb is background RGB, then Rrgb is the compositing result:

                Rrgb = (Fa * Frgb) + (Ba * Brgb)

                Fa and Ba are alpha density value with range from 0.0 to 1.0
                */
                Plus = 6,
            };
        }

        /**
        Common errors for encoders, decoders, muxers, demuxers and other A/V components.

        @see primo::ErrorFacility::Codec
        */
        namespace CodecError
        {
            enum Enum
            {
                /*
                Success.
                */
                Success = 0,

                /**
                Failed to initialize or open codec
                */
                Init = 1,

                /**
                Codec configuration data is required, but is missing.
                This data is part of a container format and is stored separately
                from the audio/video stream.
                It is essential for decoding an elementary stream stored in the container.
                */
                NoConfigData = 2,

                /**
                No output has been generated (insufficient input data).
                This is a non-fatal error.
                The component may recover after it receives more valid data.
                */
                NoOutput = 3,

                /**
                The output buffer is not big enough for the generated output data.
                */
                OutputBufferNotEnough = 4,

                /**
                Unsupported format or operation.
                */
                Unsupported = 5,

                /**
                The operation failed for unknown reason.
                */
                Failed = 6,

                /**
                The operation cannot be completed because the component is not initialized.
                */
                NotInitialized = 7,

                /**
                One or more of the input parameters are <b>NULL</b>
                but are expected to be valid pointers.
                */
                NullPointer = 8,

                /**
                End of stream; no more data.
                */
                EOS = 9,

                /**
                Failed to allocate memory.
                */
                Alloc = 10,

                /**
                Invalid bitstream.
                */
                InvalidStream = 11,

                /**
                The operation is not implemented yet.
                */
                NotImplemented = 12,

                /**
                At least one of the input parameters has an invalid value for the specified operation.
                Check ErrorInfo::hint() for details.
                */
                InvalidParams = 13,

                /**
                The operation is not allowed.
                Check ErrorInfo::hint() for more details.
                */
                InvalidOperation = 14,

                /**
                A required library/dependency is not present.
                Check ErrorInfo::hint() for more details.
                */
                MissingDependency = 15,

                /**
                Could not open a user stream (primo::Stream).
                */
                StreamOpen = 16,

                /**
                Could not read from a user stream (primo::Stream).
                */
                StreamRead = 17,

                /**
                Could not write to a user stream (primo::Stream).
                */
                StreamWrite = 18,

                /**
                Could not seek in a user stream (primo::Stream).
                */
                StreamSeek = 19,

                /**
                One of the input/output buffers is full and cannot accept more data.
                */
                BufferFull = 20,

                /**
                Intel QuickSync Video hardware not found.
                */
                MissingHardwareIntelQuickSync = 21,

                /**
                AMD VCE hardware not found.
                */
                MissingHardwareAmdVce = 22,

                /**
                NVIDIA NVENC hardware not found.
                */
                MissingHardwareNvidiaNVENC = 23,
            };
        }


        /**
        Defines constants for various LPCM properties.

        The constants are expressed as bit flags and can be combined together.

        @see AudioStreamInfo::flags
        */
        namespace PcmFlags
        {
            /**
            PcmFlags::Enum
            */
            enum Enum
            {
                /**
                No flag is set.
                */
                None = 0,

                /**
                The audio sample should be interpreted as an unsigned integral value.
                The sample resolution (8-bit, 16-bit, etc.) is a separate LPCM property and is not expressed with PcmFlags.
                This flag is irrelevant when the Float flag is set.
                */
                Unsigned = 0x1,

                /**
                The audio sample should be interpreted as a floating point value.
                The sample resolution (32-bit, 64-bit, etc.) is a separate LCPM property and is not expressed with PcmFlags.
                */
                Float = 0x2,

                /**
                The sample value is stored in a big-endian format.
                When this flag is not set the sample format is little-endian.
                */
                BigEndian = 0x4,

                /**
                When this flag is set audio samples from different channels are not interleaved.
                All samples from channel 0 are first in memory. They are followed by all samples from channel 1 and so on
                for all channels.

                When this flag is not set the samples are interleaved. The first samples from all channels are first in memory.
                They are followed by the second samples from all channels and so on until the last sample in time.
                */
                NonInterleaved = 0x8,
            };
        }

        /**
        Defines the speaker location of one or more audio channels.

        These channel constants are expressed as bit flags and can be combined together.
        A number of common speaker layouts are defined for convenience.
        */
        namespace AudioChannelFlags
        {
            /**
            AudioChannelFlags::Enum
            */
            enum Enum
            {
                /** No flag is set. */
                None = 0x0,

                /** Left */
                Left = 0x1,

                /** Right */
                Right = 0x2,

                /** Center */
                Center = 0x4,

                /** Low-frequency effect. Subwoofer. */
                LFE = 0x8,

                /** Back-left. Also known as Left Surround. */
                BackLeft = 0x10,

                /** Back-right. Also known as Right Surround. */
                BackRight = 0x20,

                /** Front, left of center. */
                LeftCenter = 0x40,

                /** Front, right of center. */
                RightCenter = 0x80,

                /** Back-center. Also known as Center Surround. */
                BackCenter = 0x100,

                /** Side-left. Also known as Left Surround Direct. */
                SideLeft = 0x200,

                /** Side-right. Also known as Right Surround Direct. */
                SideRight = 0x400,

                /** Top-center. Also known as Top Center Surround. */
                TopCenter = 0x800,

                /** Top-front-left. Also known as Vertical Height Left. */
                TopFrontLeft = 0x1000,

                /** Top-front-center. Also known as Vertical Height Center. */
                TopFrontCenter = 0x2000,

                /** Top-front-right. Also known as Vertical Height Right. */
                TopFrontRight = 0x4000,

                /** Top-back-left */
                TopBackLeft = 0x8000,

                /** Top-back-center */
                TopBackCenter = 0x10000,

                /** Top-back-right */
                TopBackRight = 0x20000,


                /* Standard layouts */

                /** Mono */
                LayoutMono = Center,

                /** Stereo */
                LayoutStereo = Left | Right,

                /** Standard 3-channel layout (2.1) */
                Layout2p1 = Left | Right | LFE,

                /** Standard 4-channel layout. Surround. */
                LayoutSurround = Left | Right | Center | BackCenter,

                /** Alternative 4-channel layout. */
                LayoutQuad = Left | Right | BackLeft | BackRight,

                /** Standard 5-channel layout (4.1) */
                Layout4p1 = Left | Right | LFE | BackLeft | BackRight,

                /** Standard 6-channel layout (5.1) */
                Layout5p1 = Left | Right | Center | LFE | BackLeft | BackRight,

                /** Standard 8-channel layout (7.1). Surround. */
                Layout7p1Surround = Left | Right | Center | LFE | BackLeft | BackRight | SideLeft | SideRight,

                /** Alternative 6-channel surround (5.1). Surround. */
                Layout5p1Surround = Left | Right | Center | LFE | SideLeft | SideRight,

                /** Alternative 8-channel layout (7.1) */
                Layout7p1 = Left | Right | Center | LFE | BackLeft | BackRight | LeftCenter | RightCenter,

            };
        }

        /**
        Defines the bitrate mode of the audio/video stream.
        */
        namespace BitrateMode
        {
            /**
            BitrateMode::Enum
            */
            enum Enum
            {
                /**
                The bitrate mode is not specified or unknown.
                */
                Unknown = 0,

                /**
                Constant bitrate.

                For uncompressed audio/video with constant bitrate the quality is also constant.
                For compressed audio/video with constant bitrate the quality may be constant or variable.
                */
                CBR = 1,

                /**
                Variable bitrate - quality oriented. Constant quality (more or less).
                */
                VBR = 2,

                /**
                Average bitrate.

                Guarantees a predictable audio/video size similar to CBR.
                The bitrate changes to achieve the best quality within the available bitrate capacity.
                */
                ABR = 3,
            };
        }


        /**
        Defines constants for various color formats.

        A color format is combination of color space, color depth, chroma subsampling, component layout and packing.
        */
        namespace ColorFormat
        {
            /**
            ColorFormat::Enum
            */
            enum Enum
            {
                /** The color format is not specified or unknown. */
                Unknown = 0,

                /** Planar Y, V, U (4:2:0) (note V,U order!) */
                YV12 = 1,

                /** Planar Y, merged U->V (4:2:0) */
                NV12 = 2,

                /**  Composite Y->U->Y->V (4:2:2) */
                YUY2 = 3,

                /** Composite U->Y->V->Y (4:2:2) */
                UYVY = 4,

                /** Planar Y, U, V (4:1:1) */
                YUV411 = 5,

                /** Planar Y, U, V (4:2:0) */
                YUV420 = 6,

                /** Planar Y, U, V (4:2:2) */
                YUV422 = 7,

                /** Planar Y, U, V (4:4:4) */
                YUV444 = 8,

                /** Composite Y, U, V (4:1:1) */
                Y411 = 9,

                /** Composite Y, U, V (4:1:1) */
                Y41P = 10,

                /** Composite B->G->R */
                BGR32 = 11,

                /** Composite B->G->R */
                BGR24 = 12,

                /** Composite B->G->R, 5 bit per B & R, 6 bit per G */
                BGR565 = 13,

                /** Composite B->G->R->A, 5 bit per component, 1 bit per A */
                BGR555 = 14,

                /** Composite B->G->R->A, 4 bit per component */
                BGR444 = 15,

                /** Luminance component only. */
                GRAY = 16,

                /** Planar Y, U, V, Alpha */
                YUV420A = 17,

                /** Planar Y, U, V, Alpha */
                YUV422A = 18,

                /** Planar Y, U, V, Alpha */
                YUV444A = 19,

                /** Planar Y, V, U
                The vertical subsampling interval is 4. The horizontal subsampling interval is also 4
                This means that a single V and a single U sample are taken for each square block of 16 image pixels.
                Effectively this averages 9 bits per pixel (16 pixels => 16Y + 1V + 1U => 144bits => 9bits/pixel)
                */
                YVU9 = 20,

                /** Composite B->G->R->A */
                BGRA32 = 21,
            };
        }

        /**
        Defines major audio and video stream types, and file formats (a.k.a. containers).
        */
        namespace StreamType
        {
            /**
            StreamType::Enum
            */
            enum Enum
            {
                /**
                The stream type is not defined or is unknown.
                */
                Unknown = 0,

                /**
                This constant marks the start of audio elementary streams.
                It is not a valid stream type on its own.
                */
                BEGIN_AUDIO = 0x1000,

                /** Audio encoded using pulse code modulation (PCM) with linear quantization. */
                LPCM = 0x1001,

                /** A-law PCM. */
                ALAW_PCM = 0x1002,

                /** Mu-law PCM. */
                MULAW_PCM = 0x1003,

                /**
                LPCM with specific sample packing for Video-DVD.
                */
                VIDEO_DVD_PCM = 0x1004,


                /** Dolby Digital Audio (a.k.a. AC-3). AC-3 is defined in ATSC A/52-A and A/52-B. */
                AC3 = 0x1005,

                /** DTS (Digital Theater Systems) Coherent Acoustics codec, transportable through S/PDIF and used on DVDs, CD-DAs, LDs, and in wave files. */
                DTS = 0x1006,

                /**
                MPEG-1 or MPEG-2 Audio, Layer I, II and III.
                */
                MPEG_Audio = 0x1007,

                /**
                Vorbis Audio Codec. This is an open standard audio codec developed by the Xiph.Org Foundation.
                */
                Vorbis = 0x1008,

                /**
                Advanced Audio Coding.

                Initially it was defined in Part 7 of the MPEG-2 standard (known formally as ISO/IEC 13818-7) which is now obsolete.

                Currently it is defined in Part 3 of the MPEG-4 standard (known formally as ISO/IEC 14496-3).
                */
                AAC = 0x1009,

                /** Adaptive Multi-Rate audio (Narrow Band). */
                AMRNB = 0x100A,

                /** Adaptive Multi-Rate audio (Wide Band). */
                AMRWB = 0x100B,

                /** G.726 ITU-T ADPCM  */
                G726_ADPCM = 0x100C,

                /** Microsoft Windows Media Audio. */
                WMA = 0x100D,

                /** Microsoft Windows Media Audio Professional. */
                WMA_Professional = 0x100E,

                /** Microsoft Windows Media Audio Lossless. */
                WMA_Lossless = 0x100F,

                /**
                This constant marks the end of audio elementary streams.
                It is not a valid stream type on its own.
                */
                END_AUDIO = 0x1FFF,


                /**
                This constant marks the start of video elementary streams.
                It is not a valid stream type on its own.
                */
                BEGIN_VIDEO = 0x2000,


                /** Uncompressed video stream. */
                UncompressedVideo = 0x2001,

                /** MPEG-1 Video is defined in Part 2 of the MPEG-1 standard (known formally as ISO/IEC-11172-2). */
                MPEG1_Video = 0x2002,

                /** MPEG-2 Video is defined in Part 2 of the MPEG-2 standard (known formally as ISO/IEC 13818-2 and as ITU-T H.262). */
                MPEG2_Video = 0x2003,

                /** MPEG-4 Video is defined in Part 2 of the MPEG-4 standard (known formally as ISO/IEC 14496-2). */
                MPEG4_Video = 0x2004,

                /** H.261 Video is defined in the ITU-T H.261 standard. */
                H261 = 0x2005,

                /** H.263 Video is defined in the ITU-T H.263 standard. */
                H263 = 0x2006,

                /**
                H.264 Video (also known as MPEG-4 AVC / Advanced Video Codec).

                Defined in Part 10 of the MPEG-4 standard (known formally as ISO/IEC 14496-10 and as ITU-T H.264).

                The actual bitstream format is determined by the stream sub-type:

                <p>
                [StreamSubType::AVC_Annex_B](@ref primo::codecs::StreamSubType::AVC_Annex_B)

                H.264 Bitstream WITH start codes

                H.264 bitstreams are transmitted in this format over the air, or contained in MPEG-2 program or transport streams.
                The H.264 bitstream is formatted as described in Annex B of ITU-T Rec. H.264. According to this specification, the bitstream consists of a sequence of NALUs (Network Abstraction Layer Units).
                Each NALU is prefixed with a start code equal to 0x000001 or 0x00000001.

                <p>
                [StreamSubType::AVC1](@ref primo::codecs::StreamSubType::AVC1)

                H.264 Bitstream WITHOUT start codes

                H.264 bitstreams are stored in the MP4 container in this format. Instead of start codes, each NALU (Network Abstraction Layer Units) is prefixed by a length field, which gives the length of the NALU in bytes.
                The size of the length field can vary, but is typically 1, 2, or 4 bytes.
                */
                H264 = 0x2007,

                /**
                Alias. Same as [StreamType::H264](@ref primo::codecs::StreamType::H264).
                */
                AVC = H264,

                /** Microsoft Windows Media Video. */
                WMV = 0x2008,

                /**  Motion JPEG (M-JPEG). */
                MJPEG = 0x2009,

                /** VC-1 Video is defined in the SMPTE 421M-2006 standard. */
                VC1 = 0x200A,

                /** Advanced Video Standard (Chinese national standard) */
                AVS = 0x200B,

                /**
                VP8 is an open video codec released by Google, originally created by On2 Technologies. VP8 is part of the WebM format.
                */
                VP8 = 0x200C,

                /**
                Theora Video Codec. This is an open standard video codec developed by the Xiph.Org Foundation.
                */
                Theora = 0x200D,

                /**
                H.265 Video (also known as MPEG-H HEVC / High Efficiency Video Codec )

                Defined in Part 2 of the MPEG-H standard (known formally as ISO/IEC 23008-2 and as ITU-T H.265).
                */
                H265 = 0x200E,

                /**
                Alias. Same as [StreamType::H265](@ref primo::codecs::StreamType::H265).
                */
                HEVC = H265,

                /**
                This constant marks the end of the elementary video streams.
                It is not a valid stream type on its own.
                */
                END_VIDEO = 0x2FFF,

                /**
                This constant marks the start of audio/video containers.
                It is not a valid stream type on its own.
                */
                BEGIN_CONTAINER = 0x3000,

                /** Audio Video Interleave (AVI) multimedia container format. */
                AVI = 0x3001,

                /**  MPEG-4 multimedia container format specified in Part 14 of the MPEG-4 standard (formally ISO/IEC 14496-14:2003).  */
                MP4 = 0x3002,

                /** Advanced Systems Format (formerly Advanced Streaming Format, Active Streaming Format) (ASF).  */
                ASF = 0x3003,

                /** MPEG program stream. */
                MPEG_PS = 0x3004,

                /** MPEG transport stream. */
                MPEG_TS = 0x3005,

                /** MPEG Packetized Elementary Stream (PES). */
                MPEG_PES = 0x3006,

                /** Waveform Audio file format (WAV, WAVE). */
                WAVE = 0x3007,

                /**
                Adobe Flash Video container format (FLV).
                Not to be confused with F4V which is a newer Flash Video container format based on ISO/IEC 14496-12 (MPEG-4 Part 12).
                */
                FLV = 0x3008,

                /**
                Open standard container format developed by the Xiph.Org Foundation.
                */
                OGG = 0x3009,

                /**
                Open standard container format based on a profile of Matroska.
                */
                WebM = 0x300A,

                /**
                This constant marks the end of audio/video containers.
                It is not a valid stream type on its own.
                */
                END_CONTAINER = 0x3FFF,

                /**
                This constant marks the start of image types.
                It is not an image type on its own.
                */
                BEGIN_IMAGE = 0x4000,

                /**
                BMP image.
                */
                BMP = 0x4001,

                /**
                PNG image.
                */
                PNG = 0x4002,

                /**
                JPEG image.
                */
                JPEG = 0x4003,

                /**
                TIFF image.
                */
                TIFF = 0x4004,

                /**
                GIF image.
                */
                GIF = 0x4005,

                /**
                This constant marks the end of image types.
                It is not an image type on its own.
                */
                END_IMAGE = 0x4FFF,

                /**
                This constant marks the start of various data types.
                */
                BEGIN_DATA = 0x5000,

                /**
                Teletext data
                */
                Teletext = 0x5001,

                /**
                MPEG Transport Stream Program Specific Information
                */
                MPEG_PSI_PACKETS = 0x5002,

                /**
                MPEG transport stream packets.
                This is not a complete transport stream but rather a subset of the original transport stream.
                */
                MPEG_TS_PACKETS = 0x5003,

                /**
                This constant marks the end of various data types.
                */
                END_DATA = 0x5FFF,
            };
        }

        /**
        Defines audio and video stream subtypes.
        */
        namespace StreamSubType
        {
            /**
            StreamSubType::Enum
            */
            enum Enum
            {
                /** The stream subtype is unknown. */
                Unknown = 0,

                /** No stream subtype is defined. */
                None = 0xFFFF,

                /**
                Audio Data Transport Stream: AAC data in frames, similar to mp3. Somewhat compliant.
                Defined in Part 7 of the MPEG-2 standard (known formally as ISO/IEC 13818-7).
                */
                AAC_ADTS = 1,

                /**
                Audio Data Interchange Format: AAC data with a single header. Worst compliance.
                Defined in Part 7 of the MPEG-2 standard (known formally as ISO/IEC 13818-7).
                */
                AAC_ADIF = 2,

                /** AAC data packed in MPEG-4 container (MP4). Best compliance.  */
                AAC_MP4 = 3,

                /**
                H.264/AVC bitstream without start codes.

                This is how the bitstream is stored in a MPEG-4 container (MP4).
                Instead of start codes, each NALU is prefixed by a length field, which gives the length of the NALU in bytes.
                The size of the length field can vary, but is typically 1, 2, or 4 bytes.

                Defined in ISO 14496-15.
                */
                AVCC = 4,

                /**
                Alias. Same as [StreamSubType::AVCC](@ref primo::codecs::StreamSubType::AVCC).
                */
                AVC1 = AVCC,

                /** MPEG2 transport stream with valid packet time stamps. */
                MPEG_TS_BDAV = 5,

                /**
                MPEG-1 Audio Layer I is commonly abbreviated to MP1. MPEG-1 Layer I is defined in ISO/IEC 11172-3.

                MPEG-2 Layer I is defined in the MPEG-2 Part 3 standard (known formally as ISO/IEC 13818-3).
                */
                MPEG_Audio_Layer1 = 6,

                /**
                MPEG-1 Audio Layer II is commonly abbreviated to MP2. MP2 is a dominant standard for audio broadcasting.
                MPEG-1 Layer II is defined in ISO/IEC 11172-3.

                MPEG-2 Layer II is defined in the MPEG-2 Part 3 standard (known formally as ISO/IEC 13818-3).
                */
                MPEG_Audio_Layer2 = 7,

                /**
                MPEG-1 Audio Layer III is commonly abbreviated to MP3. MPEG-1 Layer III is defined in ISO/IEC 11172-3.

                MPEG-2 Layer III is defined in the MPEG-2 Part 3 standard (known formally as ISO/IEC 13818-3).
                */
                MPEG_Audio_Layer3 = 8,

                /**
                G.726 stream packed according to RFC 3551.
                This type of packetization is used in RTP.
                */
                G726_PACKED_RFC3551 = 9,

                /**
                G.726 stream packed according to ITU-T I.366.2 (AAL type 2).
                This type of packetization is used in Wave files.
                */
                G726_PACKED_AAL2 = 10,

                /**
                MPEG-1 Program Stream as specified in MPEG-1 Part 1 (ISO/IEC 11172-1).
                This stream subtype can be used only with the [MPEG_PS](@ref primo::codecs::StreamType::MPEG_PS) stream type.
                */
                MPEG1_System = 11,

                /**
                MPEG-2 Program Stream as specified in MPEG-2 Part 1 (ISO/IEC 13818-1).
                This stream subtype can be used only with the [MPEG_PS](@ref primo::codecs::StreamType::MPEG_PS) stream type.
                */
                MPEG2_System = 12,

                /**
                H.264 bitstream with start codes.

                H.264 bitstreams are transmitted in this format over the air, or contained in MPEG-2 program or transport streams.
                The H.264 bitstream is formatted as described in Annex B of ITU-T Rec. H.264. According to this specification,
                the bitstream consists of a sequence of NALUs (Network Abstraction Layer Units).
                Each NALU is prefixed with a start code equal to 0x000001 or 0x00000001.
                */
                AVC_Annex_B = 13,

                /**
                    Raw AAC data format: AAC data is stored without headers.
                */
                AAC_RAW = 14,

            };
        }

        /**
        Defines video interlace/scan types.
        */
        namespace ScanType
        {
            /**
            ScanType::Enum
            */
            enum Enum
            {
                /** The interlace/scan type is not specified or not known. */
                Unknown = 0,

                /** Progressive (non-interlaced) video. */
                Progressive = 1,

                /** Interlaced video, where the dominant (first in time) field is the top field. */
                TopFieldFirst = 2,

                /** Interlaced video, where the dominant (first in time) field is the bottom field. */
                BottomFieldFirst = 3,
            };
        }

        /**
        Defines video deinterlacing methods
        */
        namespace DeinterlacingMethod
        {
            /**
            DeinterlacingMethod::Enum
            */
            enum Enum
            {
                /**
                No deinterlacing is performed.
                This can be used to prevent automatic deinterlacing if it's not desired.
                */
                NoDeinterlacing = 0,

                /**
                Line doubling.
                */
                Duplicate = 1,

                /**
                Motion adaptive deinterlacing
                */
                Blend = 2,

                /**
                Calculates the median of consecutive fields
                */
                Median = 3,

                /**
                Uses an edge detection filter.
                */
                EdgeDetect = 4,

                /**
                Calculates the median of consecutive fields using a threshold.
                */
                MedianThreshold = 5,

                /**
                Content adaptive vertical temporal (CAVT) filtering
                */
                CAVT = 6
            };
        }

        /**
        Defines methods for sample interpolation
        */
        namespace InterpolationMethod
        {
            /**
            InterpolationMethod::Enum
            */
            enum Enum
            {
                /**
                The interpolation method is unknown or not set.
                */
                Unknown = 0,

                /**
                Nearest neighbor interpolation
                */
                NearestNeighbor = 1,

                /**
                Linear interpolation
                */
                Linear = 2,

                /**
                Cubic convolution interpolation
                */
                Cubic = 4,

                /**
                Supersampling interpolation.
                */
                Super = 8,

                /**
                Interpolation by 3-lobed Lanczos-windowed sinc function
                */
                Lanczos = 16
            };
        }

        /**
        Defines H.264/AVC profiles

        @note The constants follow the H.264 spec for the element "profile_idc"
        */
        namespace H264Profile
        {
            /**
            H264Profile::Enum
            */
            enum Enum
            {
                /**
                None/Auto
                */
                None = 0,

                /**
                Baseline profile
                */
                Baseline = 66,

                /**
                Main profile
                */
                Main = 77,

                /**
                High
                */
                High = 100,

                /*
                High 10
                */
                High10 = 110,

                /*
                High 4:2:2
                */
                High422 = 122,

                /*
                High 4:4:4
                */
                High444 = 144
            };
        }

        /**
        Defines H.264/AVC entropy coding modes

        @note The constants follow the H.264 spec for the element "entropy_coding_mode_flag"
        */
        namespace H264EntropyCodingMode
        {
            /**
            H264EntropyCodingMode::Enum
            */
            enum Enum
            {
                /**
                Context-Adaptive Variable-Length Coding (CAVLC)
                */
                CAVLC = 0,

                /**
                Context-Adaptive Binary Arithmetic Coding (CABAC)
                */
                CABAC = 1,
            };
        }

        /**
        Defines H.264/AVC deblocking filter mode.

        @note The constants follow the H.264 spec for the element "disable_deblocking_filter_idc"
        */
        namespace H264DeblockingFilter
        {
            /**
            H264DeblockingFilter::Enum
            */
            enum Enum
            {
                /**
                Deblocking filter is applied on all luma and chroma block edges of the slice.
                */
                All = 0,

                /**
                Deblocking filter is disabled for all block edges of the slice.
                */
                Off = 1,

                /**
                Deblocking filter is applied on all luma and chroma block edges of the slice
                with exception of the block edges that coincide with slice boundaries.
                */
                InSlice = 2,
            };
        }

        /**
        Defines H.264/AVC direct predition mode.

        @note The constants follow the H.264 spec for the element "direct_spatial_mv_pred_flag"
        */
        namespace H264DirectPredMode
        {
            /*
            H264DirectPredMode::Enum
            */
            enum Enum
            {
                /**
                Temporal direct mode prediction
                */
                Temporal = 0,

                /**
                Spatial direct mode prediction
                */
                Spatial = 1,
            };
        }

        /**
        Defines the block sizes for which motion estimation should be done.

        @note Used when encoding H.264/AVC.
        */
        namespace H264MeSplitMode
        {
            /*
            H264MeSplitMode::Enum
            */
            enum Enum
            {
                /**
                Analyze only 16x16 blocks for motion estimation
                */
                Only16x16 = 0,

                /**
                Analyze all blocks down to 8x8 (16x16, 16x8, 8x16, 8x8)
                */
                DownTo8x8 = 1,

                /**
                Analyze all blocks down to 4x4 (16x16, 16x8, 8x16, 8x8, 8x4, 4x8, 4x4).

                @note This partitioning mode is very slow.
                */
                DownTo4x4 = 2,
            };
        }

        /**
        Defines the motion estimation methods used when encoding H.264/AVC
        */
        namespace H264MeMethod
        {
            /**
            H264MeMethod::Enum
            */
            enum Enum
            {
                /**
                Full (slowest)
                */
                Full = 0,

                /**
                TDL (Two Dimensional Logarithmic Search)
                */
                ClassicLog = 1,

                /**
                Log
                */
                Log = 2,

                /**
                EPZS (Enhanced Predictive Zonal Search)
                */
                EPZS = 3,

                /**
                OSA (Full orthogonal)
                */
                FullOrthogonal = 4,

                /**
                Log orthogonal
                */
                LogOrthogonal = 5,

                /**
                UMH (Uneven Multi-Hexagon Search)
                */
                UMH = 8
            };
        }

        /**
        Defines the picture coding type when encoding H.264/AVC
        */
        namespace H264PicCodingType
        {
            /**
            H264PicCodingType::Enum
            */
            enum Enum
            {
                /**
                Video pictures are coded as frames (progressive video)
                */
                Frames = 0,

                /**
                Video pictures are coded as fields (interlaced video)
                */
                Fields = 1,
            };
        }

        /**
        Defines methods for bitrate contorl when encoding H.264/AVC
        */
        namespace H264RateControlMethod
        {
            /**
            H264RateControlMethod
            */
            enum Enum
            {
                /**
                Average Bitrate.
                Ensures that the output stream achieves a predictable long-term average bitrate.
                */
                ABR = 2,

                /**
                Constant Quantization.
                Encodes the video to a constant quantizer. The encoder uses the specified target quantizer, not a target bitrate.
                The quantizer is a measure for the amount of data loss: a higher quantizer means that more data will be lost,
                which results in a better compression, but also delivers worse visual quality.
                This method could be used when a certain level of quality is required and the final bitrate is not a concern.
                The bitrate is unpredictable in this mode.
                */
                ConstantQuant = 3,
            };
        }

        /**
        Defines various media sample features.

        These features are expressed as bit flags and can be combined together.
        */
        namespace MediaSampleFlags
        {
            /**
            MediaSampleFlags::Enum
            */
            enum Enum
            {
                /**
                No feature is specified.
                */
                None = 0x0,

                /**
                The media sample / frame can be decoded on its own. It does not depend on previous or subsequent samples / frames.
                */
                KeyFrame = 0x1,

                /*
                Beginning of stream
                */
                Bos = 0x2,

                /*
                End of stream
                */
                Eos = 0x4,
            };
        }

        /**
        Defines video picture types (I/P/B)
        */
        namespace PictureType
        {
            /**
            PictureType::Enum
            */
            enum Enum
            {
                /**
                Picture type is unknown or not specified.
                */
                None = 0,

                /**
                I Picture
                */
                I = 1,

                /**
                P Picture
                */
                P = 2,

                /**
                B Picture
                */
                B = 3,

                /**
                D Picture (MPEG-1 only)
                */
                D = 4,
            };
        }

        /**
        Defines H.265/HEVC profiles
        */
        namespace H265Profile
        {
            /**
            H265Profile::Enum
            */
            enum Enum
            {
                /**
                None/Auto
                */
                None = 0,

                /**
                Main profile
                */
                Main = 1,

                /**
                Main 10
                */
                Main10 = 2,
            };
        }

        /**
        Defines H.265/HEVC levels
        */
        namespace H265Level
        {
    /**
    H265Level::Enum
    */
            enum Enum
            {
                /**
                None/Auto
                */
                None = 0,
                /**
                30
                */
                L1 = 30,
                /**
                60
                */
                L2 = 60,
                /**
                63
                */
                L21 = 63,
                /**
                90
                */
                L3 = 90,
                /**
                93
                */
                L31 = 93,
                /**
                120
                */
                L4 = 120,
                /**
                123
                */
                L41 = 123,
                /**
                150
                */
                L5 = 150,
                /**
                153
                */
                L51 = 153,
                /**
                156
                */
                L52 = 156,
                /**
                180
                */
                L6 = 180,
                /**
                183
                */
                L61 = 183,
                /**
                186
                */
                L62 = 186,
            };
        }

        /**
        Defines H.265/HEVC tiers
        */
        namespace H265Tier
        {
    /**
    H265Tier::Enum
    */
            enum Enum
            {
                /**
                0
                */
                Main = 0,
                /**
                1
                */
                High = 1
            };
        }

// TBR:
/**
Defines stereo mode in MPEG Audio and AAC.    
*/
namespace StereoMode
{
    /**
    StereoMode::Enum
    */
    enum Enum
    {
        /**
          None/LR mode is chosen by default.
        */
        None    = 0,

        /**
          If <b>MPEG/AAC</b> encoder is used with LR mode the encoding will be done using normal independent coding of left and right channels.
        */
        LR      = 2,

        /**
          If <b>AAC</b> encoder is used with mode MidSide the encoding will be done using Mid/Side coding technique.
          If <b>MPEG Audio</b> encoder is used with this mode:
            - Layers 1 and 2: the encoding will be done using normal independent coding of left and right channels(same as LR mode)
            - Layer 3: the encoding will be done using Mid/Side coding technique.
        */
        MidSide = 3,

        /**
          If <b>AAC</b> encoder is used with mode Joint the encoding of every frame will be done with Mid/Side or Left-Right Stereo coding technique
          depending on the data present in the frame.
          If <b>MPEG Audio</b> encoder is used with this mode:
            - Layer 1: the encoding will be done using normal independent coding of left and right channels(same as LR mode).
            - Layer 2: the encoding will use Intensity coding technique when the bit rate is lower than 96Kb/s for 2 channel setup
              in combination with normal independent coding of left and right channels, otherwise will use only left and right coding.
            - Layer 3: the encoding of every frame will be done with Mid/Side or Left-Right Stereo coding technique
            depending on the data present in the frame.
        */
        Joint   = 4,
    };
}

        /**
        A buffer for media data.

        <b>MediaBuffer</b> has a capacity which is the size of the buffer and data size which is the size of the actual data in the buffer.
        The buffer is a continuous block of memory.
        The valid data is also a continuous block. The data can start at any offset in the buffer.
        */
        class MediaBuffer : public primo::Reference
        {
        public:
            /**
            Sets the offset at which valid data starts in the buffer block and the data size in bytes.
            The data offset and size are validated so that they are always within the buffer.

            @param dataOffset The offset at which valid data starts in the buffer. Allowed values range from 0 to capacity()-1.
            @param dataSize The size of the valid data in the buffer. Allowed values range from 0 to capacity().

            @return TRUE The data offset and size are successfully changed. FALSE The supplied data offset and size are not valid.
            */
            virtual bool_t setData(int32_t dataOffset, int32_t dataSize) = 0;

            /**
            Returns the offset at which valid data starts in the buffer.

            @return An offset that can range from 0 to capacity()-1.
            */
            virtual int32_t dataOffset() const = 0;

            /**
            Returns the size of the valid data that in the buffer.

            @return The data size in bytes. It ranges from 0 to capacity().
            */
            virtual int32_t dataSize() const = 0;

            /**
            Returns a pointer to the first byte of data. This is effectively the same as start() + dataOffset().

            @return A pointer to the data start.
            */
            virtual uint8_t* data() const = 0;

            /**
            Returns a pointer to the start of the buffer.

            @return A pointer to the buffer start. It is guaranteed that the pointer is valid only when capacity() is greater than 0.
            */
            virtual uint8_t* start() const = 0;

            /**
            Returns the size of the buffer in bytes.

            @return Buffer capacity. Can be zero.
            */
            virtual int32_t capacity() const = 0;

            /**
            Allocates internal buffer storage

            @param size Requested buffer size.
            A buffer storage is allocated only if the requested <i>size</i> exceeds the current buffer size (capacity).
            The actual allocated buffer may be larger than the requested <i>size<i>.

            @param keepData Specifies whether to keep the existing data when the existing buffer is resized.

            @return TRUE The allocation request succeeded. FALSE The allocation failed (out of memory).

            @remarks If an external storage is attached it is automatically detached.
            */
            virtual bool_t alloc(int32_t size, bool_t keepData) = 0;

            /**
            Frees internal buffer storage

            @remarks The function does nothing if an external storage is attached to the buffer.
            */
            virtual void free() = 0;

            /**
            Attaches an external buffer storage

            @param buffer A pointer to the buffer storage.

            @param size The buffer size.

            @param setData Specifies whether to auto set data with a size equal to the buffer size.
            If TRUE the data size is set to the buffer size. If FALSE data size is 0.

            @return TRUE The buffer is attached succefully, FALSE otherwise.

            @remarks If an internal storage is already allocated it is automatically freed.
            */
            virtual bool_t attach(void* buffer, int32_t size, bool_t setData) = 0;

            /**
            Detaches an external buffer storage

            @return A pointer to the detached buffer storage. <b>NULL</b> if no external storage has been attached.
            */
            virtual uint8_t* detach() = 0;

            /**
            Returns whether the MediaBuffer uses an external data buffer.

            @return TRUE An external data buffer is attached to this MediaBuffer, FALSE otherwise.
            */
            virtual bool_t external() const = 0;

            /**
            Clears existing data.

            @remarks The operation does not change the buffer capacity.
            */
            virtual void clear() = 0;

            /**
            Appends data to the end of the MediaBuffer data.

            @param data  A pointer to the data that should be appended.

            @param dataSize The number of bytes that should be appended.

            @return TRUE The operation is successful, FALSE otherwise.

            @remarks If MediaBuffer has internal storage it will be increased if needed to fit the <i>data</i>.
            If MediaBuffer has external storage and the provided data cannot fit the operation will fail.
            */
            virtual bool_t append(const void* data, int32_t dataSize) = 0;

            /**
            Removes data from the beginning of the MediaBuffer data.

            @param dataSize The number of bytes that should be removed from the valid data.

            @return TRUE The operation is successful, FALSE otherwise.

            @remarks The operation does not change the buffer capacity.
            */
            virtual void remove(int32_t dataSize) = 0;

            /**
            Moves the existing data to the start of the buffer amd maximizes the free linear space.

            @remarks This may be useful when new data is added through direct copying after the existing data
            and then setData is used to extend/modify the data range.
            The operation does not change the buffer capacity.
            */
            virtual void normalize() = 0;

            /**
            Returns the free <b>linear</b> space that can be used to append data without normalization.

            @return The size in bytes of the free linear space in the buffer.
            This is equal to capacity() - dataSize() - dataOffset().

            @remarks The free linear space starts at data() + dataSize().
            It is less than or equal to freeSpace().

            @see freeSpace
            */
            virtual int32_t freeLinearSpace() const = 0;

            /**
            Returns the free space that can be used to append data without buffer reallocation.

            @return The size in bytes of the free space in the buffer.
            This is equal to capacity() - dataSize().

            @remarks The free space starts at data() + dataSize().
            It is greater than or equal to freeLinearSpace().

            @see freeLinearSpace
            */
            virtual int32_t freeSpace() const = 0;

            /**
            Creates a deep copy of this object.

            @return	A new MediaBuffer object. The caller is responsible for releasing the new object when it is not needed anymore.
            @return <b>NULL</b> if the object cannot be cloned because there's not enough memory.

            @remarks
            Only the pointer to the external buffer is copied when MediaBuffer does not own its data buffer, i.e. uses external / attached data.
            On the contrary, when MediaBuffer owns its data, a new copy of the data is created.

            @see \ref object_management "Object Management"
            @see MediaBuffer::external
            */
            virtual MediaBuffer* clone() const = 0;

        protected:
            /**
            This object can be destroyed only by its implementation. The caller must use Reference::release.

            @see \ref object_management "Object Management"
            */
            ~MediaBuffer() {}

        };


        /**
        MediaSample represents one or more samples of audio or video data.
        It is used for storing uncompressed or compressed data.
        */
        class MediaSample : public primo::Reference
        {

        public:

            /**
            Returns the buffer associated with this sample.

            @return A pointer to a MediaBuffer object. Can be <b>NULL</b>.
            */
            virtual MediaBuffer* buffer() = 0;

            /**
            Sets a new MediaBuffer object.
            The old MediaBuffer object is released. The new one is retained.

            @param buffer A pointer to the new MediaBuffer object. Can be <b>NULL</b>.
            */
            virtual void setBuffer(MediaBuffer* buffer) = 0;

            /**
            Resets the sample to its default values and clears the data in the buffer.
            */
            virtual void reset() = 0;

            /**
            Returns the presentation timestamp (PTS) of the media sample.

            @return time in seconds
            */
            virtual double startTime()  const = 0;

            /**
            Returns the end time of the media sample.

            @return The time when the sample should end. Time in seconds.

            @remarks The end time is optional and is valid only if it is greater than the start time.
            */
            virtual double endTime() const = 0;

            /**
            Sets presentation timestamp (PTS) of the media sample.

            @param time in seconds

            */
            virtual void setStartTime(double time) = 0;

            /**
            Sets the end time of the media sample.

            @param time [in] The time when the sample should end. Time is in seconds.

            @remarks The end time is optional and is valid only if it is greater than the start time.
            */

            virtual void setEndTime(double time) = 0;

            /**
            Returns the media sample flags.

            @return A combination of flags defined in MediaSampleFlags::Enum.
            */
            virtual int32_t flags() const = 0;

            /**
            Sets the media sample flags.

            @param flags [in] A combination of flags defined in MediaSampleFlags::Enum.
            */
            virtual void setFlags(int32_t flags) = 0;

            /**
            Sets the elementary stream to which the sample belongs.

            @param streamNumber [in] A zero-based index of the elementary stream in the container.
            */
            virtual void setStreamNumber(int32_t streamNumber) = 0;

            /**
            Returns the elementary stream to which the sample belongs.

            @return  A zero-based index of the elementary stream in the container.
            */
            virtual int32_t streamNumber() const = 0;


            /**
            Sets the required picture type (I/P/B/etc.).

            @param pictureType [in] A constant from the PictureType::Enum.

            @note Currently this is ignored by encoders.
            */
            virtual void setPictureType(PictureType::Enum pictureType) = 0;

            /**
            Returns the picture type (I/P/B/etc.) of a demuxed, decoded or encoded frame.

            @return  A constant from the PictureType::Enum.

            @note Currently this is returned only by the MPEG-2 encoder.
            */
            virtual PictureType::Enum pictureType() const = 0;

            /**
            Creates a deep copy of this object.

            @return	A new MediaSample object.
            The caller is responsible for releasing the new object when it's not needed anymore.

            @return	<b>NULL</b> if the object cannot be cloned because there is not enough memory.

            @see \ref object_management "Object Management"
            */
            virtual MediaSample* clone() const = 0;

            /**
            Calculates the exact number of bytes that are needed
            for a video frame with the specified properties

            @param frameWidth [in] frame width in pixels.
            @param frameHeight [in] frame height in pixels.
            @param colorFormat [in] the color format represents how colors are coded in the video frame.

            @return The number of bytes needed to store a video frame with the specified properties.
            Zero if it is not possible to calculate the video frame buffer size from the supplied arguments.
            */
            virtual int32_t videoBufferSizeInBytes(int32_t frameWidth,
                int32_t frameHeight,
                ColorFormat::Enum colorFormat) const = 0;

        protected:
            /**
            This object can be destroyed only by its implementation. The caller must use Reference::release.

            @see \ref object_management "Object Management"
            */
            ~MediaSample() {}

        };


        /**
        Defines media types.
        */
        namespace MediaType
        {
            /**
            MediaType::Enum
            */
            enum Enum
            {
                /** The media type is unknown or not specified. */
                Unknown = 0,

                /** Audio data. */
                Audio = 1,

                /** Video data */
                Video = 2,

                /** Text data */
                Text = 3,

                /** Generic data */
                Data = 4,
            };
        }


        /**
        Provides properties and operations that are common for all elementary streams.

        AudioStreamInfo and VideoStreamInfo inherit from this interface.

        @see AudioStreamInfo
        @see VideoStreamInfo
        */
        class StreamInfo : public primo::Reference
        {
        public:
            /**
            Returns the stream media type. The media type of a StreamInfo object cannot be changed.

            @return media type

            If the media type is MediaType::Audio this StreamInfo object can be safely cast to AudioStreamInfo.

            If the media type is MediaType::Video this StreamInfo object can be safely cast to VideoStreamInfo.
            */
            virtual MediaType::Enum mediaType() const = 0;

            /**
            Returns the stream type.

            @return stream type
            */
            virtual StreamType::Enum streamType() const = 0;

            /**
            Sets the stream type.

            @param type major stream type.

            @return TRUE if the stream type is successfully set, otherwise FALSE.
            */
            virtual bool_t setStreamType(StreamType::Enum type) = 0;

            /**
            Returns the stream subtype.

            @return stream subtype
            */
            virtual StreamSubType::Enum streamSubType() const = 0;

            /**
            Sets the stream subtype.

            @param type stream subtype.

            @return TRUE if the stream subtype is successfully set, otherwise FALSE.
            */
            virtual bool_t setStreamSubType(StreamSubType::Enum type) = 0;

            /**
            Returns the stream duration.

            @return duration in seconds
            */
            virtual double duration() const = 0;

            /**
            Sets the stream duration in seconds.

            @param duration in seconds

            @return TRUE if the duration is successfully set, otherwise FALSE.
            */
            virtual bool_t setDuration(double duration) = 0;

            /**
            Returns the ID of the elementary stream.

            @return stream id
            */
            virtual int32_t ID() const = 0;

            /**
            Sets the ID of the elementary stream.

            @param id elementary stream id

            @return TRUE if the ID is successfully set, otherwise FALSE.
            */
            virtual bool_t setID(int32_t id) = 0;

            /**
            Returns the number of the program to which the elementary stream belongs.

            @return program number

            @warning If the elementary stream is shared by 2 or more programs this property
            will return the last parsed program that uses this stream.
            */
            virtual int32_t programNumber() const = 0;

            /**
            Sets the program to which the elementary stream belongs.

            @param programNumber program number

            @return TRUE if the program number is successfully set, otherwise FALSE.
            */
            virtual bool_t setProgramNumber(int32_t programNumber) = 0;

            /**
            Returns the stream bitrate.

            @return bitrate in bits per second.
            */
            virtual int32_t bitrate() const = 0;

            /**
            Sets the stream bitrate.

            @param bitrate the bitrate is expressed in bits per second.

            @return TRUE if the bitrate is successfully set, otherwise FALSE.
            */
            virtual bool_t setBitrate(int32_t bitrate) = 0;

            /**
            Returns the stream bitrate mode.

            @return A BitrateMode::Enum constant.
            */
            virtual int32_t bitrateMode() const = 0;

            /**
            Sets the stream bitrate mode.

            @param bitrateMode A BitrateMode::Enum constant.

            @return TRUE if the bitrate mode is successfully set, otherwise FALSE.
            */
            virtual bool_t setBitrateMode(int32_t bitrateMode) = 0;


            /**
            Creates a copy of this object.

            @return	A new StreamInfo object that has the same dynamic type as the cloned object.

            When the object is cloned the full underlying object is copied.

            For example if a StreamInfo object is actually an instance of AudioStreamInfo and is cloned,
            the new object is also an instance of AudioStreamInfo.

            The caller is responsible for releasing the new object when it is not needed anymore.

            @remarks
            When cloned an immutable object becomes mutable.

            @see \ref object_management "Object Management"
            */
            virtual StreamInfo* clone() const = 0;

            /**
            Copies the state of the current object to a destination object of the same type.
            The reference count of the source and the destination is not modified.

            @param destination [out] A pointer to the destination object

            @return TRUE The object state is copied, FALSE otherwise.
            */
            virtual bool_t copyTo(StreamInfo* destination) const = 0;


            /**
            Resets the stream information to its default state.

            The full underlying object is reset. The media type of the stream is never reset.

            @return TRUE The object is sucessfully reset.
            @return FALSE The object is immutable and cannot be reset.
            */
            virtual bool_t reset() = 0;

            /**
            Returns whether the object is immutable.
            An immmutable object cannot be modified and all modifying methods fail to produce a result.

            @return TRUE if the object is immutable, FALSE otherwise.

            @remarks
            An immutable object <b>can</b> be modified by the AVBlocks library.

            @remarks
            Object immutability spreads to all nested objects.
            Therefore it is not possible to add/set an immutable object to a mutable object.

            @remarks
            When cloned an immutable object becomes mutable.
            */
            virtual bool_t immutable() const = 0;

        protected:
            /**
            This object can be destroyed only by its implementation. The caller must use Reference::release.

            @see \ref object_management "Object Management"
            */
            ~StreamInfo() {}
        };

        /**
        StreamInfoEnum is a read-only collection of StreamInfo objects.
        */
        class StreamInfoEnum : public primo::Reference
        {
        public:
            /**
            Returns the number of StreamInfo objects in the collection.

            @return number of objects.
            */
            virtual int32_t count() const = 0;

            /**
            Returns the stream information for the specified item.

            @param index [in] The stream index in the collection.

            @return A pointer to a StreamInfo object specified by index or <b>NULL</b> if the index is invalid.

            The caller is not required to release the returned object.

            @see \ref object_management "Object Management"

            */
            virtual primo::codecs::StreamInfo* at(int32_t index) const = 0;

        protected:
            /**
            This object can be destroyed only by the AVBlocks library.
            */
            ~StreamInfoEnum() {}

        };

        /**
        AudioStreamInfo describes an elementary audio stream.
        */
        class AudioStreamInfo : public StreamInfo
        {

        public:

            /**
            Returns the audio format flags which describe various properties of the audio stream.

            @return A combination of flags defined in PcmFlags::Enum.
            */
            virtual int32_t pcmFlags() const = 0;


            /**
            Sets the audio format flags which describe various properties of the audio stream.

            @param flags A combination of flags defined in PcmFlags::Enum.

            @return TRUE if the flags are successfully set, otherwise FALSE.
            */
            virtual bool_t setPcmFlags(int32_t flags) = 0;

            /**
            Returns the number of audio channels.

            @return Number of channels.
            */
            virtual int32_t channels() const = 0;

            /**
            Sets the number of audio channels.

            @param numChannels number of channels.

            @return TRUE if the number of channels is successfully set, otherwise FALSE.
            */
            virtual bool_t setChannels(int32_t numChannels) = 0;

            /**
            Returns the channel layout which defines the mapping between channels and speakers.

            @return A combination of flags defined in the AudioChannelFlags::Enum.
            */
            virtual int32_t channelLayout() const = 0;

            /**
            Sets the channel layout which defines the mapping between channels and speakers.

            @param channelLayout A combination of flags defined in the AudioChannelFlags::Enum.

            @return TRUE if the channel layout is successfully set, otherwise FALSE.
            */
            virtual bool_t setChannelLayout(int32_t channelLayout) = 0;

            /**
            Returns the audio sample rate.

            @return sample rate (Hz)
            */
            virtual int32_t sampleRate() const = 0;

            /**
            Sets the audio sample rate.

            @param sampleRate sample rate (Hz)

            @return TRUE if the sample rate is successfully set, otherwise FALSE.
            */
            virtual bool_t setSampleRate(int32_t sampleRate) = 0;

            /**
            Returns the number of bits per sample (sample resolution).

            @return bits per sample (usually 8, 16, 24 or 32).

            Normally bits per sample is not defined for a compressed audio stream and is 0.
            However it is possible for a compressed audio stream to return a positive value as an indication
            of audio quality. It can also designate the bits per sample of the LPCM stream once the compressed stream is decoded.
            */
            virtual int32_t bitsPerSample() const = 0;

            /**
            Sets the number of bits per sample (sample resolution).

            @param bitsPerSample (usually 8, 16, 24 or 32)

            @return TRUE if the number of bits per sample is successfully set, otherwise FALSE.
            */
            virtual bool_t setBitsPerSample(int32_t bitsPerSample) = 0;

            /**
            Returns the number of bytes occupied by an audio frame (smallest discrete unit).

            @return bytes per frame.

            A frame is the smallest unit in an audio stream. It may contain one or more samples.
            Depending on the audio stream type the frame may contain fixed or variable number of samples.

            For compressed streams the frame size is variable even if there are fixed number of samples in each frame.
            In this case bytes per frame is 0.

            For an LCPM stream the frame size is constant.
            In this case bytes per frame is a positive value.
            Normally for interleaved LPCM streams the following is true:
            - bytesPerFrame() / channels() == (sample size in bytes)
            - The valid bits in a sample are given by bitsPerSample() and they are aligned towards the hi-order byte.

            @return TRUE if bytes per sample is successfully set, otherwise FALSE.
            */
            virtual int32_t bytesPerFrame() const = 0;

            /**
            Sets the number of bytes occupied by an audio frame (smallest discrete unit).

            @param bytesPerFrame [in] Specifies how many bytes are occupied by each audio frame.
            Normally it is valid (greater than 0) for LPCM streams and invalid (0) for compressed audio streams.

            @return TRUE if the number of bytes per frame is successfully set, otherwise FALSE.
            */
            virtual bool_t setBytesPerFrame(int32_t bytesPerFrame) = 0;

            /**
            Copies the state of the current object to a destination object of the same type.
            The reference count of the source and the destination is not modified.

            @param destination [out] A pointer to the destination object

            @return TRUE if the object state is copied, FALSE otherwise.
            */
            virtual bool_t copyTo(AudioStreamInfo * destination) const = 0;

            /**
            Creates a copy of this object.

            @return A new AudioStreamInfo.
            The caller is responsible for releasing the new object when it is not needed anymore.

            @see \ref object_management "Object Management"
            */
            virtual AudioStreamInfo* clone() const = 0;

        protected:
            /**
            This object can be destroyed only by its implementation. The caller must use Reference::release.

            @see \ref object_management "Object Management"
            */
            ~AudioStreamInfo() {}
        };

        /**
        VideoStreamInfo describes an elementary video stream.
        */
        class VideoStreamInfo : public StreamInfo
        {

        public:

            /**
            Returns the video frame width.

            @return video frame width in pixels.
            */
            virtual int32_t frameWidth() const = 0;

            /**
            Sets the video frame width.

            @param frameWidth in pixels.

            @return TRUE if the frame width is successfully set, otherwise FALSE.
            */
            virtual bool_t setFrameWidth(int32_t frameWidth) = 0;

            /**
            Returns the video frame height.

            @return video frame height in pixels.
            */
            virtual int32_t frameHeight() const = 0;

            /**
            Sets the video frame height.

            @param frameHeight in pixels

            @return TRUE if the frame height is successfully set, otherwise FALSE.
            */
            virtual bool_t setFrameHeight(int32_t frameHeight) = 0;

            /**
            Returns the vertical dimension of the display aspect ratio.

            @return vertical display ratio.
            */
            virtual int32_t displayRatioHeight() const = 0;

            /**
            Returns the horizontal dimension of the display aspect ratio.

            @return horizontal display ratio.
            */
            virtual int32_t displayRatioWidth() const = 0;

            /**
            Sets the vertical dimension of the display aspect ratio.

            @param displayHeight The value has no meaning on its own. It serves only to express the display aspect ratio.

            @return TRUE if the display height is successfully set, otherwise FALSE.
            */
            virtual bool_t setDisplayRatioHeight(int32_t displayHeight) = 0;

            /**
            Sets the horizontal dimension of the display aspect ratio.

            @param displayWidth The value has no meaning on its own. It serves only to express the display aspect ratio.

            @return TRUE if the display width is successfully set, otherwise FALSE.
            */
            virtual bool_t setDisplayRatioWidth(int32_t displayWidth) = 0;

            /**
            Returns the video frame rate.

            @return video frame rate in frames per second.
            */
            virtual double frameRate() const = 0;

            /**
            Sets the video frame rate.

            @param frameRate in frames per second.

            @return TRUE if the frame rate is successfully set, otherwise FALSE.
            */
            virtual bool_t setFrameRate(double frameRate) = 0;

            /**
            Returns the video color format.

            @return A ColorFormat::Enum constant.
            */
            virtual ColorFormat::Enum colorFormat() const = 0;

            /**
            Sets the video color format.

            @param colorFormat color format

            @return TRUE if the color format is successfully set, otherwise FALSE.
            */
            virtual bool_t setColorFormat(ColorFormat::Enum colorFormat) = 0;

            /**
            Returns the video image stride.

            @return Image stride in bytes.

            <i><b>Stride</b></i> is the distance in bytes from the start of one row of pixels in memory to the start of the next row of pixels.
            It is also called a <i><b>pitch</b></i>.
            */
            virtual int32_t stride() const = 0;

            /**
            Sets the video image stride.

            @param stride Image stride in bytes.

            <i><b>Stride</b></i> is the distance in bytes from the start of one row of pixels in memory to the start of the next row of pixels.
            It is also called a <i><b>pitch</b></i>.

            @return TRUE if the stride is successfully set, otherwise FALSE.
            */
            virtual bool_t setStride(int32_t stride) = 0;

            /**
            Returns the scan type of the video frame.

            @return Scan type
            */
            virtual ScanType::Enum scanType() const = 0;

            /**
            Sets the scan type of the video frame.

            @param scanType

            @return TRUE The scan type is set, FALSE otherwise.
            */
            virtual bool_t setScanType(ScanType::Enum scanType) = 0;

            /**
            Returns whether internally the video frame is stored upside down.

            @return TRUE The frame is stored upside down (vertically flipped) but logically it should be displayed normally.
            This is typical for bitmaps. FALSE The frame is displayed as it is stored - top row is first in memory, bottom row is last.
            */
            virtual bool_t frameBottomUp() const = 0;

            /**
            Sets whether internally the video frame is stored upside down.

            @param bottomUp TRUE the frame is stored upside down, FALSE the frame is stored normally

            @return TRUE if the bottom-up flag is successfully set, otherwise FALSE.
            */
            virtual bool_t setFrameBottomUp(bool_t bottomUp) = 0;

            /**
            Copies the state of the current object to a destination object of the same type.
            The reference count of the source and the destination is not modified.

            @param destination [out] A pointer to the destination object

            @return TRUE The object state is copied, FALSE otherwise.
            */
            virtual bool_t copyTo(VideoStreamInfo * destination) const = 0;

            /**
            Creates a copy of this object.

            @return A new VideoStreamInfo.
            The caller is responsible for releasing the new object when it is not needed anymore.

            @remarks
            When cloned an immutable object becomes mutable.

            @see \ref object_management "Object Management"
            */
            virtual VideoStreamInfo* clone() const = 0;

        protected:
            /**
            This object can be destroyed only by its implementation. The caller must use Reference::release.

            @see \ref object_management "Object Management"
            */
            ~VideoStreamInfo() {}
        };


        /**
        DataStreamInfo describes a generic data stream.
        */
        /*class DataStreamInfo : public StreamInfo
        {

        };*/


        ///
        /// Metadata
        ///

        /**
        Metadata attribute names
        */
        namespace Meta
        {
            /** Comment */
            static const char Comment[] = "Comment";

            /** Involved people */
            static const char InvolvedPeople[] = "InvolvedPeople";

            /** Play counter */
            static const char PlayCounter[] = "PlayCounter";

            /** Popularity meter / rating */
            static const char Popularimeter[] = "Popularimeter";

            /** Album */
            static const char Album[] = "Album";

            /** Composer */
            static const char Composer[] = "Composer";

            /** Genre */
            static const char Genre[] = "Genre";

            /** Copyright */
            static const char Copyright[] = "Copyright";

            /** Date (YYYY-MM-DD) */
            static const char Date[] = "Date";

            /** Encoding software */
            static const char EncodedBy[] = "EncodedBy";

            /** Lyricist / text writer */
            static const char Lyricist[] = "Lyricist";

            /** File type */
            static const char FileType[] = "FileType";

            /** Time */
            static const char Time[] = "Time";

            /** Content group */
            static const char ContentGroup[] = "ContentGroup";

            /** Title */
            static const char Title[] = "Title";

            /** Subtitle */
            static const char Subtitle[] = "Subtitle";

            /** Language */
            static const char Language[] = "Language";

            /** Media type */
            static const char MediaType[] = "MediaType";

            /** Original album */
            static const char OrigAlbum[] = "OrigAlbum";

            /** Original file name */
            static const char OrigFileName[] = "OrigFileName";

            /** Original lyricist / text writer */
            static const char OrigLyricist[] = "OrigLyricist";

            /** Original artist */
            static const char OrigArtist[] = "OrigArtist";

            /** Lead artist */
            static const char LeadArtist[] = "LeadArtist";

            /** Album artist */
            static const char AlbumArtist[] = "AlbumArtist";

            /** Conductor */
            static const char Conductor[] = "Conductor";

            /** Remix / cover artist */
            static const char RemixArtist[] = "RemixArtist";

            /** Disc number is 1-based */
            static const char DiscNum[] = "DiscNum";

            /** Publisher */
            static const char Publisher[] = "Publisher";

            /** Track number is 1-based */
            static const char TrackNum[] = "TrackNum";

            /** Recording dates */
            static const char RecordingDates[] = "RecordingDates";

            /** Internet radio station name */
            static const char InternetRadioStation[] = "InternetRadioStation";

            /** Encoding software settings */
            static const char EncoderSettings[] = "EncoderSettings";

            /** Year only (YYYY) */
            static const char Year[] = "Year";

            /** User text */
            static const char UserText[] = "UserText";

            /** Unsynced lyrics */
            static const char UnsyncedLyrics[] = "UnsyncedLyrics";

            /**
            URL pointing at a web page with information such as where the album can be bought.
            */
            static const char UrlCommercialInfo[] = "UrlCommercialInfo";

            /**
            URL pointing at a web page where the terms of use and ownership of the file is described.
            */
            static const char UrlCopyright[] = "UrlCopyright";

            /**
            URL pointing at a file specific web page.
            */
            static const char UrlAudioFile[] = "UrlAudioFile";

            /**
            URL pointing at the artist's official web page.
            There may be more than one attributes with this name if the audio contains more than one performer.
            */
            static const char UrlArtist[] = "UrlArtist";

            /**
            URL pointing at the official web page for the source of the audio file, e.g. a movie.
            */
            static const char UrlAudioSource[] = "UrlAudioSource";

            /**
            URL pointing at the homepage of the Internet radio station.
            */
            static const char UrlRadioPage[] = "UrlRadioPage";

            /**
            URL pointing at a web page that will handle the process of paying for this file.
            */
            static const char UrlPayment[] = "UrlPayment";

            /**
            URL pointing at the official web page for the publisher.
            */
            static const char UrlPublisher[] = "UrlPublisher";

            /** User URL */
            static const char UrlUser[] = "UrlUser";

            /** Private comment */
            static const char Private[] = "Private";

            /** Beats per minute */
            static const char BeatsPerMinute[] = "BeatsPerMinute";
        }

        /**
        Represents the meta picture (image) type.

        @see MetaPicture::pictureType
        */
        namespace MetaPictureType
        {
            /**
            The picture type is defined as in ID3.
            */
            enum Enum
            {
                Other = 0,

                /**
                32x32 pixels 'file icon' (PNG only); only one in a ID3 tag
                */
                FileIcon = 1,

                /**
                Other icon; only one in a ID3 tag
                */
                OtherFileIcon = 2,

                /**
                Front cover
                */
                FrontCover = 3,

                /**
                Back cover
                */
                BackCover = 4,

                /**
                Leaflet page
                */
                LeafletPage = 5,

                /**
                The image is printed on the media (e.g. label side of CD)
                */
                Media = 6,

                /**
                Lead artist / Lead performer/ Soloist
                */
                LeadArtist = 7,

                /**
                Artist / Performer
                */
                Artist = 8,

                /**
                Conductor
                */
                Conductor = 9,

                /**
                Band / Orchestra
                */
                Band = 10,

                /**
                Composer
                */
                Composer = 11,

                /**
                Lyricist / Text writer
                */
                TextWriter = 12,

                /**
                Recording location
                */
                RecordingLocation = 13,

                /**
                A image taken during recording.
                */
                DuringRecording = 14,

                /**
                A image taken during performance.
                */
                DuringPerformance = 15,

                /**
                Movie / video screen capture.
                */
                VideoCapture = 16,

                /**
                A brightly coloured fish :0
                */
                BrightColoredFish = 17,

                /**
                Illustration
                */
                Illustration = 18,

                /*
                Band / Artist logotype
                */
                ArtistLogotype = 19,

                /*
                Publisher / Studio logotype
                */
                PublisherLogotype = 20,
            };
        }

        /**
        Defines MIME types used by primo codecs components
        */
        namespace MimeType
        {
            /** GIF MIME type */
            static const char Gif[] = "image/gif";

            /** JPEG MIME type */
            static const char Jpeg[] = "image/jpeg";

            /** PNG MIME type */
            static const char Png[] = "image/png";

            /** TIFF MIME type */
            static const char Tiff[] = "image/tiff";
        }


        /**
        MetaPicture describes a meta image that is part of an audio or video file.

        @see Library::createMetaPicture
        @see Metadata
        */
        class MetaPicture : public primo::Reference
        {
        public:

            /**
            Creates a deep copy of this object.

            @return	A new MetaPicture object.
            The caller is responsible for releasing the new object when it's not needed anymore.

            @return	<b>NULL</b> if the object cannot be cloned because there is not enough memory.

            @remarks
            When cloned an immutable object becomes mutable.

            @see \ref object_management "Object Management"
            */
            virtual MetaPicture* clone() const = 0;

            /**
            Returns the picture MIME type.

            @return The MIME type is an ANSI string.

            @see primo::codecs::MimeType
            */
            virtual const char* mimeType() const = 0;

            /**
            Sets the picture MIME type.

            @param mimeType [in] A pointer to ANSI string specifying the MIME type

            @return TRUE if the MIME type is successfully set, otherwise FALSE.

            @see primo::codecs::MimeType
            */
            virtual bool_t setMimeType(const char* mimeType) = 0;

            /**
            Returns the meta picture type (front cover, back cover, artist, etc.)

            @return A picture type constant defined in MetaPictureType::Enum.

            @see MetaPictureType::Enum
            */
            virtual MetaPictureType::Enum pictureType() const = 0;

            /**
            Sets the picture type (front cover, back cover, artist, etc.)

            @param [in] picType A picture type constant defined in MetaPictureType::Enum.

            @return TRUE if the picture type is successfully set, otherwise FALSE.

            @see MetaPictureType::Enum
            */
            virtual bool_t setPictureType(MetaPictureType::Enum picType) = 0;

            /**
            Returns a picture description as a free text.

            @return Unicode string. Can be <b>NULL</b> if the picture has no description.
            */
            virtual const char_t* description() const = 0;

            /**
            Sets picture description.

            @param description [in] A pointer to a Unicode string. Can be <b>NULL</b> in which case the picture description is cleared.

            @return TRUE if the description is successfully set, otherwise FALSE.
            */
            virtual bool_t setDescription(const char_t* description) = 0;

            /**
            Returns the start of the image data.
            The image is a jpeg, png or another image type.

            @return A pointer to the start of the image data.
            The number of image bytes is given by the MetaPicture::dataSize property.
            The image type (jpeg, png, etc.) is given by the MetaPicture::mimeType property.

            @see MetaPicture::mimeType
            @see MetaPicture::dataSize
            */
            virtual const uint8_t* data() const = 0;

            /**
            Returns the number of image bytes.

            @return The number of image bytes that are returned by the MetaPicture::data property.
            */
            virtual int32_t dataSize() const = 0;

            /**
            Sets image data.

            @param data [in] A pointer to the new image data.

            @param dataSize [in] The size of the new image in bytes .

            @return TRUE if the data is successfully set, FALSE if there's an error.

            The supplied <i>data</i> is copied internally and is not required to be valid after the method returns.

            @see MetaPicture::data
            @see MetaPicture::dataSize
            */
            virtual bool_t setData(const void* data, int32_t dataSize) = 0;

            /**
            Returns whether the object is immutable.
            An immmutable object cannot be modified and all modifying methods fail to produce a result.

            @return TRUE if the object is immutable, FALSE otherwise.

            @remarks
            An immutable object <b>can</b> be modified by the AVBlocks library.

            @remarks
            Object immutability spreads to all nested objects.
            Therefore it is not possible to add/set an immutable object to a mutable object.

            @remarks
            When cloned an immutable object becomes mutable.
            */
            virtual bool_t immutable() const = 0;

        protected:
            /**
            This object can be destroyed only by the library that has created it.

            @see primo::Reference::release
            */
            virtual ~MetaPicture() {};
        };


        /**
        MetaPictureList represents the meta pictures inside a Metadata object.

        This is a modifiable and indexable collection that exists only in the context of a Metadata object
        and cannot be created separately from it, nor detached.
        */
        class MetaPictureList : public primo::Reference
        {
        public:
            /**
            Returns a picture by index.

            @param index [in] A zero-based index.

            @return A pointer to the MetaPicture object specified by <i>index</i>.

            The caller is not required to release the returned object.
            */
            virtual MetaPicture* at(int32_t index) const = 0;

            /**
            Adds a new picture to the end of the list.

            @param picture [in] A pointer to a MetaPicture object.
            It is allowed to add a <b>NULL</b> item to the collection.
            Internally if the MetaPicture object is not <b>NULL</b> it is retained,
            so it is safe to release it after this method returns.

            @return TRUE if the specified item is added to the collection, otherwise FALSE.

            @see Library::createMetaPicture
            @see \ref object_management "Object Management"

            */
            virtual bool_t add(MetaPicture* picture) = 0;

            /**
            Returns the number of pictures in the list.

            @return number of items in the collection.
            */
            virtual int32_t count() const = 0;

            /**
            Removes all pictures from the collection.

            When removed from the collection each MetaPicture object is released.

            @return TRUE if the collection is cleared, otherwise FALSE.

            @see \ref object_management "Object Management"
            */
            virtual bool_t clear() = 0;

            /**
            Removes a picture by index.

            @param index [in] A zero-based index of the picture that shall be removed. When the MetaPicture object is removed it is released.

            @return TRUE if the picture is successfully removed, FALSE if there's an error.

            @see primo::Reference::release
            */
            virtual bool_t remove(int32_t index) = 0;

            /**
            Inserts a picture at the specified index in the list.

            @param itemIndex [in] The index in the collection where the new item shall be inserted.
            This must be a valid index in the collection. Indexing starts from 0.
            After the new item is inserted in the collection its index is the value of the <i>itemIndex</i> param.

            @param picture [in] A pointer to the MetaPicture object that shall be inserted.
            It is allowed to insert a <b>NULL</b> value in the collection.
            Internally the new item is retained (if not <b>NULL</b>), so it is safe to release it
            after this method returns.

            @return TRUE The item is inserted, FALSE otherwise.

            @see \ref object_management "Object Management"
            */
            virtual bool_t insert(int32_t itemIndex, MetaPicture* picture) = 0;

            /**
            Sets a new picture at the specified index.

            @param itemIndex [in] The index in the collection where the new item shall be set.
            This must be a valid index in the collection. Indexing starts from 0.

            @param picture [in] A pointer to the MetaPicture object that shall be set.
            It is allowed to set a <b>NULL</b> value to the collection.
            Internally the new item is retained (if not <b>NULL</b>), so it is safe to release it
            after this method returns. The old item is released (if not <b>NULL</b>).

            @return TRUE if the item is set, FALSE otherwise.

            @see \ref object_management "Object Management"
            */
            virtual bool_t setAt(int32_t itemIndex, MetaPicture* picture) = 0;

            /**
            Returns whether the object is immutable.
            An immmutable object cannot be modified and all modifying methods fail to produce a result.

            @return TRUE if the object is immutable, FALSE otherwise.

            @remarks
            An immutable object <b>can</b> be modified by the AVBlocks library.

            @remarks
            Object immutability spreads to all nested objects.
            Therefore it is not possible to add/set an immutable object to a mutable object.
            */
            virtual bool_t immutable() const = 0;

        protected:
            /**
            This object can be destroyed only by the library that has created it.

            @see primo::Reference::release
            */
            virtual ~MetaPictureList() {};
        };


        /**
        MetaAttribute describes a textual meta information that is part of an audio or video file.

        @see Library::createMetaAttribute
        @see Metadata
        */
        class  MetaAttribute : public primo::Reference
        {
        public:

            /**
            Creates a deep copy of this object.

            @return	A new MetaAttribute object.
            The caller is responsible for releasing the new object when it's not needed anymore.

            @return	<b>NULL</b> if the object cannot be cloned because there is not enough memory.

            @remarks
            When cloned an immutable object becomes mutable.

            @see \ref object_management "Object Management"
            */
            virtual MetaAttribute* clone() const = 0;

            /**
            Returns the value of the meta attribute.

            @return A pointer to a Unicode string.
            */
            virtual const char_t* value() const = 0;

            /**
            Sets the value a meta attribute.

            @param [in] val A pointer to a Unicode string.

            @return TRUE if the value is successfully set, otherwise FALSE.
            */
            virtual bool_t setValue(const char_t* val) = 0;

            /**
            Returns the name of the meta attribute.

            @return A pointer to an ANSI string.

            Common meta attribute names are defined in primo::codecs::Meta

            @see primo::codecs::Meta
            */
            virtual const char* name() const = 0;

            /**
            Sets the name of a meta attribute.

            @param [in] name A pointer to an ANSI string.

            Common meta attribute names are defined in primo::codecs::Meta

            @return TRUE if the name is successfully set, otherwise FALSE.

            @see primo::codecs::Meta
            */
            virtual bool_t setName(const char* name) = 0;

            /**
            Returns whether the object is immutable.
            An immmutable object cannot be modified and all modifying methods fail to produce a result.

            @return TRUE if the object is immutable, FALSE otherwise.

            @remarks
            An immutable object <b>can</b> be modified by the AVBlocks library.

            @remarks
            Object immutability spreads to all nested objects.
            Therefore it is not possible to add/set an immutable object to a mutable object.

            @remarks
            When cloned an immutable object becomes mutable.
            */
            virtual bool_t immutable() const = 0;

        protected:
            /**
            This object can be destroyed only by the library that has created it.

            @see primo::Reference::release
            */
            virtual ~MetaAttribute() {};

        };


        /**
        MetaAttributeList represents the meta attributes inside a Metadata object.

        This is a modifiable and indexable collection that exists only in the context of a Metadata object
        and cannot be created separately from it, nor detached.
        */
        class MetaAttributeList : public primo::Reference
        {
        public:

            /**
            Returns an attribute by index.

            @param index [in] A zero-based index.

            @return A pointer to the MetaAttribute object specified by <i>index</i>.

            The caller is not required to release the returned object.
            */
            virtual MetaAttribute* at(int32_t index) const = 0;

            /**
            Returns an attribute by name.

            @param name [in] A pointer to the attribute name that is searched in the list. The name is an ANSI string.

            @return A pointer to the first MetaAttribute object that matches the specified <i>name</i>.

            The caller is not required to release the returned object.

            @see primo::codecs::Meta
            */
            virtual MetaAttribute* itemByName(const char* name) const = 0;

            /**
            Adds a new attribute to the end of the list.

            @param attrib [in] A pointer to a MetaAttribute object.
            It is allowed to add a <b>NULL</b> item to the collection.
            Internally if the MetaAttribute object is not <b>NULL</b> it is retained,
            so it is safe to release it after this method returns.

            @return TRUE if the specified item is added to the collection, otherwise FALSE.

            @see Library::createMetaAttribute
            @see \ref object_management "Object Management"

            */
            virtual bool_t add(MetaAttribute* attrib) = 0;

            /**
            Returns the number of attributes in the list.

            @return number of items in the collection.
            */
            virtual int32_t count() const = 0;

            /**
            Removes all attributes from the collection.

            When removed from the collection each MetaAttribute object is released.

            @return TRUE if the collection is cleared, otherwise FALSE.

            @see \ref object_management "Object Management"
            */
            virtual bool_t clear() = 0;

            /**
            Removes an attribute by index.

            @param index [in] A zero-based index of the attribute that shall be removed. When the MetaAttribute object is removed it is released.

            @return TRUE if the attribute is successfully removed, FALSE if there's an error.
            */
            virtual bool_t remove(int32_t index) = 0;

            /**
            Inserts an attribute at the specified index in the list.

            @param itemIndex [in] The index in the collection where the new item shall be inserted.
            This must be a valid index in the collection. Indexing starts from 0.
            After the new item is inserted in the collection its index is the value of the <i>itemIndex</i> param.

            @param attrib [in] A pointer to the MetaAttribute object that shall be inserted.
            It is allowed to insert a <b>NULL</b> value in the collection.
            Internally the new item is retained (if not <b>NULL</b>), so it is safe to release it
            after this method returns.

            @return TRUE The item is inserted, FALSE otherwise.

            @see \ref object_management "Object Management"
            */
            virtual bool_t insert(int32_t itemIndex, MetaAttribute* attrib) = 0;

            /**
            Sets a new attribute at the specified index.

            @param itemIndex [in] The index in the collection where the new item shall be set.
            This must be a valid index in the collection. Indexing starts from 0.

            @param attrib [in] A pointer to the MetaAttribute object that shall be set.
            It is allowed to set a <b>NULL</b> value to the collection.
            Internally the new item is retained (if not <b>NULL</b>), so it is safe to release it
            after this method returns. The old item is released (if not <b>NULL</b>).

            @return TRUE The item is set, FALSE otherwise.

            @see \ref object_management "Object Management"
            */
            virtual bool_t setAt(int32_t itemIndex, MetaAttribute* attrib) = 0;

            /**
            Returns whether the object is immutable.
            An immmutable object cannot be modified and all modifying methods fail to produce a result.

            @return TRUE if the object is immutable, FALSE otherwise.

            @remarks
            An immutable object <b>can</b> be modified by the AVBlocks library.

            @remarks
            Object immutability spreads to all nested objects.
            Therefore it is not possible to add/set an immutable object to a mutable object.
            */
            virtual bool_t immutable() const = 0;

        protected:

            /**
            This object can be destroyed only by the library that has created it.

            @see primo::Reference::release
            */
            virtual ~MetaAttributeList() {};
        };


        /**
        Metadata describes meta information that is part of an audio or video file.
        It can hold various textual information and images.

        @see Library::createMetaAttribute
        @see Library::createMetaPicture
        @see Library::createMetadata
        */
        class Metadata : public primo::Reference
        {
        public:

            /**
            Creates a deep copy of this object.

            @return	A new Metadata object.
            The caller is responsible for releasing the new object when it is not needed anymore.

            @return	<b>NULL</b> if there is not enough memory and the object cannot be cloned.

            @remarks
            When cloned an immutable object becomes mutable.

            @see \ref object_management "Object Management"
            */
            virtual Metadata* clone() const = 0;

            /**
            Returns a modifiable collection with all the attributes in the Metadata object.

            @return A collection of MetaAttribute objects.
            The default value is an empty collection which can be modified but it cannot be replaced.

            The returned collection exists only in the context of the Metadata object.

            @see Library::createMetaAttribute
            @see MetaAttributeList
            */
            virtual MetaAttributeList *attributes() const = 0;

            /**
            Returns a modifiable collection with all the pictures in the Metadata object.

            @return A collection of MetaPicture objects.
            The default value is an empty collection which can be modified but it cannot be replaced.

            The returned collection exists only in the context of the Metadata object.

            @see Library::createMetaPicture
            @see MetaPictureList
            */
            virtual MetaPictureList* pictures() const = 0;

            /**
            Returns whether the object is immutable.
            An immmutable object cannot be modified and all modifying methods fail to produce a result.

            @return TRUE if the object is immutable, FALSE otherwise.

            @remarks
            An immutable object <b>can</b> be modified by the AVBlocks library.

            @remarks
            Object immutability spreads to all nested objects.
            Therefore it is not possible to add/set an immutable object to a mutable object.

            @remarks
            When cloned an immutable object becomes mutable.
            */
            virtual bool_t immutable() const = 0;

        protected:
            /**
            This object can be destroyed only by the library that has created it.

            @see primo::Reference::release
            */
            virtual ~Metadata() {};
        };



    }
} // primo::codecs

#endif // ___PRIMO_AV_H___

