#ifndef ___PRIMO_LICENSE_H___
#define ___PRIMO_LICENSE_H___

/** @file */

#include "PrimoReference.h"

namespace primo {

/**
	Defines common license constants and interfaces used in Primo Software libraries.
*/
namespace license {

/**
	Defines the library license status.
	When a license is set it may take some time until the license is resolved by the library.
*/
namespace LicenseStatusFlags
{
	/**
		LicenseStatusFlags::Enum
	*/
	enum Enum
	{
		/**
			The licensed state is permanent.
			It can be changed only by calling Library::setLicense.

			@remarks The Ready status does not imply that the library or any component is successfully licensed.

			@see Library::setLicense
		*/
		Ready				= 0,

		/**
			The license is being validated and the current licensed state is temporary.
		*/
		ValidationInProgress	= 1,
		
		/**
			This is a demo build that cannot be licensed.
		*/
		DemoBuild	= 2
	};
}

/**
	Provides information about the current license state of the library.

	A LicenseInfo object is obtained by calling Library::createLicenseInfo()
	The created object must be released when it is not needed anymore.

	@see Library::createLicenseInfo
*/
class LicenseInfo : public primo::Reference
{
public:

	/**
		Gets the current license status.

		@return A combination of flags defined in LicenseStatusFlags::Enum.
		
		@remarks Demo builds cannot be licensed. For demo builds, this method always returns LicenseStatusFlags::DemoBuild.
		
		@see LicenseStatusFlags
	*/
	virtual int32_t licenseStatus() = 0;

	/**
		Checks whether a product feature is licensed.     

		@param product The product Id. If <b>NULL</b> the default library product is implied.

		@param feature The feature Id. If <b>NULL</b> the default product feature is implied.

		@return TRUE if the specified product feature is licensed, FALSE otherwise.

		@remarks If a product is fully licensed then the function will always return TRUE for that product
		regardless of the <i>feature</i> parameter.
		
		If a product is licensed for a limited set of features then the function will return TRUE for that product
		only if the <i>feature</i> parameter specifies one of the licensed features or is <b>NULL</b>.
	*/
	virtual bool_t isLicensed(const char* product, const char* feature) = 0;

protected:
	~LicenseInfo() {}

};


}}

#endif