//------------------------------------------------------------------------------
// File: AVBlocks.h
// AVBlocks SDK 
// (c) Copyright 2012  Primo Software Corp. All rights reserved.      
//------------------------------------------------------------------------------

/** @file */

#ifndef ___AVBLOCKS_H___
#define ___AVBLOCKS_H___

// Platform Configuration
#include "PrimoConfig.h"

#ifndef RC_INVOKED	// defined when parsed by the Windows Resource Compiler
#include "PrimoReference.h"
#include "PrimoAV.h"
#include "PrimoStream.h"
#include "PrimoLicense.h"
#include "PrimoPP.h"
#endif


/*
These version constants should be used when the library version is needed at compile-time.
*/
#define AVBLOCKS_MAJOR_VERSION 2
#define AVBLOCKS_MINOR_VERSION 2
#define AVBLOCKS_PATCH_VERSION 0

#if defined(PLATFORM_WINDOWS)

	#define AVBLOCKS_CALL __cdecl

    #ifdef AVBLOCKS_STATIC
        // Static library 
        #define AVBLOCKS_API
    #else
        // DLL
        #ifdef AVBLOCKS_EXPORTS
            #define AVBLOCKS_API __declspec(dllexport)
        #else
            #define AVBLOCKS_API __declspec(dllimport)
        #endif
    #endif
#elif defined(PLATFORM_MACOSX) || defined(PLATFORM_LINUX)
    #define AVBLOCKS_CALL

#ifdef AVBLOCKS_STATIC
// Static library 
#define AVBLOCKS_API
#else
// DLL
#ifdef AVBLOCKS_EXPORTS
#define AVBLOCKS_API __attribute__((visibility("default")))
#else
#define AVBLOCKS_API 
#endif
#endif
#endif

#define AVBLOCKS_UNUSED_INT_RANGE 0x7FFF0000

namespace primo {

    /**
        Shared namespace for error definitions
    */
    namespace error
    {
        /**
            Common error facilities
        */
        namespace ErrorFacility
        {
            /**
            AVBlocks Error

            @see primo::avblocks::AVBlocksError::Enum
            */
            const int32_t AVBlocks = 5;

            /**
            Transcoder Error

            @see primo::avblocks::TranscoderError::Enum
            */
            const int32_t Transcoder = 6;
        }
    }

    /**
        AVBlocks Application Programming Interface
    */
    namespace avblocks {

#pragma pack(push, 1)

    /**
        Defines API usage errors. These errors are raised when the AVBlocks API is used incorrectly.
    */
    namespace 	AVBlocksError
    {
        /**
            AVBlocksError::Enum
        */
        enum Enum
        {
            /**
				= 9<br/>
				A required feature (codec/format) is not licensed and demo mode is not enabled.

                @see Transcoder::allowDemoMode
                @see Transcoder::setAllowDemoMode
            */
            UnlicensedFeature = 9,

            /**
				= 10<br/>
                Library is not initialized.

                @see primo::avblocks::Library::initialize()
            */
            LibraryNotInitialized = 10,

            /**		
                @deprecated Replaced by primo::codecs::CodecError::Failed
            */
            Failed = AVBLOCKS_UNUSED_INT_RANGE,
            
            /**		
                @deprecated Replaced by primo::codecs::CodecError::NullPointer
            */
            NullPointer,

            /**		
                @deprecated Replaced by primo::codecs::CodecError::InvalidParams
            */
            InvalidParams,
            
            /**		
                @deprecated Replaced by primo::codecs::CodecError::Alloc
            */
            Alloc,

            /**		
                @deprecated Replaced by primo::codecs::CodecError::NotImplemented
            */
            NotImplemented,

            /**		
                @deprecated Replaced by primo::codecs::CodecError::InvalidOperation
            */
            InvalidOperation, 

            /**		
                @deprecated Replaced by primo::codecs::CodecError::Unsupported
            */
            UnsupportedFormat,

            /**		
                @deprecated Replaced by primo::codecs::CodecError::MissingDependency
            */
            MissingDependency,
        };
    }

    /**
        Defines Transcoder specific errors. 

        The Transcoder may also return a CodecError or a system error.
    */
    namespace TranscoderError
    {
        /**
            TranscoderError::Enum
        */
        enum Enum
        {
            /**
                The transcoder could not build a processing chain based on its inputs and outputs.
                Either the transcoder inputs and outputs are not configured properly or
                they are not compatible with each other.

                This error is returned by Transcoder::open()
            */
            NoChain							= 1,

            /**
                The stream type of a MediaSocket is not defined.
                When a media socket describes a file, the stream type is deduced from the file extension.
                When a media socket describes a stream the stream type must be set explicitly. See StreamType::Enum.

                This error is returned by Transcoder::open()
            */
            UndefinedSocketType				= 2,

            /**
                It is not possible to convert from progressive to interlaced video.
                Also it is not possible to switch interlace type from top-first to bottom-first or vice versa.

                This error is returned by Transcoder::open()
            */
            UnsupportedInterlacedConversion = 3,

            /**
                A required encoder or decoder is not available.
            */
            NoCodec							= 4,

            /**
                It is not possible to convert from one uncompressed video format to another. See ColorFormat::Enum.
            */
            UnsupportedVideoConversion		= 5,

            /**
                Cannot accept more input through Transcoder::push(). Call Transcoder::pull() to obtain output samples.
            */
            InputFull                       = 6,

            /**
            @deprecated Replaced by primo::avblocks::TranscoderError::InputFull
            */
            BufferFull                      = InputFull,

            /**
                Cannot return output through Transcoder::pull(). Call Transcoder::push() to provide input samples.
            */
            InputNeeded						= 7,
        };
    }

    /**
        These constants are used as special values of MediaPin.ID.
        
        The Transcoder tries to connect the input/output pins depending on the values of MediaPin.ID

        A value greater than 0 specifies that the media pin participates in explicit mapping.
    */
    namespace PinConnection
    {
        /**
            PinConnection::Enum
        */
        enum Enum
        {
            /**
                The media pin does not participate in pin mapping. It is ignored by the transcoder and is never used.
            */
            Disabled	= -1,

            /**
                The media pin participates in an automatic (implicit) mapping. 
                When 2 pins are automatically connected MediaPin.ID is updated by the transcoder with a value greater than 0.
            */
            Auto		= 0,
        };
    }

    /**
        Specifies whether particular feature should be used.
    */
    namespace Use
    {
        /**
            Use::Enum
        */
        enum Enum
        {
            /**
                The feature is OFF.
            */
            Off = 0,

            /**
                The feature is ON.
            */
            On = 1,

            /**
                The feature usage is automatic.
            */
            Auto = 2
        };
    }

    /**
        Defines which hardware encoder should be used for hardware accelerated encoding.
    */
    namespace HardwareEncoder
    {
        /**
            HardwareEncoder::Enum
        */
        enum Enum
        {
            /**
                Hardware encoders are disabled.
            */
            Off		= 0,

            /**
                A hardware encoder is selected automatically depending on the available hardware and the output format.
                <p>
                    The order of preference is: Intel QuickSync, NVIDIA NVENC, AMD VCE.
                    If none of the known encoders are available then a software encoder is used.
                </p>
            */
            Auto	= 1,

            /**
                AMD Video Coding Engine (VCE)
            */
            AMD		= 2,

            /**
                Intel QuickSync Video
            */
            Intel	= 3,

            /**
                NVIDIA NVENC
            */
            Nvidia	= 4,
        };
    }

    /**
        Defines parameters that can be passed to the transcoder and to the components that are used in the transcoding process.

        A parameter can be used with one or more components like decoders, encoders, demuxers, muxers, and others.
        <p>A parameter can apply to:</p>

        <list type="bullet">
        <item>
            <description>any component</description>
        </item>
        <item>
        <description>a certain component category like encoders, decoders, demuxers, muxers, etc.</description>
        </item>
    
        <item>
        <description>a component that handles a specific format like H.264 encoder, MPEG-2 encoder, MPEG-2 muxer, etc.</description>
        </item>
    
        <item>
        <description>a specific implementation of a component, or a block like PrimoCodecs H.264 encoder</description>
        </item>
        </list>

        The value representation, range and interpretation is different for each parameter.

        @see ParameterList
        @see MediaPin::params
        @see MediaSocket::params	
    */
    namespace Param 
    {
        /**
            Specifies whether a stream will be re-encoded.

            This parameter is an integer.
            <p>
            The supported values are defined in the primo::avblocks::Use::Enum.
            The default value is \ref primo::avblocks::Use::Auto "Use::Auto".

            If this parameter is Use::Auto the re-encoding is done only when the input and the output formats are not equivalent.
            If this parameter is Use::On the re-encoding is always done even if the input and the output formats are equivalent.
            The case when this parameter is Use::Off is not implemented.
            </p>
        */
        static const char ReEncode[] = "REENCODE_STREAM";

        /**
            Specifies which hardware encoder shall be used.

            This parameter is an integer.
            <p>
            The supported values are defined in the primo::avblocks::HardwareEncoder::Enum.
            The default value is \ref primo::avblocks::HardwareEncoder::Off "HardwareEncoder::Off".
            </p>
        */
        static const char HardwareEncoder[] = "HARDWARE_ENCODER";

        /**
            Specifies the [device ID](@ref primo::codecs::HwDevice::ID) that shall be used for hardware encoding.

            This parameter is an integer.
            <p>
            The [device ID](@ref primo::codecs::HwDevice::ID) includes uniquely identified devices that support the hardware encoder
            specified by the HardwareEncoder parameter.
            The default value is 0.
            </p>

            @see primo::codecs::HwDevice
        */
        static const char HardwareEncoderDevice[] = "HARDWARE_ENCODER_DEVICE";

        /**
        Specifies which hardware API shall be used.

        This parameter is an integer.
        <p>
        The supported values are defined in the primo::codecs::HwApi::Enum.
        The default value is \ref primo::codecs::HwApi::None "HwApi::None" which lets AVBlocks 
        select the hardware API.
        
        @note
        \ref primo::codecs::HwApi::AmdOpenVideo "AMD OpenVideo" is never selected automatically by AVBlocks.
        It must be explicitly requested through this parameter.
        */
        static const char HardwareEncoderApi[] = "HARDWARE_ENCODER_API";

        /** 
			Parameters specific to video filters
		*/
        namespace Video
        {
            /**
                Parameters specific to the video overlay filter
            */
            namespace Overlay
            {
                /**
                    Specifies an alpha compositing mode - the way a foreground overlay image is combined with the background video.  
					
					This is an IntParameter. The acceptable values are defined in primo::codecs::AlphaCompositingMode::Enum. 
					
					The default value is AlphaCompositingMode::Over (Foreground over Background), in effect, a normal painting operation.
                */
                static const char Mode[] = "OVERLAY_MODE";

                /**
                Specifies the X coordinate of the location where the overlay is applied on the frame.

                This is an IntParameter.

                X increases from left to right.
                The top left point in the frame is the origin and has coordinates (0,0).
                */
                static const char LocationX[] = "OVERLAY_LOCATION_X";

                /**
                Specifies the Y coordinate of the location where the overlay is applied on the frame.

                This is an IntParameter.

                Y increases from top to bottom.
                The top left point in the frame is the origin and has coordinates (0,0).
                */
                static const char LocationY[] = "OVERLAY_LOCATION_Y";

                PRIMO_DEPRECATED("Deprecated. Please use primo::avblocks::Param::Video::Overlay::LocationX")
                /**
                    Specifies the X coordinate of the background origin where the overlay must be applied.
                    
                    This is an IntParameter. 

                    @deprecated Please use primo::avblocks::Param::Video::Overlay::LocationX instead.
                */
                static const char* BackgroundX = LocationX;


                PRIMO_DEPRECATED("Deprecated. Please use primo::avblocks::Param::Video::Overlay::LocationY.")
                /**
                    Specifies the Y	coordinate of the background origin where the overlay must be applied.

					This is an IntParameter.

                    @deprecated Please use primo::avblocks::Param::Video::Overlay::LocationY instead.
				*/
                static const char* BackgroundY = LocationX;

                /**
                    Specifies the alpha value for all pixels in the background.

					This is a FloatParameter with range from 0.0 to 1.0. The default value is 1.0.
                */
                static const char BackgroundAlpha[] = "OVERLAY_BACKGROUND_ALPHA";

                /**
                    Specifies the foreground that should be overlaid with the background.
                    
					This is a MediaBufferParameter.
                */
                static const char ForegroundBuffer[] = "OVERLAY_FOREGROUND_BUFFER";

                /**
                    Specifies the format of the foreground buffer.

					This is a VideoStreamInfoParameter.
                */
                static const char ForegroundBufferFormat[] = "OVERLAY_FOREGROUND_BUFFER_FORMAT";

                /**
                    Specifies the alpha value for all pixels in the foreground.
                    
					This is a FloatParameter with range from 0.0 to 1.0. 
					
					The default value is 1.0 which in combination with other overlay parameters results in showing the foreground and hiding the background.
                */
                static const char ForegroundAlpha[] = "OVERLAY_FOREGROUND_ALPHA";
            }

            /** 
				Parameters specific to the video frame rate converter filter 
			*/
            namespace FrameRateConverter
            {
                /**
                    Video: Specifies whether a video frame rate filter is inserted in the processing pipeline.
                    The filter converts the input frame rate to the output frame rate.

                    This parameter is an integer.
                        <p>
                            The supported values are defined in the primo::avblocks::Use::Enum.
                            The default value is \ref primo::avblocks::Use::Auto "Use::Auto".

                            If this parameter is Use::Auto the frame rate conversion is done only when the
                            input frame rate differs form the output frame rate.
                        </p>
                */
                static const char Use[] = "VIDEO_FRAME_RATE_FILTER";

                /**
                    Video: Specifies whether a video frame rate filter is allowed to drop frames
                    when a video encoder cannot keep up with the input in realtime.

                    This parameter is a boolean value expressed as an integer (1|0).<br/>
                    <p>
                    The default is False (0).

                    <ul>
                        <li>If True (1) the video frame rate filter is allowed to drop frames when a video encoder cannot keep up with the input in realtime.</li>
                        <li>If False (0) the video frame rate filter does not drop frames when a video encoder cannot keep up with the input in realtime.</li>
                    </ul>
                    </p>
                */
                static const char RealTime[] = "VIDEO_FRAME_RATE_FILTER_REAL_TIME";
            }

            /** Parameters specific to video deinterlacing */
            namespace Deinterlacing
            {
                /**
                    Video: Specifies the deinterlacing method used by the deinterlacing filter.

                    This parameter is an integer.
                    <p>
                        The supported values are defined in the primo::codecs::DeinterlacingMethod::Enum.
                        The default value is \ref primo::codecs::DeinterlacingMethod::Blend "DeinterlacingMethod::Blend".
                    </p>
                */
                static const char Method[] = "DEINTERLACING_METHOD";

                /**
                    Video: Specifies the deinterlacing threshold that is used by some deinterlacing methods.

                    The threshold is used by the \ref primo::codecs::DeinterlacingMethod::MedianThreshold "MedianThreshold" and 
                    primo::codecs::DeinterlacingMethod::CAVT "CAVT" deinterlacing methods. 

                    This parameter is an integer.
                    <p>
                        The supported values are in the range from 0 to 255.
                        Zero specifies that a default threshold should be used. Each deinterlacing method may have its own default threshold value.
                    </p>
                */
                static const char Threshold[] = "DEINTERLACING_THRESHOLD";

                /*
                    Video: Specifies whether a deinterlacing filter is inserted in the processing chain.

                    This parameter is an integer.
                        <p>
                            The supported values are defined in the primo::avblocks::Use::Enum.
                            The default value is \ref primo::avblocks::Use::Auto "Use::Auto".

                            If this parameter is Use::Auto the frame rate filter is used only when the
                            input interlace/scan type differs form the output type.
                        </p>

                        @warning
                        Not implemented yet.
                */
                static const char Use[] = "DEINTERLACING_FILTER";
            }

            /** Parameters specific to video resizing */
            namespace Resize
            {
                /**
                    Video: Specifies the interpolation method that is used for resizing video frames.
            
                    This parameter is an integer.
                    <p>
                        The supported values are defined in the primo::codecs::InterpolationgMethod::Enum.
                        The default value is \ref primo::codecs::InterpolationMethod::Cubic "InterpolationMethod::Cubic".
                    </p>
                */
                static const char InterpolationMethod[] =	"RESIZE_INTERPOLATION_METHOD";
            }

            /** 
            Parameters specific to video cropping

            @note Cropping is done <b>before</b> padding. 
            */
            namespace Crop
            {
                /**
                    Video: Specifies the area to be cropped inside the left edge of the input frame.

                    This parameter is an integer.
                */
                static const char Left[] = "CROP_LEFT";

                /**
                    Video: Specifies the area to be cropped inside the right edge of the input frame.

                    This parameter is an integer.
                */
                static const char Right[] =	"CROP_RIGHT";

                /**
                    Video: Specifies the area to be cropped inside the top edge of the input frame.

                    This parameter is an integer.
                */
                static const char Top[] = "CROP_TOP";

                /**
                    Video: Specifies the area to be cropped inside the bottom edge of the input frame.

                    This parameter is an integer.
                */
                static const char Bottom[] = "CROP_BOTTOM";
            }

            /** 
            Parameters specific to video padding

            @note The padding is applied inside the output frame at the expense of the usable frame area.
            <p>
            Example: 
            Transcoder input settings: 
            Input frame resolution: 640x480, output frame resolution: not specified, padding as follows: top(60), bottom(60)
            <br />
            Transcoder result:
            The output frame is 640x480 because the input resolution is kept implicitly.
            The padding is applied inside the 640x480 frame and the usable frame area becomes 640x360.
            </p>

            @note Padding is done <b>after</b> cropping. 
            */
            namespace Pad
            {
                /**
                    Video: Specifies the area to be padded inside the left edge of the output frame.

                    The parameter is an integer.
                */
                static const char Left[] = "PAD_LEFT";

                /**
                    Video: Specifies the area to be padded inside the right edge of the output frame. 
            
                    The parameter is an integer.
                */
                static const char Right[] = "PAD_RIGHT";

                /**
                    Video: Specifies the area to be padded inside the top edge of the output frame. 
            
                    The parameter is an integer.
                */
                static const char Top[] = "PAD_TOP";

                /**
                    Video: Specifies the area to be padded inside the bottom edge of the output frame. 
            
                    The parameter is an integer.
                */
                static const char Bottom[] = "PAD_BOTTOM";

                /**
                    Video: Specifies the padding color if any padding is set.

                    The parameter is an integer.
                    The color format is ARGB32: bits 0-7 is blue, bits 8-15 is green, bits 16-23 is red and bits 24-31 is the alpha channel.
            
                */
                static const char Color[] = "PAD_COLOR";
            }
        }

        /** Parameters specific to decoders */
        namespace Decoder
        {
            /**
                Decoder configuration data.

                This parameter is a MediaBufferParameter.
            */
            static const char Config[] = "DECODER_CONFIG";

            /** Parameters specific to video decoders */
            namespace Video
            {
                /** Parameters specific to H.264 decoders */
                namespace H264
                {
                    /** 
                        Parameters the are part of the Video Usability Information in the H.264 stream.
                        These parameters can aid decoding but a conformant decoder is not required to process them.
                    */
                    namespace VUI
                    {
                        /**
                        H264 Video: Specifies the requested size of the Decoded Picture Buffer.
                        This parameter can be used to achieve low latency decoding.

                        This parameter is an integer (0-1).<br/>
                        <p>
                            A typical usage of this parameter is to set it to 1 or 0.
                            In this case the number of decoding threads is implicitly limited to 1.
                            A negative value is ignored by the decoder.
                        </p>

                        @note Zero latency is possible only with progressive streams with no B-frames.
                        The decoder attempts to detect cases when this parameter can break decoding and may adjust it.
                        However it's still possible that certain streams may not decode properly when this setting is too low.
                        */
                        static const char MaxDecFrameBuffering[] =	"DECODER_H264_MAX_DEC_FRAME_BUFFERING";
                    }
                }

                /**
                Video: Specifies whether to conceal bitstream defects and errors.

                This parameter is a boolean value expressed as an integer (1|0).<br/>
                <p>
                The default is True (1).

                <ul>
                <li>If True (1) non-fatal bitstream errors are concealed.</li>
                <li>If False (0) any bitstream error is reported.</li>
                </ul>
                </p>

                @remarks The ConcealDefects parameter is currently valid only for MPEG4-Visual and H.264 video decoding. 
                When the video decoder encounters a corrupt frame, the decoder substitutes the bad macroblocks with good macroblocks from the previous frame.
                When a macroblock is partially valid, for example the luma coefficients are fine, but the chroma coefficients are corrupt, the error recovery is not perfect.
                */
                static const char ConcealDefects[] = "DECODER_VIDEO_CONCEAL_DEFECTS";
            }
        }

        /** Parameters specific to encoders */
        namespace Encoder
        {
            /** Parameters specific to audio encoders */
            namespace Audio
            {
                /** Parameters specific to MPEG1 audio encoders */
                namespace MPEG1
                {
                    /**
                        MPEG Audio: Forces MPEG audio to version 1.
            
                        This parameter is a boolean value expressed as an integer (1|0).<br/>
                        <p>
                            The default value is False (0).
                        </p>
                    */
                    static const char ForceV1[] =	"ENCODER_MPEGAUDIO_FORCE_V1";
                
                    /**
                    MPEG Audio: Specifies the stereo mode.

                    <p>
                    The supported values are defined in the primo::codecs::StereoMode::Enum.
                    The default value is \ref primo::codecs::StereoMode::LR "StereoMode::LR".
                    </p>

                    @see primo::codecs::StereoMode::Enum
                    */
                    static const char StereoMode[] = "ENCODER_MPEGAUDIO_STEREOMODE";
                }

                /** Parameters specific to AAC audio encoders */
                namespace AAC
                {
                    /**
                    AAC Audio: Specifies the stereo mode.

                    <p>
                    The supported values are defined in the primo::codecs::StereoMode::Enum.
                    The default value is \ref primo::codecs::StereoMode::LR "StereoMode::LR".
                    </p>

                    @see primo::codecs::StereoMode::Enum
                    */
                    static const char StereoMode[] = "ENCODER_AAC_STEREOMODE";
                }

                /** Parameters specific to Vorbis audio encoders */
                namespace Vorbis
                {
                    /**
                        Vorbis Audio: Specifies the quality of the compressed audio.

                        This parameter is a floating point number.<br/> 
                        <p>
                        Supported values range: from -0.1 to 1.0. The default value is 0.4.
                        </p>
                    */
                    static const char Quality[] =	"ENCODER_VORBIS_QUALITY";
                }

                /** Parameters specific to WMA audio encoders */
                namespace WMA
                {
                    /**
                        Windows Media Audio: Specifies the quality of the compressed audio.

                        This parameter is an integer.<br/> 
                        <p>
                        Supported values range: from 1 to 100. The default value is 50.
                        </p>
                        
                        @note
                        The WMA quality parameter is used only when the encoded bitrate mode is
                        \link primo::codecs::BitrateMode::VBR BitrateMode::VBR\endlink, otherwise it is ignored.
                    */
                    static const char Quality[] =	"ENCODER_WMA_QUALITY";
                }
            }

            /** Parameters specific to video encoders */
            namespace Video
            {
                /** Parameters specific to MPEG-1 and MPEG-2 video encoders */
                namespace MPEG2
                {
                    /**
                        MPEG-1 and MPEG-2 Video: Size of the Group of Pictures (GOP, N).
            
                        This parameter is an integer.<br/> 
                        <p>
                            The integer value is the distance between two I-pictures (full images).
                            The minimum value is 1 which means the stream will contain only I-pictures.
                            The maximum value for MPEG-1 is 132.
                        </p>
                        <p>The default value is 15.</p>

                        @see MPEG2::IPDistance
                    */
                    static const char GOPSize[] = "ENCODER_MPEG2VIDEO_GOPSIZE";

                    /**
                        MPEG-1 and MPEG-2 Video: Distance between successive reference pictures (GOP, M).
            
                        This parameter is an integer.<br/>
                        <p>
                            The integer value is the distance between 2 successive reference pictures: 
                            I-I, I-P, P-I or P-P.
                            The minimum value is 1 which means that the stream will contain I and P pictures but no B-pictures.
                        </p>
                        <p>
                            If IPDistance is equal to GOPSize and is greater than 1
                            then the stream will contain only I and B picutres, but no P-pictures.
                        </p>
                        <p>The default value is 3.</p>

                        @see MPEG2::GOPSize
                    */
                    static const char IPDistance[] = "ENCODER_MPEG2VIDEO_IPDISTANCE";

                    /**
                        MPEG-1 and MPEG-2 Video: Size of the internal VBV buffer in units of 16384 bits.
            
                        This parameter is an integer.<br/>
                        <p>
                            The default value is 20 for MPEG-1 and 112 for MPEG-2.
                        </p>
                        <p>
                            A value of 0 means to use the encoder default setting.
                        </p>
                    */
                    static const char VBVBufferSize[] = "ENCODER_MPEG2VIDEO_VBV_BUFFER_SIZE";
                }

                /** Parameters specific to AVC/H.264 encoders */
                namespace H264
                {
                    /**
                        H.264 Video: Encoding profile.

                        This parameter is an integer.<br/>

                        <p>
                            The supported values are defined in the primo::codecs::H264Profile::Enum.
                            The default value is \ref primo::codecs::H264Profile::Main "H264Profile::Main".
                        </p>

                        <p>Sets the value of the profile_idc syntax element.</p>

                        @note
                        Hardware encoders support: Intel QuickSync, AMD VCE, NVIDIA NVENC.

                        @see primo::codecs::H264Profile::Enum
                        @see H264::EntropyCodingMode
                        @see H264::NumBFrames
                        @see H264::Transform8x8
                    */
                    static const char Profile[]						=	"ENCODER_H264_PROFILE_IDC";

                    /**
                        H.264 Video: Entropy coding mode.

                        This parameter is an integer.<br/>
                        <p>
                            The supported values are defined in the primo::codecs::H264EntropyCodingMode::Enum.
                            The default value is \ref primo::codecs::H264EntropyCodingMode::CABAC "H264EntropyCodingMode::CABAC".
                        </p>

                        <p>CABAC compresses data more efficiently (10-20% typically) than CAVLC but requires considerably more processing to decode.</p>
                        <p>CABAC is supported at Main, High, High10, High422 and High444 profiles. CABAC is not supported at Baseline profile.</p>

                        @note
                        Hardware encoders support: Intel QuickSync, AMD VCE, NVIDIA NVENC.

                        @see primo::codecs::H264EntropyCodingMode::Enum
                        @see H264::Profile
                        @see primo::codecs::H264Profile::Enum

                    */
                    static const char EntropyCodingMode[]			=	"ENCODER_H264_ENTROPY_CODING_MODE";

                    /**
                        H.264 Video: Number of B-frames between consequent I-frames or P-frames.

                        This parameter is an integer.<br/>
                        <p>
                            The integer value must be a number that is greater than or equal to zero.
                        </p>
                        <p>The default value is 0.</p>

                        <p>B-frames are supported at Main, High, High10, High422 and High444 profiles. B frames are not supported at Baseline profile.</p>

                        <p>
                        H.264 stream without B frames looks like: IPPPPPPP...PI...
                        H.264 stream with B frames (NumBFrames 2) looks like: IBBPBBPBBP...PI
                        
                        The main advantage of the usage of B frames is coding efficiency. In most cases, B frames will result in less bits being coded overall.
                        B-frames can use both previous and forward frames for data reference to get the highest amount of data compression.</p>

                        @note 
                        Hardware encoders support: Intel QuickSync, NVIDIA NVENC, AMD VCE.

                        @note
                        AMD VCE supports B frames with the following restrictions:
                        <ul>
                        <li>VCE2 and up must be used. VCE1 has no support for B frames and ignores this parameter.</li>
                        <li>Maximum 3 B frames are accepted, 4+ is treated as an invalid parameter.</li>
                        <li>AMD OpenVideo (\ref primo::codecs::HwApi::AmdOpenVideo "HwApi::AmdOpenVideo")
                        does not support B frames and ignores this parameter.
                        </li>
                        </ul>

                        @see H264::Profile
                        @see primo::codecs::H264Profile::Enum
                    */
                    static const char NumBFrames[]					=	"ENCODER_H264_NUM_B_FRAMES";

                    /**
                        H.264 Video: Allows B-pictures to be used as reference pictures.

                        This parameter is a boolean value expressed as an integer (1|0).<br/>
                        <p>The default value is 0 (False).</p>

                        @note
                        Hardware encoders support: Intel QuickSync.

                        @see H264::NumBFrames
                    */
                    static const char TreatBFramesAsReference[]	=	"ENCODER_H264_TREAT_B_FRAMES_AS_REFERENCE";

                    /**
                        H.264 Video: Maximum number of reference pictures.
                            Sets the maximum number of references stored in the Decoded Picture Buffer (DPB) for motion estimation and compensation.
                            Essentially sets the syntax element num_ref_frames in the sequence parameter sets.

                        This parameter is an integer.<br/>
                        <p>
                            The integer value must be consistent with profile/level parameters for a given frame size, 
                            since they determine reference picture and buffer constraints for a decoder.<br/> 
                            If P-pictures are allowed, the number should be greater than 0.<br/>
                            If B-pictures are allowed, the number should be greater than 1.<br/>
                            Note: If NumRefFrames is 0 it is automatically determined by the encoder.
                        </p>
                        <p>The default value is 3. Supported values: from 0 to 16.</p>

                        <p>This setting controls how many frames can be referenced by P- and B-Frames.
                            Higher values will usually result in a more efficient compression, 
                                but will also require more time for encoding. </p>

                        @note
                        Hardware encoders support: Intel QuickSync, AMD VCE (only OpenVideo), NVIDIA NVENC.
                    */
                    static const char NumRefFrames[]				=	"ENCODER_H264_NUM_REF_FRAMES";

                    /**
                        H.264 Video: Number of slices per frame/field.

                        This parameter is an integer.<br/>
                        <p>
                            The integer value cannot exceed the number of macro blocks per frame/field.<br/>
                            A frame/field is divided into equal parts except for the last one that can be shorter.
                        </p>
                        <p>The default value is 0 (auto).</p>

                        @note
                        Hardware encoders support: Intel QuickSync, NVIDIA NVENC.
                    */
                    static const char NumSlices[]					=	"ENCODER_H264_NUM_SLICES";

                    /**
                        H.264 Video: Computation level.
                        Set the value of the level_idc syntax element.

                        This parameter is an integer.<br/>
                        <p>
                            A level designated as <i>x.y</i> is expressed as an <i>xy</i> integer value.
                            For example level 1 is expressed as 10, level 1.1 as 11 and so on.
                            The level must be in accordance with the H.264 standard.
                            It must correspond to the number of reference frames and frame size.
                            A value of 0 specifies automatic level computation.
                        </p>
                        <p>The default value is 0 (auto).</p>

                        <p>Note: Similar with the Profile, Level specifies the capabilities a decoder must fulfill to decode a bitstream of a certain level.
                          Most level restrictions are driven by memory restrictions and set restrictions such as resolution supported, maximum number of references, frame rate etc.</p>

                        @note
                        Hardware encoders support: Intel QuickSync, AMD VCE, NVIDIA NVENC.
                    */
                    static const char Level[]						=	"ENCODER_H264_LEVEL_IDC";

                    /**
                        H.264 Video: De-blocking filter mode.

                        This parameter is an integer.<br/>
                        <p>
                            The supported values are defined in the primo::codecs::H264DeblockingFilter::Enum.
                            The default value is \ref primo::codecs::H264DeblockingFilter::All "H264DeblockingFilter::All".
                        </p>

                        <p>The in-loop deblocking filter helps prevent the blocking artifacts common to other DCT-based image compression techniques, 
                           resulting in better visual appearance and compression efficiency</p>

                        @note
                        Hardware encoders support: Intel QuickSync, AMD VCE, NVIDIA NVENC.

                        @see primo::codecs::H264DeblockingFilter::Enum
                    */
                    static const char DeblockingFilter[]			=	"ENCODER_H264_DEBLOCKING_FILTER_IDC";

        
                    /**
                        H.264 Video: De-blocking filter strength.

                        This parameter is an integer.<br/>
                        <p>
                            Supported values range: from -6 to 6.
                            Negative values give a more sharp video, but they also increases the danger of visible block artifacts.
                            In contrast positive values result in a smoother video, but they will also remove more details.
                            The default value is 0.
                        </p>

                        @note
                        Hardware encoders support: AMD VCE.

                        @see H264::DeblockingFilter
                        @see H264::DeblockingFilterBeta
                    */
                    static const char DeblockingFilterAlpha[]		=	"ENCODER_H264_DEBLOCKING_FILTER_ALPHA";

                    /**
                        H.264 Video: De-blocking filter threshold.

                        This parameter is an integer.<br/>
                        <p>
                            This parameter controls the threshold for block detection.
                        </p>
                        <p>
                            Supported values range: from -6 to 6.<br />
                            Negative values "save" more details, but more blocks might slip through (especially in flat areas).
                            In contrast positive values will remove more details and catch more blocks.
                            The default value is 0 and usually it's enough to detect all blocks.
                        </p>

                        @note
                        Hardware encoders support: AMD VCE.

                        @see H264::DeblockingFilter
                        @see H264::DeblockingFilterAlpha
                    */
                    static const char DeblockingFilterBeta[]		=	"ENCODER_H264_DEBLOCKING_FILTER_BETA";

                    /**
                        H.264 Video: Transform 8x8 mode.

                        This parameter is a boolean value expressed as an integer (1|0).<br/>
                        <p>Supported modes are:</p>
                        <p>
                            0 - Disabled. Only 4x4 transforms are used.<br/>
                            1 - Enabled. Allows the additional use of 8x8 transform.<br/>
                        </p>

                        The default is Disabled (0).

                        <p>Transform 8x8 improves the visual quality at a minor speed cost.</p>
                        <p>Transform 8x8 is supported at High, High10, High422 and High444. 8x8 transform is not supported at Main and Baseline profiles.</p>

                        @note
                        Hardware encoders support: None.

                        @see H264::Profile
                    */
                    static const char Transform8x8[]					=	"ENCODER_H264_TRANSFORM8X8";

                    /**
                        H.264 Video: Direct prediction mode.
                        Sets the value of the syntax element direct_spatial_mv_pred_flag.

                        This parameter is an integer.<br/>
                        <p>
                            The supported values are defined in the primo::codecs::H264DirectPredMode::Enum.
                            The default value is \ref primo::codecs::H264DirectPredMode::Temporal "H264DirectPredMode::Temporal".
                        </p>

                        @note
                        Hardware encoders support: NVIDIA NVENC.
                    */
                    static const char DirectPredMode[]				=	"ENCODER_H264_DIRECT_PRED_MODE";

                    /**
                        H.264 Video: Quality/Speed trade off.

                        This parameter is an integer.<br/>
                        <p>
                            Supported values range from 0 to 6 where 0 means max speed and 6 means max quality.
                            The default value is 0.
                        </p>

                        @note
                        Hardware encoders support: Intel QuickSync.
                    */
                    static const char QualitySpeed[]				=	"ENCODER_H264_QUALITY_SPEED";

                    /**
                        H.264 Video: Specifies how far motion estimation should be done in a horizontal direction.

                        This parameter is an integer.<br/>
                        <p>The default value is 8.</p>

                        @note
                        Hardware encoders support: AMD VCE.
                    */
                    static const char MERangeSearchX[]			=	"ENCODER_H264_ME_RANGE_SEARCH_X";

                    /**
                        H.264 Video: Specifies how far motion estimation should be done in a vertical direction.

                        This parameter is an integer.<br/>
                        <p>The default value is 8.</p>

                        @note
                        Hardware encoders support: AMD VCE.
                    */
                    static const char MERangeSearchY[]			=	"ENCODER_H264_ME_RANGE_SEARCH_Y";

                    /**
                        H.264 Video: Specifies the block size for which motion estimation should be done.

                        This parameter is an integer.<br/>
                        <p>
                            The supported values are defined in the primo::codecs::H264MeSplitMode::Enum.
                            The default value is \ref primo::codecs::H264MeSplitMode::Only16x16 "H264MeSplitMode::Only16x16".
                        </p>

                        @note
                        Hardware encoders support: None.
                    */
                    static const char MESplitMode[]				=	"ENCODER_H264_ME_SPLIT_MODE";

                    /**
                        H.264 Video: Motion estimation method.

                        This parameter is an integer.<br/>
                        <p>
                            The supported values are defined in the primo::codecs::H264MeMethod::Enum.
                            The default value is \ref primo::codecs::H264MeMethod::Log "H264MeMethod::Log".
                        </p>

                        @note
                        Hardware encoders support: None.
                    */
                    static const char MEMethod[]					=	"ENCODER_H264_ME_METHOD";

                    /**
                        H.264 Video: Picture coding type.

                        This parameter is an integer.<br/>
                        <p>
                            The supported values are defined in the primo::codecs::H264PicCodingType::Enum.
                            The default value is \ref primo::codecs::H264PicCodingType::Frames "H264PicCodingType::Frames".
                        </p>

                        @note
                        Hardware encoders support: None.
                    */
                    static const char PictureCodingType[]			=	"ENCODER_H264_PIC_CODING_TYPE";

                    /**
                        H.264 Video: Rate control method.

                        This parameter is an integer.<br/>
                        <p>
                            The supported values are defined in the primo::codecs::H264RateControlMethod::Enum.
                            The default value is \ref primo::codecs::H264RateControlMethod::ABR "H264RateControlMethod::ABR".
                        </p>

                        @note
                        Hardware encoders support: Intel QuickSync, AMD VCE, NVIDIA NVENC.

                        @see H264::RateControlQuantI
                        @see H264::RateControlQuantP
                        @see H264::RateControlQuantB
                    */
                    static const char RateControlMethod[]			=	"ENCODER_H264_RATE_CONTROL_METHOD";

                    /**
                        H.264 Video: Rate control quantizer for I slices.
                            Used only when RateControlMethod is set to Constant Quantization.

                        This parameter is an integer.<br/>
                        <p>Supported values range: from 0 to 51. The default value is 20.</p>

                        @note
                        Hardware encoders support: Intel QuickSync, AMD VCE, NVIDIA NVENC.

                        @see H264::RateControlMethod
                    */
                    static const char RateControlQuantI[]			=	"ENCODER_H264_RATE_CONTROL_QUANT_I";

                    /**
                        H.264 Video: Rate control quantizer for P slices.
                            Used only when RateControlMethod is set to Constant Quantization.

                        This parameter is an integer.<br/>
                        <p>Supported values range: from 0 to 51. The default value is 20.</p>

                        @note
                        Hardware encoders support: Intel QuickSync, AMD VCE, NVIDIA NVENC.

                        @see H264::RateControlMethod
                    */
                    static const char RateControlQuantP[]			=	"ENCODER_H264_RATE_CONTROL_QUANT_P";

                    /**
                        H.264 Video: Rate control quantizer for B slices.
                            Used only when RateControlMethod is set to Constant Quantization.

                        This parameter is an integer.<br/>
                        <p>Supported values range: from 0 to 51. The default value is 20.</p>

                        @note
                        Hardware encoders support: Intel QuickSync, AMD VCE, NVIDIA NVENC.

                        @see H264::RateControlMethod
                    */
                    static const char RateControlQuantB[]			=	"ENCODER_H264_RATE_CONTROL_QUANT_B";

                    /**
                        H.264 Video: Distance between two sequential I-pictures.

                        This parameter is an integer.<br/>
                        <p>
                            The integer value must be greater than or equal to 0. Both 0 and 1 mean that every picture is coded as I-picture.
                        </p>
                        <p>The default value is 90.</p>

                        @note
                        Hardware encoders support: Intel QuickSync, AMD VCE, NVIDIA NVENC.
                    */
                    static const char KeyFrameInterval[]			=	"ENCODER_H264_KEY_FRAME_INTERVAL";

                    /**
                        H.264 Video: Distance between two sequential IDR-pictures in terms of I pictures. IDR stands for Instantaneous Decoding Refresh.

                        This parameter is an integer.<br/>
                        <p>The integer value must be greater or equal to 0.</p>
                        <p>
                            For example if this parameter is 10 then every tenth I-picture is also an IDR-picture.
                        </p> 
                        <p>
                            An IDR picture is a special kind of I frame used in an H.264 stream.
                            Pictures following an IDR-picture may not refer back to pictures preceding the IDR-picture.
                            IDR-pictures can be used to create H.264 streams which are more easily edited.
                        </p>
                        <p>The default value is 1.</p>

                        @note
                        Hardware encoders support: Intel QuickSync, AMD VCE.
                    */
                    static const char KeyFrameIDRInterval[]		=	"ENCODER_H264_KEY_FRAME_IDR_INTERVAL";

                    /**
                        H.264 Video: Use access unit delimiters.

                        This parameter is a boolean value expressed as an integer (1|0).<br/>
                        <p>
                        The default value is False (0).
                        </p>

                        @note
                        Hardware encoders support: Intel QuickSync, NVIDIA NVENC.
                    */
                    static const char AccessUnitDelimiters[] = "ENCODER_H264_ACCESS_UNIT_DELIMITERS";

                    /**
                        H.264 Video: Write timing info with a fixed frame rate in the bitstream.

                        This parameter is a boolean value expressed as an integer (1|0).<br/>
                        <p>
                        The default value is True (1).
                        </p>
                        <p>
                        The timing info is written in the bitstream only when this parameter is True and a positive frame rate is specified.
                        </p>

                        @note
                        Hardware encoders support: Intel QuickSync.
                    */
                    static const char FixedFramerate[] = "ENCODER_H264_FIXED_FRAMERATE";
                }

                /** Parameters specific to HEVC/H.265 encoders */
                namespace H265
                {
                    /**
                    H.265 encoding profile. <br/>

                    This parameter is an integer. <br/>

                    <p>
                    The supported values are defined in the primo::codecs::H265Profile.
                    The default value is [H265Profile::Main](@ref primo::codecs::H265Profile::Enum).
                    </p>
                    <p>Supported profiles: Main, Main10.</p>
                    <p>Sets the value of the profile_idc syntax element.</p>

                    @see primo::codecs::H265Profile::Enum
                    @see H265::Level
                    @see H265::Tier
                    */
                    static const char Profile[] = "ENCODER_H265_PROFILE_IDC";

                    /**
                    H.265 computation level.

                    This parameter is an integer. <br/>

					Sets the value of the level_idc syntax element. <br/>

                    <p>The default value is [H265Level::None](@ref primo::codecs::H265Level::None) (level is selected automatically).</p>

                    <p>Note: Similar with the Profile, Level specifies the capabilities a decoder must fulfill to decode a bitstream of a certain level.
                    Most level restrictions are driven by memory restrictions and set restrictions such as resolution supported, maximum number of references, frame rate etc.</p>

                    @see primo::codecs::H265Level
                    @see H265::Profile
                    @see H265::Tier
                    */
                    static const char Level[] = "ENCODER_H265_LEVEL_IDC";
                    
                    /**
                    H.265 bitrate tier.

                    <p>
                    The supported values are defined in the primo::codecs::H265Tier.
                    The default value is [H265Tier::Main](primo::codecs::H265Tier::Main).
                    </p>

                    <p>
                    High tier specifies higher bitrate than Main tier.
                    The High tier is not supported under level 4.
                    </p>

                    @see primo::codecs::H265Tier
                    @see H265::Profile
                    @see H265::Level
                    */
                    static const char Tier[] = "ENCODER_H265_TIER";

					/**
					H.265 `Group of Pictures (GOP)` size.

					This parameter is an integer.<br/>
					<p>
					The integer value is the distance between two I-pictures.
					If 1 then stream will contain only I-pictures.
					If 0 then there will be no I-pictures in the stream after the first picture.
					</p>
					<p>The default value is 0.</p>

					@see H265::Profile
					@see H265::Level
					@see H265::Tier
					*/
					static const char GOPLength[] = "ENCODER_H265_GOP_LENGTH";
				}

				/** Parameters specific to VP8 video encoder */
				namespace VP8
				{
					/**
					<b>%VP8 %Video:</b> Max keyframe distance.

					This parameter is an integer. <br />
					<p>
					This value, expressed as a number of frames, forces the encoder to code
					a keyframe if one has not been coded in the last KeyFrameMaxDistance frames.
					</p><p>
					A value of 0 implies all frames will be keyframes. Set keyframe KeyFrameMinDistance
					equal to KeyFrameMaxDistance for a fixed interval.
					</p>
					<p>The default value is 100.</p>
					*/
					static const char KeyFrameMaxDistance[] = "ENCODER_VP8_KEY_FRAME_MAX_DISTANCE";

					/**
					<b>%VP8 %Video:</b> Min keyframe distance.

					This parameter is an integer. <br />
					<p>
					This value, expressed as a number of frames, prevents the encoder from
					placing a keyframe nearer than KeyFrameMinDistance to the previous keyframe.
					</p><p>
					At least KeyFrameMinDistance frames non-keyframes will be coded before the next
					keyframe. Set KeyFrameMinDistance equal to KeyFrameMaxDistance for a fixed interval.
					</p>
					<p>The default value is 0.</p>
					*/
					static const char KeyFrameMinDistance[] = "ENCODER_VP8_KEY_FRAME_MIN_DISTANCE";
				}
            }
        }
        
        /** Parameters specific to muxers */
        namespace Muxer
        {
            /** Parameters specific to MP4 muxers */
            namespace MP4
            {
                /**
                    Run a second pass moving the moov atom before the mdat atom.

                    Unless specified otherwise, the moov atom is normally stored at the end of the file.
                    If the planned delivery method is progressive download or streaming, the moov atom will
                    have to be moved to the beginning of the file.

                    This parameter is a boolean value expressed as an integer (1|0).<br/>
                    <p>
                        The default is False (0).

                        <ul>
                        <li>If True (1) a second pass will reorder the atoms in the file. The atoms will be in the order: moov, mdat.</li>
                        <li>If False (0) the atoms in the file will be in the order: mdat, moov.</li>
                        </ul>
                    </p>
                */
                static const char FastStart[] = "MUXER_MP4_FAST_START";

                /**
                Specifies whether a temp file should be used during the second pass.

                This parameter is a boolean value expressed as an integer (1|0).<br/>
                    <p>
                        The default is False (0).

                        <ul>
                        <li>If True (1) a temp file will be used to reorder the atoms in the mp4 file.</li>
                        <li>If False (0) the atoms reordering will be done in-place.</li>
                        </ul>
                    </p>
                */
                static const char FastStartUseTempFile[] = "MUXER_MP4_FAST_START_USE_TEMP_FILE";

                /**
                    Specifies the temp file directory. If this parameter is not set then the system temp path will be used.

                    This parameter is a StringParameter (const char_t*).<br/>
                */
                static const char FastStartTempFileDirectory[] = "MUXER_MP4_FAST_START_TEMP_FILE_DIRECTORY";
            }
        }


        /** Parameters specific to demuxers. */
        namespace Demuxer
        {
            /** Parameters specific to the Mpeg demuxer. */
            namespace Mpeg
            {
                /**
                    Specifies the Program Number for which to deliver MPEG TS packets. Only packets that belong to the Program with the specified Program Number will be delivered.<br/>
                    The Program Number is the field that identifies the program in the Program Association Table (PAT) of a MPEG-2 TS stream.<br/>
                    This parameter is an integer.
                */
                static const char ProgramNumber[] = "DEMUXER_MPEG_PROGRAM_NUMBER";

                /**
                    Specifies the Packet ID (PID) of the MPEG TS packets to be delivered. Only packets with the specified PID will be delivered.<br/>
                    The Packet ID is used to identify Program Specific Info packets, such as PAT, CAT, PMT packets, or elementary stream packets in MPEG-2 TS stream.<br/>
                    This parameter is an integer.
                */
                static const char PacketId[] = "DEMUXER_MPEG_PACKET_ID";
            }
        }
    };


    /**
        The ParamType represents the type of the data stored in a Parameter.

        Depending on this constant a Parameter interface can be cast to child interfaces such as
        IntParameter, StringParameter, etc.

        @see Parameter::type
    */	
    namespace ParamType
    {
        /**
            ParamType::Enum
        */
        enum Enum
        {
            /**
                The parameter represents a 64-bit floating point number (double precision).
            */
            Float	= 1,

            /**
                The parameter represents a MediaBuffer object and can be cast to MediaBufferParameter.
            */
            MediaBuffer	= 3,

            /**
                The parameter represents a VideoStreamInfo object and can be cast to VideoStreamInfo.
            */
            VideoStreamInfo	= 4,

            /**
                The parameter represents a generic value which is stored in a char_t string.
                The parameter name constant documents how this value should be interpreted.
                
                The parameter can be cast to StringParameter.
            */
            String	= 5,

            /**
                The parameter represents a 64-bit Integer.
            */
            Int	= 6,
        };
    }

    /**
        Parameter is the base class for named parameters passed to various classes in AVBlocks.
        
        The parameter name is an ANSI string.
        The parameter content can be a simple value or a complex structure. This is determined by the parameter type.
    */
    class Parameter : public primo::Reference
    {
    public:
        /**
            Returns the parameter name. 
            
            @return A pointer to zero terminated ANSI string. 
            The returned pointer cannot be <b>NULL</b> but the string can be empty.
        */
        virtual const char* name() const = 0;

        /**
            Sets the parameter name.

            @param name [in] A pointer to zero terminated ANSI string.
            If <b>NULL</b> it is interpreted as if an empty string is provided.
            The string pointed by <i>name</i> is copied internally and is not required to be valid after the method returns. 
        */
        virtual bool_t setName(const char* name) = 0;

        /**
            Returns the parameter type. 
            
            @return A ParamType constant. 
        */
        virtual ParamType::Enum type() const = 0;

        /**
            Creates a deep copy of the underlying StringParameter, IntParameter, FloatParameter, or MediaBufferParameter object.

            @return	A new Parameter object.
            The caller is responsible for releasing the new object when it is not needed anymore.

            @return	<b>NULL</b> when there is not enough memory to clone the object.

            @remarks
            this method uses primo::codecs::MediaBuffer::clone, to clone the MediaBufferParameter::buffer object.

            @see \ref object_management "Object Management"
        */
        virtual Parameter* clone() const = 0;

    protected:
        /**
            This object can be destroyed only by the AVBlocks library.
        */
        ~Parameter() {}
    };

    /**
        The parameter represents a MediaBuffer object.
    */
    class MediaBufferParameter : public Parameter
    {
    public:
        /**
            Returns the buffer instance. 
            
            @return A pointer to MediaBuffer object. 
            The returned pointer can be <b>NULL</b>.
        */
        virtual primo::codecs::MediaBuffer* buffer() const = 0;

        /**
            Sets the buffer.

            @param buffer [in] A pointer to MediaBuffer that will be stored in the parameter.
            It is allowed to set a <b>NULL</b> object.
            Internally the MediaBuffer specified by <i>buffer</i> is retained (if not <b>NULL</b>), so it is safe to release it 
            after this method returns. The previous MediaBuffer is released (if not <b>NULL</b>).
        */
        virtual bool_t setBuffer(primo::codecs::MediaBuffer * buffer) = 0;
    };


    /**
        The parameter represents an untyped value which is stored in a char_t string.
        The parameter name constant documents how this value should be interpreted.
    */
    class StringParameter : public Parameter
    {
    public:

        /**
            Returns the parameter value. 
            
            @return A pointer to zero terminated string.
            The returned pointer cannot be <b>NULL</b> but the string can be empty.
        */
        virtual const char_t* value() const = 0;

        /**
            Sets the parameter value.

            @param value [in] A pointer to zero terminated string.
            If <b>NULL</b> it is interpreted as if an empty string is provided.
            The string pointed by <i>value</i> is copied internally and is not required to be valid after the method returns. 
        */
        virtual bool_t setValue(const char_t* value) = 0;
    };

    /**
    The parameter represents a VideoStreamInfo object.
    */
    class VideoStreamInfoParameter : public Parameter
    {
    public:
        /**
        Returns the VideoStreamInfo instance.

        @return A pointer to VideoStreamInfo object.
        The returned pointer can be NULL.
        */
        virtual primo::codecs::VideoStreamInfo* streamInfo() const = 0;

        /**
        Sets the video stream info.

        @param buffer [in] A pointer to VideoStreamInfo that will be stored in the parameter.
        It is allowed to set a NULL object.
        Internally the VideoStreamInfo specified by <i>streamInfo</i> is retained (if not NULL), so it is safe to release it
        after this method returns. The previous VideoStreamInfo is released (if not NULL).
        */
        virtual bool_t setStreamInfo(primo::codecs::VideoStreamInfo * streamInfo) = 0;
    };

    /**
        The parameter represents a 64-bit integer.
    */
    class IntParameter : public Parameter
    {
    public:

        /**
            Returns the parameter value. 
            
            @return A 64-bit integer. 
        */
        virtual int64_t value() const = 0;

        /**
            Sets the parameter value.

            @param value [in] A 64-bit integer.
        */
        virtual bool_t setValue(int64_t value) = 0;
    };


    /**
        The parameter represents a 64-bit floating point number (double precision).
    */
    class FloatParameter : public Parameter
    {
    public:

        /**
            Returns the parameter value. 
            
            @return A 64-bit floating point number.
        */
        virtual double value() const = 0;

        /**
            Sets the parameter value.

            @param value [in] A 64-bit floating point number.
        */
        virtual bool_t setValue(double value) = 0;
    };

    
    /**
        A collection of Parameter objects. It is used for passing parameters to different AVBlocks components. 

        @see Parameter
        @see Library::createParameterList
    */
    class ParameterList : public primo::Reference
    {
    public:
        /**
            Adds a parameter to the collection.

            @param item [in] A pointer to a Parameter object that should be added to the collection.
            It is allowed to add a <b>NULL</b> item to the collection.
            Internally if the Parameter object is not <b>NULL</b> it is retained by ParameterList, 
            so it is safe to release it after this method returns.

            @return TRUE if the specified item is added to the collection, otherwise FALSE.

            @see \ref Library "Parameter creating methods in Library"
            @see \ref object_management "Object Management Topic"
        */
        virtual bool_t add(Parameter* item) = 0;
        
        /**
            Removes a parameter from the collection.

            @param itemIndex [in] The index of the parameter that should be removed from the collection.
            Indexing starts from 0.

            @return TRUE if the specified item is removed from the collection, otherwise FALSE.

            When a Parameter object is removed from the collection it is also released.

            @see \ref object_management "Object Management"
        */
        virtual bool_t remove(int32_t itemIndex) = 0;
        
        /**
            Removes all parameters from the collection.

            When removed from the collection each Parameter object is released.

            @return TRUE if the collection is cleared, otherwise FALSE.

            @see \ref object_management "Object Management"
        */
        virtual bool_t clear() = 0;

        /**
            Returns the number of parameters in the collection.

            @return number of items in the collection.
        */
        virtual int32_t count() const = 0;

        /**
            Returns a parameter specified by index.

            @param index [in] The index of the parameter to be returned. Indexing starts from 0.

            @return A pointer to the Parameter object specified by index or <b>NULL</b> if the index is invalid.

            The caller is not required to release the returned object.
            @see \ref object_management "Object Management"
        */
        virtual Parameter* at(int32_t index) const = 0;

        /**
            Returns a parameter specified by its name.

            @param name [in] A pointer to a parameter name for which a Parameter object shall be returned.
            The name is case sensitive and only the first 64 characters are compared.

            @return A pointer to a Parameter object which has the name specified by the <i>name</i> parameter.
            <b>NULL</b> if there's no parameter with the specified name in the collection.

            
            If two or more parameters in the collection have the same name the one with the lowest index is returned.
        */
        virtual Parameter* itemByName(const char* name) const = 0;

        /**
            Sets a new parameter at the specified index.

            @param itemIndex [in] The index in the collection where the new item shall be set.
            This must be a valid index in the collection. Indexing starts from 0.

            @param item [in] A pointer to the Parameter object that shall be set.
            It is allowed to set a <b>NULL</b> value to the collection.
            Internally the new item is retained (if not <b>NULL</b>), so it is safe to release it 
            after this method returns. The old item is released (if not <b>NULL</b>).

            @return TRUE The item is set, FALSE otherwise.
            @see \ref object_management "Object Management"
        */
        virtual bool_t setAt(int32_t itemIndex, Parameter* item) = 0;

        /**
            Inserts a parameter at the specified index.

            @param itemIndex [in] The index in the collection where the new item shall be inserted.
            This must be a valid index in the collection. Indexing starts from 0.
            After the new item is inserted in the collection its index is the value of the <i>itemIndex</i> param.

            @param item [in] A pointer to the Parameter object that shall be inserted.
            It is allowed to insert a <b>NULL</b> value in the collection.
            Internally the new item is retained (if not <b>NULL</b>), so it is safe to release it 
            after this method returns.

            @return TRUE The item is set, FALSE otherwise.
            @see \ref object_management "Object Management"
        */
        virtual bool_t insert(int32_t itemIndex, Parameter* item) = 0;

        /**
            Creates an IntParameter and adds it to the collection.

            @param name [in] The parameter name. A pointer to zero terminated ANSI string.
            If <b>NULL</b> it is interpreted as if an empty string is provided.
            The string pointed by <i>name</i> is copied internally and is not required to be valid after the method returns.

            @param value [in] The parameter value.

            @return TRUE if the parameter is added to the collection, otherwise FALSE.

            @see Library::createIntParameter
        */
        virtual bool_t addInt(const char* name, int64_t value) = 0;

        /**
            Creates a FloatParameter and adds it to the collection.

            @param name [in] The parameter name. A pointer to zero terminated ANSI string.
            If <b>NULL</b> it is interpreted as if an empty string is provided.
            The string pointed by <i>name</i> is copied internally and is not required to be valid after the method returns.

            @param value [in] The parameter value.

            @return TRUE if the parameter is added to the collection, otherwise FALSE.

            @see Library::createFloatParameter
        */
        virtual bool_t addFloat(const char* name, double value) = 0;

        /**
            Creates a StringParameter and adds it to the collection.

            @param name [in] The parameter name. A pointer to zero terminated ANSI string.
            If <b>NULL</b> it is interpreted as if an empty string is provided.
            The string pointed by <i>name</i> is copied internally and is not required to be valid after the method returns.

            @param value [in] The parameter value.

            @return TRUE if the parameter is added to the collection, otherwise FALSE.

            @see Library::createStringParameter
        */
        virtual bool_t addString(const char* name, const char_t* value) = 0;

        /**
            Creates a MediaBufferParameter and adds it to the collection.

            @param name [in] The parameter name. A pointer to zero terminated ANSI string.
            If <b>NULL</b> it is interpreted as if an empty string is provided.
            The string pointed by <i>name</i> is copied internally and is not required to be valid after the method returns.

            @param value [in] A pointer to MediaBuffer that will be stored in the parameter.

            @return TRUE if the parameter is added to the collection, otherwise FALSE.

            @see Library::createMediaBufferParameter
        */
        virtual bool_t addMediaBuffer(const char* name, primo::codecs::MediaBuffer* value) = 0;

        /**
            Creates a VideoStreamInfoParameter and adds it to the collection.

            @param name [in] The parameter name. A pointer to zero terminated ANSI string.
            If <b>NULL</b> it is interpreted as if an empty string is provided.
            The string pointed by <i>name</i> is copied internally and is not required to be valid after the method returns.

            @param value [in] A pointer to VideoStreamInfo that will be stored in the parameter.

            @return TRUE if the parameter is added to the collection, otherwise FALSE.

            @see Library::createVideoStreamInfoParameter
        */
        virtual bool_t addVideoStreamInfo(const char* name, codecs::VideoStreamInfo* value) = 0;

        /**
            Creates a deep copy of this object.

            @return	A new ParameterList object.
            The caller is responsible for releasing the new object when it is not needed anymore.

            @return	<b>NULL</b> when there is not enough memory to clone the object.

            @remarks
            This method clones every parameter in the collection by calling Parameter::clone.

            @remarks
            When cloned an immutable object becomes mutable.

            @see \ref primo::avblocks::Parameter::clone() "Parameter::clone"
            @see \ref object_management "Object Management"
        */
        virtual ParameterList* clone() const = 0;

        /**
            Returns whether the object is immutable. 
            An immmutable object cannot be modified and all modifying methods fail to produce a result.

            @return TRUE if the object is immutable, FALSE otherwise.

            @remarks
            An immutable object <b>can</b> be modified by the AVBlocks library.

            @remarks
            Object immutability spreads to all nested objects. 
            Therefore it is not possible to add/set an immutable object to a mutable object.

            @remarks
            When cloned an immutable object becomes mutable.
        */
        virtual bool_t immutable() const = 0;

    protected:
        /**
            This object can be destroyed only by the AVBlocks library.
        */
        ~ParameterList() {}
    };

    
    /**
        MediaPin represents an elementary media stream.
        
        MediaPin object can exist for audio, video or any other type of stream. 
        However, AVBlocks can process only the audio and video streams.
        MediaPin objects are used as inputs and outputs of Transcoder.

        @see Library::createMediaPin
        @see \ref object_management "Object Management"
    */
    class MediaPin: public primo::Reference
    {	
    public:
        /**
            Returns the Pin connection ID. This should not be confused with the stream ID that is defined for certain stream types.

            @return Connection ID which is either an unique value or a special value from the PinConnection::Enum.

            A Transcoder object matches and connects input and output pins by their connection ID.
            The Transcoder processes the connected pins and ignores the unconnected pins.

            Pins are connected to one another according to the following rules:
            <ul>
            <li>Input pins with connection ID greater than PinConnection::Auto are connected to output pins with the same connection ID.
            In this case the Transcoder requires that a single unique connection ID is used by exactly one input and one output pin.
            </li>
            <li>Input and output pins with connection ID equal to PinConnection::Auto are connected automatically based on their media type (audio or video)</li>
            <li>Input and output pins with connection ID equal to PinConnection::Disabled are ignored.</li>
            </ul>

            @see PinConnection::Enum
            @see Transcoder
        */
        virtual int32_t connection() const = 0;

        /**
            Sets the Pin connection ID. This should not be confused with the stream ID that is defined for certain stream types.

            @param connectionID [in] Pin connection ID should be either an unique value or a special value from the PinConnection::Enum.

            A Transcoder object matches and connects input and output pins by their connection ID.
            The Transcoder processes the connected pins and ignores the unconnected pins.

            Pins are connected to one another according to the following rules:
            <ul>
            <li>Input pins with connection ID greater than PinConnection::Auto are connected to output pins with the same connection ID.
            In this case the Transcoder requires that a single unique connection ID is used by exactly one input and one output pin.
            </li>
            <li>Input and output pins with connection ID equal to PinConnection::Auto are connected automatically based on their media type (audio or video)</li>
            <li>Input and output pins with connection ID equal to PinConnection::Disabled are ignored.</li>
            </ul>

            @return TRUE if the connection ID is successfully set, otherwise FALSE.

            @see PinConnection::Enum
            @see Transcoder
        */
        virtual bool_t setConnection(int32_t connectionID) = 0;

        /**
            Returns information about the elementary stream represented by this pin.

            @return A \link primo::codecs::StreamInfo StreamInfo\endlink object.

            The Transcoder ignores pins that are not audio or video elementary streams.

            If the media type of the returned \link primo::codecs::StreamInfo StreamInfo\endlink object
            is \link primo::codecs::MediaType::Audio MediaType::Audio\endlink, 
            it is safe to cast it to \link primo::codecs::AudioStreamInfo AudioStreamInfo\endlink.

            If the media type of the returned \link primo::codecs::StreamInfo StreamInfo\endlink object
            is \link primo::codecs::MediaType::Video MediaType::Video\endlink, 
            it is safe to cast it to \link primo::codecs::VideoStreamInfo VideoStreamInfo\endlink.
            
            The caller is not required to release the returned object.

            @see \ref object_management "Object Management"
        */
        virtual primo::codecs::StreamInfo* streamInfo() const = 0;

        /**
            Sets the stream information for the pin.

            @param streamInfo [in] A pointer to a StreamInfo object. If <b>NULL</b> the stream info is cleared.
            Internally the StreamInfo object is retained by the MediaPin, so it is safe to release it after this method returns.

            @see MediaPin::streamInfo
            @see \ref object_management "Object Management"
        */
        virtual bool_t setStreamInfo(primo::codecs::StreamInfo* streamInfo) = 0;

        /**
            Returns a collection of pin parameters.
            These parameters affect the processing of the elementary stream represented by this pin.
            
            @return A ParameterList object. Can be <b>NULL</b> if the collection is not set. The default value is an empty list.

            The caller is not required to release the returned object.
            @see \ref object_management "Object Management"
        */
        virtual ParameterList* params() const = 0;

        /**
            Sets the filter parameters for the pin.

            @param params [in] A pointer to a ParameterList object. If <b>NULL</b> the stored collection is reset to <b>NULL</b>.
            Internally the ParameterList object is retained by the MediaPin, so it is safe to release it after this method returns.

            @return TRUE if the parameter list is successfully set, otherwise FALSE.

            @see \ref object_management "Object Management"
        */
        virtual bool_t setParams(ParameterList* params) = 0;

        /**
            Creates a deep copy of this object.

            @return	A new MediaPin object.
            The caller is responsible for releasing the new object when it is not needed anymore.
         
            @return 
            <b>NULL</b> if there is not enough memory to clone the object.

            @remarks
            This method uses the \link primo::codecs::StreamInfo::clone StreamInfo::clone \linkend and \link primo::avblocks::ParameterList::clone ParameterList::clone \linkend methods
            to clone the streamInfo and the params members.

            @remarks
            When cloned an immutable object becomes mutable.

            @see \ref object_management "Object Management"
        */
        virtual MediaPin* clone() const = 0;

        /**
            Returns whether the object is immutable.
            An immmutable object cannot be modified and all modifying methods fail to produce a result.

            @return TRUE if the object is immutable, FALSE otherwise.

            @remarks
            An immutable object <b>can</b> be modified by the AVBlocks library.

            @remarks
            Object immutability spreads to all nested objects.
            Therefore it is not possible to add/set an immutable object to a mutable object.

            @remarks
            When cloned an immutable object becomes mutable.
        */
        virtual bool_t immutable() const = 0;

    protected:
        /**
            This object can be destroyed only by the AVBlocks library.
        */
        ~MediaPin() {}
    };

    /**
        MediaPinList represents the elementary pins within a MediaSocket object.

        This is a modifiable and indexable collection that exists only in the context of a MediaSocket
        and cannot be created separately from it, nor detached.
    */
    class MediaPinList
    {
    public:

        /**
            Adds a pin (elementary stream) to the collection.

            @param item [in] A pointer to a MediaPin object that should be added to the collection.
            It is allowed to add a <b>NULL</b> item to the collection.
            Internally if the MediaPin object is not <b>NULL</b> it is retained by MediaPinList, 
            so it is safe to release it after this method returns.

            @return TRUE if the specified item is add to the collection, otherwise FALSE.

            @see Library::createMediaPin
            @see \ref object_management "Object Management"
        */
        virtual bool_t add(MediaPin* item) = 0;
        
        /**
            Removes a pin (elementary stream) from the collection.

            @param itemIndex [in] The index of the pin that should be removed from the collection.
            Indexing starts from 0.

            @return TRUE if the specified item is removed from the collection, otherwise FALSE.

            When an MediaPin object is removed from the collection it is also released.

            @see \ref object_management "Object Management"
        */
        virtual bool_t remove(int32_t itemIndex) = 0;
        
        /**
            Removes all pins from the collection.

            When removed from the collection each MediaPin object is released.
            
            @return TRUE if the collection is cleared, otherwise FALSE.

            @see \ref object_management "Object Management"
        */
        virtual bool_t clear() = 0;

        /**
            Returns the number of pins in the collection.

            @return number of items in the collection.
        */
        virtual int32_t count() const = 0;

        /**
            Returns a pin (elementary stream) specified by index.

            @param itemIndex The index of the elementary pin to be returned. Indexing starts from 0.

            @return A pointer to the MediaPin object specified by <i>itemIndex</i>.

            The caller is not required to release the returned object.
            
            @see \ref object_management "Object Management"
        */
        virtual MediaPin* at(int32_t itemIndex) const = 0;

        /**
            Sets a new pin at the specified index.

            @param itemIndex [in] The index in the collection where the new item shall be set.
            This must be a valid index in the collection. Indexing starts from 0.

            @param item [in] A pointer to the MediaPin object that shall be set.
            It is allowed to set a <b>NULL</b> value to the collection.
            Internally the new item is retained (if not <b>NULL</b>), so it is safe to release it 
            after this method returns. The old item is released (if not <b>NULL</b>).

            @return TRUE The item is set, FALSE otherwise.
            
            @see \ref object_management "Object Management"
        */
        virtual bool_t setAt(int32_t itemIndex, MediaPin* item) = 0;

        /**
            Inserts a pin at the specified index.

            @param itemIndex [in] The index in the collection where the new item shall be inserted.
            This must be a valid index in the collection. Indexing starts from 0.
            After the new item is inserted in the collection its index is the value of the <i>itemIndex</i> param.

            @param item [in] A pointer to the MediaPin object that shall be inserted.
            It is allowed to insert a <b>NULL</b> value in the collection.
            Internally the new item is retained (if not <b>NULL</b>), so it is safe to release it 
            after this method returns.

            @return TRUE The item is inserted, FALSE otherwise.
            
            @see \ref object_management "Object Management"
        */
        virtual bool_t insert(int32_t itemIndex, MediaPin* item) = 0;

        /**
            Returns whether the object is immutable.
            An immmutable object cannot be modified and all modifying methods fail to produce a result.

            @return TRUE if the object is immutable, FALSE otherwise.

            @remarks
            An immutable object <b>can</b> be modified by the AVBlocks library.

            @remarks
            Object immutability spreads to all nested objects.
            Therefore it is not possible to add/set an immutable object to a mutable object.
        */
        virtual bool_t immutable() const = 0;

    protected:
        /**
            This object can be destroyed only by the AVBlocks library.
        */
        ~MediaPinList() {}
    };

    /**
        Defines output presets supported by AVBlocks.

        Each AVBlocks preset represents a collection of audio and video properties and parameters.
        It can be used as an easy way to specify the required output of the transcoder.

        @see Library::createMediaSocket (const char* preset)
    */
    namespace Preset
    {	
            /**
            Describes output audio presets structured by codec and device type.
            */
            namespace Audio
            {
                /**
                Generic audio presets (unrelated to specific devices)
                */
                namespace Generic
                {
                    /**
                    AAC audio stream in ADTS format, stereo, 128 kbps
                    */
                    static const char AAC[] = "aac";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [AAC](@ref primo::codecs::StreamType::AAC)										|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_ADTS](@ref primo::codecs::StreamSubType::AAC_ADTS)							|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    */

                    /** M4A presets (AAC audio in MP4 container) */
                    namespace M4A
                    {
                        /**
                        AAC audio stream in MP4 container, 44100 Hz, stereo, 96 kbps, CBR
                        */
                        static const char LowQuality[] = "mp4.aac.low.quality";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)			    				|
                        | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 96000 bps 																		|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        AAC audio stream in MP4 container, 44100 Hz, stereo, 128 kbps, CBR
                        */
                        static const char GoodQuality[] = "mp4.aac.good.quality";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)			    				|
                        | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps 																		|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        AAC audio stream in MP4 container, 44100 Hz, stereo, 160 kbps, CBR
                        */
                        static const char HighQuality[] = "mp4.aac.high.quality";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)			    				|
                        | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 160000 bps 																		|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        // TODO: Possible duplicate of Generic::M4A::GoodQuality?
                        /**
                        AAC audio stream in MP4 container, 44100 Hz, stereo, 128 kbps, CBR<br/>
                        Normal (standard) quality.
                        */
                        static const char CBR_128kbps[] = "mp4.aac.44100hz.128kbps";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        AAC audio stream in MP4 format, 44100 Hz, stereo, 256 kbps, CBR<br />
                        High quality.
                        */
                        static const char CBR_256kbps[] = "mp4.aac.44100hz.256kbps";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 256000 bps																		|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */
                    }

                    /** MP3 presets (MPEG-1 Layer 3) */
                    namespace MP3
                    {
                        /**
                        MPEG-1 Layer 3 audio, 44100Hz, stereo, 128 kbps, CBR<br />
                        */
                        static const char CBR_128kbps[] = "mp3.44100hz.128kbps";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer3](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer3)			|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        MPEG-1 Layer 3 audio, 44100Hz, stereo, 256 kbps, CBR <br />
                        */
                        static const char CBR_256kbps[] = "mp3.44100hz.256kbps";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer3](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer3)			|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 256000 bps																		|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        MPEG-1 Layer 3 audio, 44100 Hz, mono, 32 kbps, CBR
                        */
                        static const char AMRadio[] = "mp3.am.radio.32kbps";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer3](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer3)			|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 32000 bps																		    |
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutMono](@ref primo::codecs::AudioChannelFlags::LayoutMono)				    |
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 1																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        MPEG-1 Layer 3 audio, 44100 Hz, stereo, 96 kbps, CBR
                        */
                        static const char FMRadio[] = "mp3.fm.radio.32kbps";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer3](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer3)			|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 96000 bps																		    |
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        MPEG-1 Layer 3 audio, 44100 Hz, mono, 64 kbps, CBR
                        */
                        static const char Voice[] = "mp3.voice.64kbps";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer3](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer3)			|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 64000 bps																		    |
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutMono](@ref primo::codecs::AudioChannelFlags::LayoutMono)		    		|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 1																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        MPEG-1 Layer 3 audio, 44100 Hz, stereo, 112 kbps, CBR
                        */
                        static const char Radio[] = "mp3.radio.112kbps";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer3](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer3)			|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 112000 bps																	    |
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)	    		|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        <b>Audio</b>: MPEG-1 Layer 3 audio, 44100 Hz, stereo, 128 kbps, CBR <br/>
                        <b>Container</b>: MP3 <br/>
                        */
                        static const char Tape[] = "mp3.tape.128kbps";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer3](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer3)			|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																	    |
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)	    		|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        <b>Audio</b>: MPEG-1 Layer 3 audio, 44100 Hz, stereo, 160 kbps, CBR <br/>
                        <b>Container</b>: MP3 <br/>
                        */
                        static const char HiFi[] = "mp3.hi.fi.160kbps";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer3](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer3)			|
                        | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [VBR](@ref primo::codecs::BitrateMode::VBR)										|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 160000 bps																	    |
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)	    		|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        <b>Audio</b>: MPEG-1 Layer 3 audio, 44100 Hz, stereo, 192 kbps, CBR <br/>
                        <b>Container</b>: MP3 <br/>
                        */
                        static const char CD[] = "mp3.audiocd.192kbps";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer3](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer3)			|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 192000 bps																		|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)	    		|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        <b>Audio</b>: MPEG-1 Layer 3 audio, 44100 Hz, stereo, 256 kbps, CBR <br/>
                        <b>Container</b>: MP3 <br/>
                        */
                        static const char Studio[] = "mp3.studio.256kbps";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer3](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer3)			|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 256000 bps																		|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)	    		|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */
                    }

                    /** Presets for Windows Media Audio */
                    namespace WMA
                    {
                        /** Presets for the WMA standard codec */
                        namespace Standard
                        {
                            /**
                            <b>Audio</b>: Windows Media Standard, 44100 Hz, stereo, 64 kbps, CBR<br/>
                            <b>Container</b>: [ASF](@ref primo::codecs::StreamType::ASF) <br/>
                            */
                            static const char CBR_64kbps[] = "wma.cbr.64kbps";
                            /*!<
                            ### Socket
                            | Property																									| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [StreamType](@ref primo::codecs::StreamType)																| [ASF](@ref primo::codecs::StreamType::ASF)										|

                            ### %Audio Pin
                            | Property																									| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [WMA](@ref primo::codecs::StreamType::WMA)										|
                            | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 64000 bps																		    |
                            | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                            | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                            | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                            | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                            */

                            /**
                            <b>Audio</b>: Windows Media Standard, 44100 Hz, stereo, 96 kbps, CBR <br/>
                            <b>Container</b>: [ASF](@ref primo::codecs::StreamType::ASF) <br/>
                            */
                            static const char CBR_96kbps[] = "wma.cbr.96kbps";
                            /*!<
                            ### Socket
                            | Property																									| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [StreamType](@ref primo::codecs::StreamType)																| [ASF](@ref primo::codecs::StreamType::ASF)										|

                            ### %Audio Pin
                            | Property																									| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [WMA](@ref primo::codecs::StreamType::WMA)										|
                            | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 96000 bps																		    |
                            | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                            | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                            | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                            | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                            */

                            /**
                            <b>Audio</b>: Windows Media Standard, 44100 Hz, stereo, 128 kbps, CBR <br/>
                            <b>Container</b>: [ASF](@ref primo::codecs::StreamType::ASF) <br/>
                            */
                            static const char CBR_128kbps[] = "wma.cbr.128kbps";
                            /*!<
                            ### Socket
                            | Property																									| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [StreamType](@ref primo::codecs::StreamType)																| [ASF](@ref primo::codecs::StreamType::ASF)										|

                            ### %Audio Pin
                            | Property																									| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [WMA](@ref primo::codecs::StreamType::WMA)										|
                            | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                            | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                            | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                            | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                            | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                            */

                            /**
                            <b>Audio</b>: Windows Media Standard, 44100 Hz, stereo, 160 kbps, CBR <br/>
                            <b>Container</b>: [ASF](@ref primo::codecs::StreamType::ASF) <br/>
                            */
                            static const char CBR_160kbps[] = "wma.cbr.160kbps";
                            /*!<
                            ### Socket
                            | Property																									| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [StreamType](@ref primo::codecs::StreamType)																| [ASF](@ref primo::codecs::StreamType::ASF)										|

                            ### %Audio Pin
                            | Property																									| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [WMA](@ref primo::codecs::StreamType::WMA)										|
                            | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 160000 bps																		|
                            | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                            | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                            | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                            | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                            */
                        }

                        /** Presets for the WMA Pro codec */
                        namespace Professional
                        {
                            /**
                            <b>Audio</b>: Windows Media Professional, 44100 Hz, stereo, Quality 75 (0-100), VBR <br/>
                            <b>Container</b>: [ASF](@ref primo::codecs::StreamType::ASF) <br/>

                            Normal quality VBR with average bitrate close to 128 kbps.
                            */
                            static const char VBR_Q75[] = "wma.pro.q75";
                            /*!<
                            ### Socket
                            | Property																									| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [StreamType](@ref primo::codecs::StreamType)																| [ASF](@ref primo::codecs::StreamType::ASF)										|

                            ### %Audio Pin
                            | Property																									| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [WMA_Professional](@ref primo::codecs::StreamType::WMA_Professional)				|
                            | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [VBR](@ref primo::codecs::BitrateMode::VBR)										|
                            | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                            | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                            | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                            | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|

                            ### %Audio Pin %Parameters
                            | %Parameter																								| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [Quality](@ref primo::avblocks::Param::Encoder::Audio::WMA)												| 75																				|
                            */

                            /**
                            <b>Audio</b>: Windows Media Professional, 44100 Hz, stereo, Quality 90 (0-100), VBR <br/>
                            <b>Container</b>: [ASF](@ref primo::codecs::StreamType::ASF) <br/>

                            High quality VBR with average bitrate close to 192 kbps.
                            */
                            static const char VBR_Q90[] = "wma.pro.q90";
                            /*!<
                            ### Socket
                            | Property																									| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [StreamType](@ref primo::codecs::StreamType)																| [ASF](@ref primo::codecs::StreamType::ASF)										|

                            ### %Audio Pin
                            | Property																									| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [WMA_Professional](@ref primo::codecs::StreamType::WMA_Professional)				|
                            | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [VBR](@ref primo::codecs::BitrateMode::VBR)										|
                            | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                            | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                            | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                            | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|

                            ### %Audio Pin %Parameters
                            | %Parameter																								| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [Quality](@ref primo::avblocks::Param::Encoder::Audio::WMA)												| 90																				|
                            */

                            /**
                            <b>Audio</b>: Windows Media Professional, 44100 Hz, stereo, Quality 100 (0-100), VBR <br/>
                            <b>Container</b>: [ASF](@ref primo::codecs::StreamType::ASF) <br/>

                            High quality VBR with average bitrate close to 256 kbps.
                            */
                            static const char VBR_Q100[] = "wma.pro.q100";
                            /*!<
                            ### Socket
                            | Property																									| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [StreamType](@ref primo::codecs::StreamType)																| [ASF](@ref primo::codecs::StreamType::ASF)										|

                            ### %Audio Pin
                            | Property																									| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [WMA_Professional](@ref primo::codecs::StreamType::WMA_Professional)				|
                            | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [VBR](@ref primo::codecs::BitrateMode::VBR)										|
                            | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                            | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                            | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                            | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|

                            ### %Audio Pin %Parameters
                            | %Parameter																								| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [Quality](@ref primo::avblocks::Param::Encoder::Audio::WMA)												| 100																				|
                            */
                        }

                        /** Presets for the WMA Lossless codec */
                        namespace Lossless
                        {
                            /**
                            <b>Audio</b>: Windows Media Lossless, 44100 Hz, stereo <br/>
                            <b>Container</b>: [ASF](@ref primo::codecs::StreamType::ASF) <br/>

                            Windows Media Lossless Audio (CD Quality)
                            */
                            static const char CD[] = "wma.lossless.audiocd";
                            /*!<
                            ### Socket
                            | Property																									| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [StreamType](@ref primo::codecs::StreamType)																| [ASF](@ref primo::codecs::StreamType::ASF)										|

                            ### %Audio Pin
                            | Property																									| Value																				|
                            | ----------------------------------------------------														| :-----------:																		|
                            | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [WMA_Lossless](@ref primo::codecs::StreamType::WMA_Lossless)						|
                            | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                            | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                            | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                            */
                        }
                    }

                    /** Presets for Vorbis audio in Ogg container */
                    namespace OggVorbis
                    {
                        /**
                        <b>Audio</b>: Vorbis, frequency not explicitly specified, quality 0.4 (from -0.1 to 1.0), VBR<br/>
                        <b>Container</b>: [OGG](@ref primo::codecs::StreamType::OGG) <br/>
                         
                        Normal quality VBR with average bitrate close to 128 kbps at 44100 Hz.
                        */
                        static const char VBR_Q4[] = "ogg.vorbis.q4";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [OGG](@ref primo::codecs::StreamType::OGG)										|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [Vorbis](@ref primo::codecs::StreamType::Vorbis)									|

                        ### %Audio Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [Quality](@ref primo::avblocks::Param::Encoder::Audio::Vorbis)											| 0.4																				|
                        */

                        /**
                        <b>Audio</b>: Vorbis, frequency not explicitly specified, quality 0.8 (from -0.1 to 1.0), VBR<br/>
                        <b>Container</b>: [OGG](@ref primo::codecs::StreamType::OGG) <br/>

                        High quality VBR with average bitrate close to 256 kbps at 44100 Hz.
                        */
                        static const char VBR_Q8[] = "ogg.vorbis.q8";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [OGG](@ref primo::codecs::StreamType::OGG)										|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [Vorbis](@ref primo::codecs::StreamType::Vorbis)									|

                        ### %Audio Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [Quality](@ref primo::avblocks::Param::Encoder::Audio::Vorbis)											| 0.8																				|
                        */
                    }
                }

                /** Presets for audio compatible with AudioCD (LPCM, 44100 Hz, stereo, 16-bit) */
                namespace AudioCD
                {
                    /**
                    <b>Audio</b>: WAV with parameters as in Audio CD (PCM, 44100 Hz, 16-bit, stereo)<br/>
                    <b>Container</b>: [WAVE](@ref primo::codecs::StreamType::WAVE) <br/>
                    */
                    static const char WAV[] = "wav.audiocd";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [WAVE](@ref primo::codecs::StreamType::WAVE)										|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [LPCM](@ref primo::codecs::StreamType::LPCM)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    | [BytesPerFrame (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBytesPerFrame)					| 4																					|
                    */
                }

                /** Presets for audio compatible with DVD-Video */
                namespace DVD
                {
                    /**
                    MPEG-1 Layer 2 audio, 48000 Hz (suitable for DVD) <br />
                    */
                    static const char MP2[] = "mp2.dvd";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer2](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer2)			|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 192000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */
                }
            }

            /**
            Describes output video presets structured by codec and device type.
            */
            namespace Video
            {
                /**
                Generic video presets. These presets are not related to any specific device.
                */
                namespace Generic
                {
                    /**
                    Generic MP4 presets (audio and video streams in MP4 container). These presets are not related to any specific device.
                    */
                    namespace MP4
                    {
                        /**
                        Video: H.264, 29.97 fps, 1920x1080, bitrate 5400 kbps, profile Baseline, aspect 16:9, Level 4 <br />
                        Audio: AAC, 44100 Hz, stereo, bitrate 160 kbps<br />
                        Container: MP4<br />
                        */
                        static const char H264_1080p[] = "generic.mp4.h264.1080p";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 5400000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1920																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                        | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                        | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                        | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                        | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                        | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                        | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                        | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                        | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                        | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 4.0																				|
                        | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 160000 bps																		|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        Video: H.264, 29.97 fps, 1280x720, bitrate 2400kbps, profile Baseline, aspect 16:9, Level 3.1 <br />
                        Audio: AAC, 44100 Hz, stereo, bitrate 160 kbps<br />
                        Container: MP4<br />
                        */
                        static const char H264_720p[] = "generic.mp4.h264.720p";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 2400000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                        | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                        | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                        | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                        | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                        | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                        | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                        | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                        | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                        | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.1																				|
                        | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 160000 bps																		|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        Video: H.264, 29.97 fps, 854x480, bitrate 1200 kbps, profile Baseline, aspect 16:9, Level 3.1 <br />
                        Audio: AAC, 44100 Hz, stereo, bitrate 128 kbps
                        Container: MP4
                        */
                        static const char H264_Wide_480p[] = "generic.mp4.h264.wide.480p";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 1200000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 854																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                        | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                        | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                        | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                        | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                        | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                        | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                        | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                        | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                        | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.1																				|
                        | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        Video: H264, 29.97 fps, 640x480, bitrate 900 kbps, profile Baseline, aspect 4:3, Level 3 <br />
                        Audio: AAC, 44100 Hz, stereo, bitrate 128 kbps<br />
                        Container: MP4<br />
                        */
                        static const char H264_480p[] = "generic.mp4.h264.480p";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 900000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 640																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                        | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 4																					|
                        | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 3																					|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                        | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                        | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                        | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                        | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                        | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                        | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                        | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.0																				|
                        | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        Video: H.264, 29.97 fps, 640x360, bitrate 720 kbps, profile Baseline, aspect 16:9, Level 3 <br />
                        Audio: AAC, 44100 Hz, stereo, bitrate 128 kbps<br />
                        Container: MP4<br />
                        */
                        static const char H264_Wide_360p[] = "generic.mp4.h264.wide.360p";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 720000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 640																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 360																				|
                        | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                        | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                        | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                        | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                        | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                        | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                        | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                        | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                        | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.0																				|
                        | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        Video: H.264, 29.97 fps, 480x360, bitrate 600 kbps, profile Baseline, aspect 4:3, Level 3 <br />
                        Audio: AAC, 44100 Hz, stereo, bitrate 128 kbps<br />
                        Container: MP4<br />
                        */
                        static const char H264_360p[] = "generic.mp4.h264.360p";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 600000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 480																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 360																				|
                        | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 4																					|
                        | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 3																					|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                        | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                        | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                        | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                        | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                        | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                        | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                        | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.0																				|
                        | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        Video: H264, 15 fps, 320x240, bitrate 300 kbps, profile Baseline, aspect 4:3, Level 1.3 <br />
                        Audio: AAC, 22050 Hz, stereo, bitrate 64 kbps<br />
                        Container: MP4<br />
                        */
                        static const char H264_240p[] = "generic.mp4.h264.240p";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 300000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 320																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 240																				|
                        | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 4																					|
                        | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 3																					|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 15																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                        | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                        | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                        | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                        | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                        | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                        | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                        | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 1.3																				|
                        | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 64000 bps																			|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 22050 Hz																			|
                        */

                        /**
                        Video: H.264<br/>
                        Audio: AAC<br/>
                        Container: MP4<br/>
                        
                        Besides the video codec, audio codec and the container, this encoding preset does not enforce any specific audio and video properties.
                        Width, height, frame rate, audio sampling rate, and other properties will be taken from the input.
                        */
                        static const char Base_H264_AAC[] = "mp4.h264.aac";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                        */
                    }
                    
                    /** WebM presets */
                    namespace WebM
                    {
                        /**
                        Video: VP8<br/>
                        Audio: Vorbis<br/>
                        Container: %WebM<br/>

                        Besides the video codec, audio codec and the container, this encoding preset does not enforce any specific audio and video properties.
                        Width, height, frame rate, audio sampling rate, and other properties will be taken from the input.
                        */
                        static const char Base_VP8_Vorbis[] = "webm.vp8.vorbis";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [WebM](@ref primo::codecs::StreamType::WebM)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [VP8](@ref primo::codecs::StreamType::VP8)										|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [Vorbis](@ref primo::codecs::StreamType::Vorbis)									|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                        */

                        /**
                        Video: VP8, 24 fps, 320x240, bitrate 350 kbps, VBR, keyframe distance 120 <br />
                        Audio: Vorbis, 44100 Hz, 64 kbit, 2 channels, VBR <br />
                        Container: %WebM
                        */
                        static const char VP8_3G_240p_350K_Vorbis_64K[] = "webm.vp8.3g.240p.350k.vorbis.64k";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [WebM](@ref primo::codecs::StreamType::WebM)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [VP8](@ref primo::codecs::StreamType::VP8)										|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 350000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 320																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 240																				|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 24																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [KeyFrameMaxDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMaxDistance)				| 120																				|
                        | [KeyFrameMinDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMinDistance)				| 120																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [Vorbis](@ref primo::codecs::StreamType::Vorbis)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 64000 bps																			|
                        | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [ABR](@ref primo::codecs::BitrateMode::ABR)										|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        Video: VP8, 24 fps, 426x240, bitrate 350 kbps, VBR, keyframe distance 120 <br />
                        Audio: Vorbis, 44100 Hz, 64 kbit, 2 channels, VBR <br />
                        Container: %WebM
                        */
                        static const char VP8_Wide_3G_240p_350K_Vorbis_64K[] = "webm.vp8.wide.3g.240p.350k.vorbis.64k";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [WebM](@ref primo::codecs::StreamType::WebM)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [VP8](@ref primo::codecs::StreamType::VP8)										|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 350000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 426																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 240																				|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 24																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [KeyFrameMaxDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMaxDistance)				| 120																				|
                        | [KeyFrameMinDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMinDistance)				| 120																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [Vorbis](@ref primo::codecs::StreamType::Vorbis)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 64000 bps																			|
                        | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [ABR](@ref primo::codecs::BitrateMode::ABR)										|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        Video: VP8, 30 fps, 480x360, bitrate 700 kbps, VBR, keyframe distance 150 <br />
                        Audio: Vorbis, 44100 Hz, 64 kbit, 2 channels, VBR <br />
                        Container: %WebM
                        */
                        static const char VP8_4G_360p_700K_Vorbis_64K[] = "webm.vp8.4g.360p.700k.vorbis.64k";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [WebM](@ref primo::codecs::StreamType::WebM)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [VP8](@ref primo::codecs::StreamType::VP8)										|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 700000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 480																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 360																				|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [KeyFrameMaxDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMaxDistance)				| 150																				|
                        | [KeyFrameMinDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMinDistance)				| 150																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [Vorbis](@ref primo::codecs::StreamType::Vorbis)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 64000 bps																			|
                        | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [ABR](@ref primo::codecs::BitrateMode::ABR)										|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        Video: VP8, 30 fps, 640x360, bitrate 700 kbps, VBR, keyframe distance 150 <br />
                        Audio: Vorbis, 44100 Hz, 64 kbit, 2 channels, VBR <br />
                        Container: %WebM
                        */
                        static const char VP8_Wide_4G_360p_700K_Vorbis_64K[] = "webm.vp8.wide.4g.360p.700k.vorbis.64k";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [WebM](@ref primo::codecs::StreamType::WebM)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [VP8](@ref primo::codecs::StreamType::VP8)										|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 700000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 640																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 360																				|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [KeyFrameMaxDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMaxDistance)				| 150																				|
                        | [KeyFrameMinDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMinDistance)				| 150																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [Vorbis](@ref primo::codecs::StreamType::Vorbis)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 64000 bps																			|
                        | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [ABR](@ref primo::codecs::BitrateMode::ABR)										|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        Video: VP8, 30 fps, 640x480, bitrate 1200 kbps, VBR, keyframe distance 300 <br />
                        Audio: Vorbis, 44100 Hz, 128 kbit, 2 channels, VBR <br />
                        Container: %WebM
                        */
                        static const char VP8_SD_480p_1200K_Vorbis_128K[] = "webm.vp8.sd.480p.1200k.vorbis.128k";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [WebM](@ref primo::codecs::StreamType::WebM)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [VP8](@ref primo::codecs::StreamType::VP8)										|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 1200000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 640																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [KeyFrameMaxDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMaxDistance)				| 300																				|
                        | [KeyFrameMinDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMinDistance)				| 300																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [Vorbis](@ref primo::codecs::StreamType::Vorbis)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                        | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [ABR](@ref primo::codecs::BitrateMode::ABR)										|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        Video: VP8, 30 fps, 854x480, bitrate 1200 kbps, VBR, keyframe distance 300 <br />
                        Audio: Vorbis, 44100 Hz, 128 kbit, 2 channels, VBR <br />
                        Container: %WebM
                        */
                        static const char VP8_Wide_SD_480p_1200K_Vorbis_128K[] = "webm.vp8.wide.sd.480p.1200k.vorbis.128k";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [WebM](@ref primo::codecs::StreamType::WebM)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [VP8](@ref primo::codecs::StreamType::VP8)										|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 1200000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 854																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [KeyFrameMaxDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMaxDistance)				| 300																				|
                        | [KeyFrameMinDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMinDistance)				| 300																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [Vorbis](@ref primo::codecs::StreamType::Vorbis)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                        | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [ABR](@ref primo::codecs::BitrateMode::ABR)										|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        Video: VP8, 25 fps, 720x576, bitrate 1600 kbps, VBR, keyframe distance 250 <br />
                        Audio: Vorbis, 44100 Hz, 128 kbit, 2 channels, VBR <br />
                        Container: %WebM
                        */
                        static const char VP8_SD_576p_1600K_Vorbis_128K[] = "webm.vp8.sd.576p.1600k.vorbis.128k";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [WebM](@ref primo::codecs::StreamType::WebM)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [VP8](@ref primo::codecs::StreamType::VP8)										|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 1600000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 720																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 576																				|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 25																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [KeyFrameMaxDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMaxDistance)				| 250																				|
                        | [KeyFrameMinDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMinDistance)				| 250																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [Vorbis](@ref primo::codecs::StreamType::Vorbis)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                        | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [ABR](@ref primo::codecs::BitrateMode::ABR)										|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        Video: VP8, 25 fps, 1024x576, bitrate 1600 kbps, VBR, keyframe distance 250 <br />
                        Audio: Vorbis, 44100 Hz, 128 kbit, 2 channels, VBR <br />
                        Container: %WebM
                        */
                        static const char VP8_Wide_SD_576p_1600K_Vorbis_128K[] = "webm.vp8.wide.sd.576p.1600k.vorbis.128k";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [WebM](@ref primo::codecs::StreamType::WebM)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [VP8](@ref primo::codecs::StreamType::VP8)										|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 1600000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1024																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 576																				|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 25																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [KeyFrameMaxDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMaxDistance)				| 250																				|
                        | [KeyFrameMinDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMinDistance)				| 250																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [Vorbis](@ref primo::codecs::StreamType::Vorbis)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                        | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [ABR](@ref primo::codecs::BitrateMode::ABR)										|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */

                        /**
                        Video: VP8, 30 fps, 1280x720, bitrate 2500 kbps, VBR, keyframe distance 300 <br />
                        Audio: Vorbis, 48000 Hz, 192 kbit, 2 channels, VBR <br />
                        Container: %WebM
                        */
                        static const char VP8_Wide_HD_720p_2500K_Vorbis_192K[] = "webm.vp8.wide.hd.720p.2500k.vorbis.192k";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [WebM](@ref primo::codecs::StreamType::WebM)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [VP8](@ref primo::codecs::StreamType::VP8)										|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 2500000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [KeyFrameMaxDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMaxDistance)				| 300																				|
                        | [KeyFrameMinDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMinDistance)				| 300																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [Vorbis](@ref primo::codecs::StreamType::Vorbis)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 192000 bps																		|
                        | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [ABR](@ref primo::codecs::BitrateMode::ABR)										|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                        */

                        /**
                        Video: VP8, 30 fps, 1920x1080, bitrate 5000 kbps, VBR, keyframe distance 300 <br />
                        Audio: Vorbis, 48000 Hz, 192 kbit, 2 channels, VBR <br />
                        Container: %WebM
                        */
                        static const char VP8_Wide_HD_1080p_5000K_Vorbis_192K[] = "webm.vp8.wide.hd.1080p.5000k.vorbis.192k";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [WebM](@ref primo::codecs::StreamType::WebM)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [VP8](@ref primo::codecs::StreamType::VP8)										|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 5000000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1920																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [KeyFrameMaxDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMaxDistance)				| 300																				|
                        | [KeyFrameMinDistance](@ref primo::avblocks::Param::Encoder::Video::VP8::KeyFrameMinDistance)				| 300																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [Vorbis](@ref primo::codecs::StreamType::Vorbis)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 192000 bps																		|
                        | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [ABR](@ref primo::codecs::BitrateMode::ABR)										|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                        */
                    }
                }

                /** Kindle presets */
                namespace Kindle
                {
                    /**
                    Video: H264, 30 fps, 1920x1080, bitrate 5400 kbps, profile Main, aspect 16:9, Level 4 <br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 160 kbps<br />
                    Container: MP4<br />
                    */
                    static const char KindleFireHD_8_9_MP4_H264_1080p[] = "KindleFireHD8.9.mp4.h264.1080p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 5400000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1920																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Main](@ref primo::codecs::H264Profile::Main)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 4.0																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 160000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Video: H264, 30 fps, 1280x720, bitrate 2200 kbps, profile Main, aspect 16:9, Level 4 <br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 160 kbps<br />
                    Container: MP4<br />
                    */
                    static const char KindleFireHD_MP4_H264_720p[] = "KindleFireHD.mp4.h264.720p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 2200000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Main](@ref primo::codecs::H264Profile::Main)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 4.0																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 160000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Video: H264, 30 fps, 1024x576, bitrate 1600 kbps, profile Main, aspect 16:9, Level 3.1 <br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 160 kbps<br />
                    Container: MP4<br />
                    */
                    static const char KindleFire_MP4_H264_576p[] = "KindleFire.mp4.h264.576p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 1600000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1024																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 576																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Main](@ref primo::codecs::H264Profile::Main)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.1																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 160000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */
                }

                /** Web Video Presets*/
                namespace Web
                {
                    /** 
                    Web MP4 Presets
                    */
                    namespace MP4
                    {
                        /**
                        Video: H264, 30 fps, 1280x720, bitrate 2200 kbps, profile Main, aspect 16:9, Level 3.1<br />
                        Audio: AAC, 44100 Hz, stereo, bitrate 160 kbps<br />
                        Container: MP4<br />
                        */
                        static const char H264_720p[] = "Web.mp4.h264.720p";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 2200000 bps																		|
                        | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                        | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                        | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                        | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                        | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|
                        | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Main](@ref primo::codecs::H264Profile::Main)										|
                        | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                        | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                        | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                        | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                        | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                        | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                        | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.1																				|
                        | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                        | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 160000 bps																		|
                        | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                        | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                        | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                        */
                    }
                }

                /** Presets for DVD-Video */
                namespace DVD
                {
                    /**
                    Video: MPEG-2, 25 fps, 720 x 576, aspect 4:3<br/>
                    Audio: MPEG-1 Layer 2, 48000 Hz, stereo <br />
                    Container: MPEG_PS <br \>
                    */
                    static const char PAL_4x3_MP2[] = "dvd.pal.4x3.mp2";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_PS](@ref primo::codecs::StreamType::MPEG_PS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG2_Video](@ref primo::codecs::StreamType::MPEG2_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 6000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ColorFormat (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setColorFormat)						| [YUV420](@ref primo::codecs::ColorFormat::YUV420)									|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 720																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 576																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 4																					|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 3																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 25.0																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [GOPSize](@ref primo::avblocks::Param::Encoder::Video::MPEG2::GOPSize)									| 15																				|
                    | [FrameRateConverter::Use](@ref primo::avblocks::Param::Video::FrameRateConverter)							| [On](@ref primo::avblocks::Use::On)												|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer2](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer2)			|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 192000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|

                    ### %Audio Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [ForceV1](@ref primo::avblocks::Param::Encoder::Audio::MPEG1)												| TRUE																				|
                    */

                    /**
                    Video: MPEG-2, 25 fps, 720 x 576, aspect 16:9<br/>
                    Audio: MPEG-1 Layer 2, 48000 Hz, stereo <br \>
                    Container: MPEG_PS <br \>
                    */
                    static const char PAL_16x9_MP2[] = "dvd.pal.16x9.mp2";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_PS](@ref primo::codecs::StreamType::MPEG_PS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG2_Video](@ref primo::codecs::StreamType::MPEG2_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 6000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ColorFormat (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setColorFormat)						| [YUV420](@ref primo::codecs::ColorFormat::YUV420)									|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 720																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 576																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 25.0																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [GOPSize](@ref primo::avblocks::Param::Encoder::Video::MPEG2::GOPSize)									| 15																				|
                    | [FrameRateConverter::Use](@ref primo::avblocks::Param::Video::FrameRateConverter)							| [On](@ref primo::avblocks::Use::On)												|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer2](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer2)			|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 192000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|

                    ### %Audio Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [ForceV1](@ref primo::avblocks::Param::Encoder::Audio::MPEG1)												| TRUE																				|
                    */

                    /**
                    Video: MPEG-2, 29.97 fps, 720 x 480, aspect 4:3 <br />
                    Audio: PCM, 16-bit, 48000 Hz, stereo <br />
                    Container: PS (MPEG-2 Program Stream) <br />
                    */
                    static const char NTSC_4x3_PCM[] = "dvd.ntsc.4x3.pcm";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_PS](@ref primo::codecs::StreamType::MPEG_PS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG2_Video](@ref primo::codecs::StreamType::MPEG2_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 6000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ColorFormat (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setColorFormat)						| [YUV420](@ref primo::codecs::ColorFormat::YUV420)									|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 720																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 4																					|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 3																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [GOPSize](@ref primo::avblocks::Param::Encoder::Video::MPEG2::GOPSize)									| 18																				|
                    | [FrameRateConverter::Use](@ref primo::avblocks::Param::Video::FrameRateConverter)							| [On](@ref primo::avblocks::Use::On)												|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [LPCM](@ref primo::codecs::StreamType::LPCM)										|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    | [BytesPerFrame (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBytesPerFrame)					| 4																					|
                    */

                    /**
                    Video: MPEG-2, 29.97 fps, 720 x 480, aspect 4:3 <br />
                    Audio: MPEG-1 Layer 2, 48000 Hz, stereo <br />
                    Container: PS (MPEG-2 Program Stream) <br />
                    */
                    static const char NTSC_4x3_MP2[] = "dvd.ntsc.4x3.mp2";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_PS](@ref primo::codecs::StreamType::MPEG_PS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG2_Video](@ref primo::codecs::StreamType::MPEG2_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 6000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ColorFormat (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setColorFormat)						| [YUV420](@ref primo::codecs::ColorFormat::YUV420)									|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 720																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 4																					|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 3																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [GOPSize](@ref primo::avblocks::Param::Encoder::Video::MPEG2::GOPSize)									| 18																				|
                    | [FrameRateConverter::Use](@ref primo::avblocks::Param::Video::FrameRateConverter)							| [On](@ref primo::avblocks::Use::On)												|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer2](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer2)			|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 192000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|

                    ### %Audio Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [ForceV1](@ref primo::avblocks::Param::Encoder::Audio::MPEG1)												| TRUE																				|
                    */

                    /**
                    Video: MPEG-2, 29.97 fps, 720 x 480, aspect 16:9 <br />
                    Audio: PCM, 16-bit, 48000 Hz, stereo <br />
                    Container: PS (MPEG-2 Program Stream) <br />
                    */
                    static const char NTSC_16x9_PCM[] = "dvd.ntsc.16x9.pcm";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_PS](@ref primo::codecs::StreamType::MPEG_PS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG2_Video](@ref primo::codecs::StreamType::MPEG2_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 6000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ColorFormat (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setColorFormat)						| [YUV420](@ref primo::codecs::ColorFormat::YUV420)									|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 720																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [GOPSize](@ref primo::avblocks::Param::Encoder::Video::MPEG2::GOPSize)									| 18																				|
                    | [FrameRateConverter::Use](@ref primo::avblocks::Param::Video::FrameRateConverter)							| [On](@ref primo::avblocks::Use::On)												|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [LPCM](@ref primo::codecs::StreamType::LPCM)										|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    | [BytesPerFrame (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBytesPerFrame)					| 4																					|
                    */

                    /**
                    Video: MPEG-2, 29.97 fps, 720 x 480, aspect 16:9 <br />
                    Audio: MPEG-1 Layer 2, 48000 Hz, stereo <br />
                    Container: PS (MPEG-2 Program Stream) <br />
                    */
                    static const char NTSC_16x9_MP2[] = "dvd.ntsc.16x9.mp2";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_PS](@ref primo::codecs::StreamType::MPEG_PS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG2_Video](@ref primo::codecs::StreamType::MPEG2_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 6000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ColorFormat (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setColorFormat)						| [YUV420](@ref primo::codecs::ColorFormat::YUV420)									|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 720																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [GOPSize](@ref primo::avblocks::Param::Encoder::Video::MPEG2::GOPSize)									| 18																				|
                    | [FrameRateConverter::Use](@ref primo::avblocks::Param::Video::FrameRateConverter)							| [On](@ref primo::avblocks::Use::On)

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer2](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer2)			|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 192000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|

                    ### %Audio Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [ForceV1](@ref primo::avblocks::Param::Encoder::Audio::MPEG1)												| TRUE																				|
                    */
                }

                /** Presets for Video CD */
                namespace VCD
                {
                    /**
                    Video: MPEG-1, 25fps, 352 x 288, 1100 kbps<br/>
                    Audio: MPEG-1 Layer 2, 44100Hz, stereo, 192 kbps<br/>
                    Container: MPEG Program Stream<br/>
                    Video CD (PAL)
                    */
                    static const char PAL[] = "vcd.pal";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_PS](@ref primo::codecs::StreamType::MPEG_PS)								|
                    | [StreamSubType](@ref primo::codecs::StreamSubType)														| [MPEG1_System](@ref primo::codecs::StreamSubType::MPEG1_System)					|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG1_Video](@ref primo::codecs::StreamType::MPEG1_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 1100000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 352																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 288																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 4																					|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 3																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 25																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer2](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer2)			|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 192000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                    */

                    /**
                    Video: MPEG-1, 29.97 fps, 352 x 240, 1100 kbps<br/>
                    Audio: MPEG-1 Layer 2, 44100Hz, stereo, 192 kbps<br/>
                    Container: MPEG Program Stream<br/>
                    Video CD (NTSC)
                    */
                    static const char NTSC[] = "vcd.ntsc";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_PS](@ref primo::codecs::StreamType::MPEG_PS)								|
                    | [StreamSubType](@ref primo::codecs::StreamSubType)														| [MPEG1_System](@ref primo::codecs::StreamSubType::MPEG1_System)					|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG1_Video](@ref primo::codecs::StreamType::MPEG1_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 1100000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 352																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 240																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 4																					|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 3																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer2](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer2)			|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 192000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 44100 Hz																			|
                    */
                }

                /** Presets for iPad devices */
                namespace iPad
                {
                    /**
                    Video: H.264, 30 fps, 768 x 576, 2500 kbps <br/>
                    Audio: AAC, 48000 Hz, stereo, 128 kbps <br/>
                    Container: MP4<br/>
                    Standard Definition Video, optimized for iPad.
                    */
                    static const char H264_576p[] = "ipad.mp4.h264.576p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 2500000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 768																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 576																				|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Video: H.264, 30 fps, 1280 x 720, 4000 kbps<br/>
                    Audio: AAC, 48000 Hz, stereo, 128 kbps <br/>
                    Container: MP4<br/>
                    High Definition Video, optimized for iPad.
                    */
                    static const char H264_720p[] = "ipad.mp4.h264.720p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 4000000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Video: MPEG-4 Visual, 30 fps, 640 x 480, 2500 kbps <br/>
                    Audio: AAC, 48000 Hz, 128 kbps, stereo<br/>
                    Container: MP4<br/>
                    Near Standard Definition Video, optimized for iPad.
                    */
                    static const char MPEG4_480p[] = "ipad.mp4.mpeg4.480p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG4_Video](@ref primo::codecs::StreamType::MPEG4_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 2500000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 640																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */
                }

                /** Presets for AppleTV devices */
                namespace AppleTV
                {
                    /**
                    Video: H.264, 24 fps, 640 x 480, 1500 kbps <br/>
                    Audio: AAC, 48000 Hz, stereo, 48 kbps <br/>
                    Container: MP4 <br/>
                    Optimized for Apple TV, 1st generation.
                    */
                    static const char H264_480p[] = "appletv.h264.480p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 1500000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 640																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 24																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 48000 bps																			|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Video: H.264, 24 fps, 1280 x 720, 4000 kbps <br/>
                    Audio: AAC, 48000 Hz, stereo, 48 kbps<br/>
                    Container: MP4<br/>
                    Optimized for Apple TV, 2nd generation.
                    */
                    static const char H264_720p[] = "appletv.h264.720p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 4000000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 24																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 48000 bps																			|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Video: MPEG-4 Visual, 24 fps, 640 x 480, 2000 kbps <br/>
                    Audio: AAC, 48000 Hz, stereo, 48 kbps<br/>
                    Container: MP4<br/>
                    Optimized for Apple TV, 1st generation.
                    */
                    static const char MPEG4_480p[] = "appletv.mpeg4.480p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG4_Video](@ref primo::codecs::StreamType::MPEG4_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 2000000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 640																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 24																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 48000 bps																			|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Video: H.264, 30 fps, 1280x720, bitrate 5000 kbps, profile Main, aspect 16:9, Level 3.1<br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 160 kbps<br />
                    Container: MP4<br />
                    */
                    static const char AppleTV2G_MP4_H264_720p[] = "appleTV2g.mp4.h264.720p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 5000000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Main](@ref primo::codecs::H264Profile::Main)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.1																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 160000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Video: H.264, 30 fps, 1920x1080, bitrate 5000 kbps, profile High, aspect 16:9, Level 4<br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 160 kbps<br />
                    Container: MP4<br />
                    */
                    static const char AppleTV3G_MP4_H264_1080p[] = "appleTV3g.mp4.h264.1080p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 5000000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1920																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [High](@ref primo::codecs::H264Profile::High)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| TRUE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 4.0																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 160000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */
                }

                /** Presets for iPhone devices */
                namespace iPhone
                {
                    /**
                    Video: H.264, 25 fps, 640 x 480, 1000 kbps<br/>
                    Audio: AAC, 48000 Hz, stereo, 96 kbps<br/>
                    Container: MP4<br/>
                    Optimized for iPhone.
                    */
                    static const char H264_480p[] = "iphone.mp4.h264.480p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 1000000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 640																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 25																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 96000 bps																			|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Video: MPEG-4 Visual, 25 fps, 640 x 480, 2000 kbps<br/>
                    Audio: AAC, 48000 Hz, stereo, 64 kbps<br/>
                    Container: MP4<br/>
                    Optimized for iPhone.
                    */
                    static const char MPEG4_480p[] = "iphone.mp4.mpeg4.480p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG4_Video](@ref primo::codecs::StreamType::MPEG4_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 2000000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 640																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 25																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 64000 bps																			|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Video: H.264, 30 fps, 1280x720, bitrate 2200 kbps, profile Main, aspect 16:9, Level 3.1<br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 160 kbps<br />
                    Container: MP4<br />
                    */
                    static const char iPhone4_MP4_H264_720p[] = "iPhone4.mp4.h264.720p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 2200000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Main](@ref primo::codecs::H264Profile::Main)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.1																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 160000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Video: H.264, 30 fps, 1920x1080, bitrate 5000 kbps, profile High, aspect 16:9, Level 4.1<br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 160 kbps<br />
                    Container: MP4<br />
                    */
                    static const char iPhone4S_MP4_H264_1080p[] = "iPhone4s.mp4.h264.1080p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 5000000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1920																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [High](@ref primo::codecs::H264Profile::High)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| TRUE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 4.1																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 160000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Video: H.264, 30 fps, 640x480, bitrate 600 kbps, profile Baseline, aspect 4:3, Level 3<br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 160 kbps<br />
                    Container: MP4<br />
                    */
                    static const char iPhone3GS_MP4_H264_480[] = "iPhone3gs.mp4.h264.480p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 600000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 640																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 4																					|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 3																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.0																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 160000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */
                }

                /** Preset for iPod devices */
                namespace iPod
                {
                    /**
                    Video: H.264 Baseline Profile, 25 fps, 320 x 240, 300 kbps<br/>
                    Audio: AAC, 48000 Hz, stereo, 64 kbps<br/>
                    Container: MP4<br/>
                    Optimized for iPod.
                    */
                    static const char H264_240p[] = "ipod.mp4.h264.240p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 300000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 320																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 240																				|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 25																				|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 1																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 25																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [AccessUnitDelimiters](@ref primo::avblocks::Param::Encoder::Video::H264::AccessUnitDelimiters)			| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 64000 bps																			|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Video: MPEG-4 Visual, 25 fps, 320 x 240, 600 kbps<br/>
                    Audio: AAC, 48000 Hz, stereo, 64 kbps<br/>
                    Container: MP4<br/>
                    Optimized for iPod.
                    */
                    static const char MPEG4_240p[] = "ipod.mp4.mpeg4.240p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG4_Video](@ref primo::codecs::StreamType::MPEG4_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 600000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 320																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 240																				|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 25																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 64000 bps																			|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Video: H.264, 30 fps, 640x480, bitrate 1500 kbps, profile Baseline, aspect 4:3, Level 3<br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 160 kbps<br />
                    Container: MP4<br />
                    */
                    static const char iPodTouch_MP4_H264_480p[] = "iPodTouch.mp4.h264.480p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 1500000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 640																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 4																					|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 3																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.0																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 160000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */
                }

                /** Apple HTTP Live Streaming Presets*/
                namespace AppleLiveStreaming
                {
                    /**
                    Apple HTTP Live Streaming: 11128 Kbps <br />
                    Video: H.264, 1920x1080, bitrate 11000 kbps, 29.97 fps, aspect ratio 16:9, profile High, Level 4.0, MaxReferenceFrame 3 <br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 128 kbps <br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char WiFi_Wide_H264_1920x1080_30p_11000K_AAC_128K[] = "wifi.wide.h264.1920x1080.30p.11000k.aac.128k";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 11000000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1920																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [High](@ref primo::codecs::H264Profile::High)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| TRUE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 4.0																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Apple HTTP Live Streaming: 4628 Kbps <br />
                    Video: H.264, 1280x720, bitrate 4500 kbps, 29.97 fps, aspect ratio 16:9, profile Main, Level 3.1, MaxReferenceFrame 3 <br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 128 kbps <br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char WiFi_Wide_H264_1280x720_30p_4500K_AAC_128K[] = "wifi.wide.h264.1280x720.30p.4500k.aac.128k";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 4500000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Main](@ref primo::codecs::H264Profile::Main)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.1																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Apple HTTP Live Streaming: 2628 Kbps <br />
                    Video: H.264, 1280x720, bitrate 2500 kbps, 29.97 fps, aspect ratio 16:9, profile Main, Level 3.1, MaxReferenceFrame 3 <br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 128 kbps <br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char WiFi_Wide_H264_1280x720_30p_2500K_AAC_128K[] = "wifi.wide.h264.1280x720.30p.2500k.aac.128k";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 2500000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Main](@ref primo::codecs::H264Profile::Main)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.1																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Apple HTTP Live Streaming: 1928 Kbps <br />
                    Video: H.264, 960x540, bitrate 1800 kbps, 29.97 fps, aspect ratio 16:9, profile Main, Level 3.1, MaxReferenceFrame 3 <br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 128 kbps <br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char WiFi_Wide_H264_960x540_30p_1800K_AAC_128K[] = "wifi.wide.h264.960x540.30p.1800k.aac.128k";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 1800000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 960																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 540																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Main](@ref primo::codecs::H264Profile::Main)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.1																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Apple HTTP Live Streaming: 1296 Kbps <br />
                    Video: H.264, 640x360, bitrate 1200 kbps, 29.97 fps, aspect ratio 16:9, profile Main, Level 3.1, MaxReferenceFrame 3 <br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 96 kbps <br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char WiFi_Wide_H264_640x360_30p_1200K_AAC_96K[] = "wifi.wide.h264.640x360.30p.1200k.aac.96k";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 1200000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 640																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 360																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Main](@ref primo::codecs::H264Profile::Main)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.1																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 96000 bps																			|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Apple HTTP Live Streaming: 696 Kbps <br />
                    Video: H.264, 640x360, bitrate 600 kbps, 29.97 fps, aspect ratio 16:9, profile Baseline, Level 3.0, MaxReferenceFrame 1 <br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 96 kbps <br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char WiFi_Wide_H264_640x360_30p_600K_AAC_96K[] = "wifi.wide.h264.640x360.30p.600k.aac.96k";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 600000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 640																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 360																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 1																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.0																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 96000 bps																			|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Apple HTTP Live Streaming: 4628 Kbps <br />
                    Video: H.264, 1280x960, bitrate 4500 kbps, 29.97 fps, aspect ratio 4:3, profile Main, Level 3.1, MaxReferenceFrame 3 <br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 128 kbps <br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char WiFi_H264_1280x960_30p_4500K_AAC_128K[] = "wifi.h264.1280x960.30p.4500k.aac.128k";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 4500000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 960																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 4																					|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 3																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Main](@ref primo::codecs::H264Profile::Main)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.1																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Apple HTTP Live Streaming: 2628 Kbps <br />
                    Video: H.264, 960x720, bitrate 2500 kbps, 29.97 fps, aspect ratio 4:3, profile Main, Level 3.1, MaxReferenceFrame 3 <br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 128 kbps <br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char WiFi_H264_960x720_30p_2500K_AAC_128K[] = "wifi.h264.960x720.30p.2500k.aac.128k";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 2500000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 960																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 4																					|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 3																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Main](@ref primo::codecs::H264Profile::Main)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.1																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Apple HTTP Live Streaming: 1296 Kbps <br />
                    Video: H.264, 960x720, bitrate 1200 kbps, 29.97 fps, aspect ratio 4:3, profile Main, Level 3.1, MaxReferenceFrame 3 <br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 96 kbps <br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char WiFi_H264_960x720_30p_1200K_AAC_96K[] = "wifi.h264.960x720.30p.1200k.aac.96k";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 2500000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 960																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 4																					|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 3																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Main](@ref primo::codecs::H264Profile::Main)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.1																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Apple Live Streaming: 1296 Kbps <br />
                    Video: H.264, 640x480, bitrate 1200 kbps, 29.97 fps, aspect ratio 4:3, profile Main, Level 3.1, MaxReferenceFrame 3 <br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 96 kbps <br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char WiFi_H264_640x480_30p_1200K_AAC_96K[] = "wifi.h264.640x480.30p.1200k.aac.96k";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 1200000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 640																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 4																					|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 3																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Main](@ref primo::codecs::H264Profile::Main)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.1																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 96000 bps																			|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Apple HTTP Live Streaming: 896 Kbps <br />
                    Video: H.264, 640x480, bitrate 800 kbps, 29.97 fps, aspect ratio 4:3, profile Main, Level 3.1, MaxReferenceFrame 3 <br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 96 kbps <br />
                    Container: MPEG-2 Transport Stream
                    */
                    static const char WiFi_H264_640x480_30p_800K_AAC_96K[] = "wifi.h264.640x480.30p.800k.aac.96k";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 800000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 640																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 4																					|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 3																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Main](@ref primo::codecs::H264Profile::Main)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 2																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 3																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.1																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 96000 bps																			|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Apple HTTP Live Streaming: 696 Kbps <br />
                    Video: H.264, 640x480, bitrate 600 kbps, 29.97 fps, aspect ratio 4:3, profile Baseline, Level 3.0, MaxReferenceFrame 1 <br />
                    Audio: AAC, 48000 Hz, stereo, bitrate 96 kbps <br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char WiFi_H264_640x480_30p_600K_AAC_96K[] = "wifi.h264.640x480.30p.600k.aac.96k";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 600000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 640																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 480																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 4																					|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 3																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 1																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.0																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 96000 bps																			|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Apple HTTP Live Streaming: 440 Kbps <br />
                    Video: H264, 400x224, bitrate 400 kbps, 29.97 fps, aspect ratio 16:9, profile Baseline, Level 3.0, MaxReferenceFrame 1 <br />
                    Audio: AAC, 22050 Hz, stereo, bitrate 40 kbps <br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char Cellular_Wide_H264_400x224_30p_400K_AAC_40K[] = "cellular.wide.h264.400x224.30p.400k.aac.40k";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 400000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 400																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 224																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 1																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.0																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 40000 bps																			|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 22050 Hz																			|
                    */

                    /**
                    Apple HTTP Live Streaming: 440 Kbps <br />
                    Video: H264, 400x300, bitrate 400 kbps, 29.97 fps, aspect ratio 4:3, profile Baseline, Level 3.0, MaxReferenceFrame 1 <br />
                    Audio: AAC, 22050 Hz, stereo, bitrate 40 kbps <br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char Cellular_H264_400x300_30p_400K_AAC_40K[] = "cellular.h264.400x300.30p.400k.aac.40k";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 400000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 400																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 300																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 4																					|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 3																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 1																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 90																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 3.0																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 40000 bps																			|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 22050 Hz																			|
                    */

                    /**
                    Alias for \see primo::avblocks::Preset::Video::AppleLiveStreaming::WiFi_H264_640x480_30p_1200K_AAC_96K
                    */
                    const char* const H264_480p = Preset::Video::AppleLiveStreaming::WiFi_H264_640x480_30p_1200K_AAC_96K;

                    /**
                    Alias for \see Preset::Video::AppleLiveStreaming::WiFi_Wide_H264_1280x720_30p_4500K_AAC_128K
                    */
                    const char* const H264_720p = Preset::Video::AppleLiveStreaming::WiFi_Wide_H264_1280x720_30p_4500K_AAC_128K;
                }

                /** Presets for Android phones */
                namespace AndroidPhone
                {
                    /**
                    Video: H.264 Baseline Profile, 30 fps, 480 x 360, 800 kbps<br/>
                    Audio: AAC, 48000 Hz, stereo, 128 kbps<br/>
                    Container: MP4<br/>
                    */
                    static const char H264_360p[] = "android-phone.mp4.h264.360p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 800000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 480																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 360																				|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 1																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 30																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Video: H.264 Baseline Profile, 30 fps, 1280 x 720, 2000 kbps<br/>
                    Audio: AAC, 48000 Hz, stereo, 128 kbps<br/>
                    Container: MP4<br/>
                    */
                    static const char H264_720p[] = "android-phone.mp4.h264.720p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 2000000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 1																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 30																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 128000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */
                }

                /** Presets for Android tablets */
                namespace AndroidTablet
                {
                    /**
                    Video: H.264 Baseline Profile, 30 fps, 1280 x 720, 4000 kbps<br/>
                    Audio: AAC, 48000 Hz, stereo, 192 kbps<br/>
                    Container: MP4<br/>
                    */
                    static const char H264_720p[] = "android-tablet.mp4.h264.720p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 4000000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 1																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 30																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 192000 bps																		|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */

                    /**
                    Video: VP8, 30 fps, 1280 x 720, 4000 kbps<br/>
                    Audio: Vorbis, 48000 Hz, stereo, 192 kbps<br/>
                    Container: WebM<br/>
                    */
                    static const char WebM_VP8_720p[] = "android-tablet.webm.vp8.720p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [WebM](@ref primo::codecs::StreamType::WebM)										|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [VP8](@ref primo::codecs::StreamType::VP8)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 4000000 bps																		|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 30																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [Vorbis](@ref primo::codecs::StreamType::Vorbis)									|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 192000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [ABR](@ref primo::codecs::BitrateMode::ABR)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    */
                }

                /** Presets for Blu-ray Disc Audio-Visual MPEG-2 Transport Stream */
                namespace BDAV
                {
                    /**
                    Video: H.264, 1920 x 1080, 29.97 fps interlaced, aspect ratio 16:9, 25000 kbps<br/>
                    Audio: LPCM, 48000 Hz, 16-bit<br/>
                    Container: BDAV MPEG-2 Transport Stream (m2ts)<br/>
                    */
                    static const char H264_1920x1080_5994i_PCM[] = "bdav-h264-1920x1080-59.94i-pcm";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|
                    | [StreamSubType](@ref primo::codecs::StreamType)															| [MPEG_TS_BDAV](@ref primo::codecs::StreamSubType::MPEG_TS_BDAV)					|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 25000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [VBR](@ref primo::codecs::BitrateMode::VBR)										|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1920																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 59.94																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [TopFieldFirst](@ref primo::codecs::ScanType::TopFieldFirst)						|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [High](@ref primo::codecs::H264Profile::High)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 3																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 2																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| TRUE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 20																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 41																				|
                    | [PictureCodingType](@ref primo::avblocks::Param::Encoder::Video::H264::PictureCodingType)					| 1																					|
                    | [NumSlices](@ref primo::avblocks::Param::Encoder::Video::H264::NumSlices)									| 4																					|
                    | [AccessUnitDelimiters](@ref primo::avblocks::Param::Encoder::Video::H264::AccessUnitDelimiters)			| TRUE																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [LPCM](@ref primo::codecs::StreamType::LPCM)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    | [BytesPerFrame (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBytesPerFrame)					| 4																					|
                    */

                    /**
                    Video: H.264, 1920 x 1080, 25 fps interlaced, aspect ratio 16:9, 25000 kbps<br/>
                    Audio: LPCM, 48000 Hz, 16-bit<br/>
                    Container: BDAV MPEG-2 Transport Stream (m2ts)<br/>
                    */
                    static const char H264_1920x1080_50i_PCM[] = "bdav-h264-1920x1080-50i-pcm";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|
                    | [StreamSubType](@ref primo::codecs::StreamType)															| [MPEG_TS_BDAV](@ref primo::codecs::StreamSubType::MPEG_TS_BDAV)					|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 25000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [VBR](@ref primo::codecs::BitrateMode::VBR)										|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1920																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 50																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [TopFieldFirst](@ref primo::codecs::ScanType::TopFieldFirst)						|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [High](@ref primo::codecs::H264Profile::High)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 3																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 2																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| TRUE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 20																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 41																				|
                    | [PictureCodingType](@ref primo::avblocks::Param::Encoder::Video::H264::PictureCodingType)					| 1																					|
                    | [NumSlices](@ref primo::avblocks::Param::Encoder::Video::H264::NumSlices)									| 4																					|
                    | [AccessUnitDelimiters](@ref primo::avblocks::Param::Encoder::Video::H264::AccessUnitDelimiters)			| TRUE																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [LPCM](@ref primo::codecs::StreamType::LPCM)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    | [BytesPerFrame (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBytesPerFrame)					| 4																					|
                    */

                    /**
                    Video: H.264, 1920 x 1080, 24 fps progressive, aspect ratio 16:9, 25000 kbps<br/>
                    Audio: LPCM, 48000 Hz, 16-bit<br/>
                    Container: BDAV MPEG-2 Transport Stream (m2ts)<br/>
                    */
                    static const char H264_1920x1080_24p_PCM[] = "bdav-h264-1920x1080-24p-pcm";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|
                    | [StreamSubType](@ref primo::codecs::StreamType)															| [MPEG_TS_BDAV](@ref primo::codecs::StreamSubType::MPEG_TS_BDAV)					|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 25000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [VBR](@ref primo::codecs::BitrateMode::VBR)										|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1920																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 24																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [High](@ref primo::codecs::H264Profile::High)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 3																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 2																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| TRUE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 20																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 41																				|
                    | [PictureCodingType](@ref primo::avblocks::Param::Encoder::Video::H264::PictureCodingType)					| 0																					|
                    | [NumSlices](@ref primo::avblocks::Param::Encoder::Video::H264::NumSlices)									| 4																					|
                    | [AccessUnitDelimiters](@ref primo::avblocks::Param::Encoder::Video::H264::AccessUnitDelimiters)			| TRUE																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [LPCM](@ref primo::codecs::StreamType::LPCM)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    | [BytesPerFrame (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBytesPerFrame)					| 4																					|
                    */

                    /**
                    Video: H.264, 1920 x 1080, 23.976 fps progressive, aspect ratio 16:9, 25000 kbps<br/>
                    Audio: LPCM, 48000 Hz, 16-bit<br/>
                    Container: BDAV MPEG-2 Transport Stream (m2ts)<br/>
                    */
                    static const char H264_1920x1080_2398p_PCM[] = "bdav-h264-1920x1080-23.976p-pcm";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|
                    | [StreamSubType](@ref primo::codecs::StreamType)															| [MPEG_TS_BDAV](@ref primo::codecs::StreamSubType::MPEG_TS_BDAV)					|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 25000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [VBR](@ref primo::codecs::BitrateMode::VBR)										|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1920																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 23.976																			|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [High](@ref primo::codecs::H264Profile::High)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 3																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 2																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| TRUE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 20																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 41																				|
                    | [PictureCodingType](@ref primo::avblocks::Param::Encoder::Video::H264::PictureCodingType)					| 0																					|
                    | [NumSlices](@ref primo::avblocks::Param::Encoder::Video::H264::NumSlices)									| 4																					|
                    | [AccessUnitDelimiters](@ref primo::avblocks::Param::Encoder::Video::H264::AccessUnitDelimiters)			| TRUE																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [LPCM](@ref primo::codecs::StreamType::LPCM)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    | [BytesPerFrame (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBytesPerFrame)					| 4																					|
                    */

                    /**
                    Video: H.264, 1440 x 1080, 29.97 fps interlaced, aspect ratio 16:9, 25000 kbps<br/>
                    Audio: LPCM, 48000 Hz, 16-bit<br/>
                    Container: BDAV MPEG-2 Transport Stream (m2ts)<br/>
                    */
                    static const char H264_1440x1080_5994i_PCM[] = "bdav-h264-1440x1080-59.94i-pcm";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|
                    | [StreamSubType](@ref primo::codecs::StreamType)															| [MPEG_TS_BDAV](@ref primo::codecs::StreamSubType::MPEG_TS_BDAV)					|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 25000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [VBR](@ref primo::codecs::BitrateMode::VBR)										|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1440																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 59.94																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [TopFieldFirst](@ref primo::codecs::ScanType::TopFieldFirst)						|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [High](@ref primo::codecs::H264Profile::High)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 3																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 2																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| TRUE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 20																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 41																				|
                    | [PictureCodingType](@ref primo::avblocks::Param::Encoder::Video::H264::PictureCodingType)					| 1																					|
                    | [NumSlices](@ref primo::avblocks::Param::Encoder::Video::H264::NumSlices)									| 4																					|
                    | [AccessUnitDelimiters](@ref primo::avblocks::Param::Encoder::Video::H264::AccessUnitDelimiters)			| TRUE																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [LPCM](@ref primo::codecs::StreamType::LPCM)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    | [BytesPerFrame (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBytesPerFrame)					| 4																					|
                    */

                    /**
                    Video: H.264, 1440 x 1080, 25 fps interlaced, aspect ratio 16:9, 25000 kbps<br/>
                    Audio: LPCM, 48000 Hz, 16-bit<br/>
                    Container: BDAV MPEG-2 Transport Stream (m2ts)<br/>
                    */
                    static const char H264_1440x1080_50i_PCM[] = "bdav-h264-1440x1080-50i-pcm";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|
                    | [StreamSubType](@ref primo::codecs::StreamType)															| [MPEG_TS_BDAV](@ref primo::codecs::StreamSubType::MPEG_TS_BDAV)					|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 25000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [VBR](@ref primo::codecs::BitrateMode::VBR)										|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1440																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 50																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [TopFieldFirst](@ref primo::codecs::ScanType::TopFieldFirst)						|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [High](@ref primo::codecs::H264Profile::High)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 3																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 2																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| TRUE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 20																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 41																				|
                    | [PictureCodingType](@ref primo::avblocks::Param::Encoder::Video::H264::PictureCodingType)					| 1																					|
                    | [NumSlices](@ref primo::avblocks::Param::Encoder::Video::H264::NumSlices)									| 4																					|
                    | [AccessUnitDelimiters](@ref primo::avblocks::Param::Encoder::Video::H264::AccessUnitDelimiters)			| TRUE																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [LPCM](@ref primo::codecs::StreamType::LPCM)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    | [BytesPerFrame (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBytesPerFrame)					| 4																					|
                    */

                    /**
                    Video: H.264, 1440 x 1080, 24 fps progressive, aspect ratio 16:9, 25000 kbps<br/>
                    Audio: LPCM, 48000 Hz, 16-bit<br/>
                    Container: BDAV MPEG-2 Transport Stream (m2ts)<br/>
                    */
                    static const char H264_1440x1080_24p_PCM[] = "bdav-h264-1440x1080-24p-pcm";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|
                    | [StreamSubType](@ref primo::codecs::StreamType)															| [MPEG_TS_BDAV](@ref primo::codecs::StreamSubType::MPEG_TS_BDAV)					|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 25000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [VBR](@ref primo::codecs::BitrateMode::VBR)										|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1440																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 24																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [High](@ref primo::codecs::H264Profile::High)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 3																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 2																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| TRUE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 20																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 41																				|
                    | [PictureCodingType](@ref primo::avblocks::Param::Encoder::Video::H264::PictureCodingType)					| 0																					|
                    | [NumSlices](@ref primo::avblocks::Param::Encoder::Video::H264::NumSlices)									| 4																					|
                    | [AccessUnitDelimiters](@ref primo::avblocks::Param::Encoder::Video::H264::AccessUnitDelimiters)			| TRUE																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [LPCM](@ref primo::codecs::StreamType::LPCM)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    | [BytesPerFrame (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBytesPerFrame)					| 4																					|
                    */

                    /**
                    Video: H.264, 1440 x 1080, 23.976 fps progressive, aspect ratio 16:9, 25000 kbps<br/>
                    Audio: LPCM, 48000 Hz, 16-bit<br/>
                    Container: BDAV MPEG-2 Transport Stream (m2ts)<br/>
                    */
                    static const char H264_1440x1080_2398p_PCM[] = "bdav-h264-1440x1080-23.976p-pcm";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|
                    | [StreamSubType](@ref primo::codecs::StreamType)															| [MPEG_TS_BDAV](@ref primo::codecs::StreamSubType::MPEG_TS_BDAV)					|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 25000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [VBR](@ref primo::codecs::BitrateMode::VBR)										|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1440																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 23.976																			|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [High](@ref primo::codecs::H264Profile::High)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 3																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 2																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| TRUE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 20																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 41																				|
                    | [PictureCodingType](@ref primo::avblocks::Param::Encoder::Video::H264::PictureCodingType)					| 0																					|
                    | [NumSlices](@ref primo::avblocks::Param::Encoder::Video::H264::NumSlices)									| 4																					|
                    | [AccessUnitDelimiters](@ref primo::avblocks::Param::Encoder::Video::H264::AccessUnitDelimiters)			| TRUE																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [LPCM](@ref primo::codecs::StreamType::LPCM)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    | [BytesPerFrame (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBytesPerFrame)					| 4																					|
                    */

                    /**
                    Video: H.264, 1280 x 720, 59.94 fps progressive, aspect ratio 16:9, 15000 kbps<br/>
                    Audio: LPCM, 48000 Hz, 16-bit<br/>
                    Container: BDAV MPEG-2 Transport Stream (m2ts)<br/>
                    */
                    static const char H264_1280x720_5994p_PCM[] = "bdav-h264-1280x720-59.94p-pcm";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|
                    | [StreamSubType](@ref primo::codecs::StreamType)															| [MPEG_TS_BDAV](@ref primo::codecs::StreamSubType::MPEG_TS_BDAV)					|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 15000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [VBR](@ref primo::codecs::BitrateMode::VBR)										|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 59.94																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [High](@ref primo::codecs::H264Profile::High)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 3																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 2																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| TRUE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 20																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 41																				|
                    | [PictureCodingType](@ref primo::avblocks::Param::Encoder::Video::H264::PictureCodingType)					| 0																					|
                    | [NumSlices](@ref primo::avblocks::Param::Encoder::Video::H264::NumSlices)									| 4																					|
                    | [AccessUnitDelimiters](@ref primo::avblocks::Param::Encoder::Video::H264::AccessUnitDelimiters)			| TRUE																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [LPCM](@ref primo::codecs::StreamType::LPCM)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    | [BytesPerFrame (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBytesPerFrame)					| 4																					|
                    */

                    /**
                    Video: H.264, 1280 x 720, 50 fps progressive, aspect ratio 16:9, 15000 kbps<br/>
                    Audio: LPCM, 48000 Hz, 16-bit<br/>
                    Container: BDAV MPEG-2 Transport Stream (m2ts)<br/>
                    */
                    static const char H264_1280x720_50p_PCM[] = "bdav-h264-1280x720-50p-pcm";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|
                    | [StreamSubType](@ref primo::codecs::StreamType)															| [MPEG_TS_BDAV](@ref primo::codecs::StreamSubType::MPEG_TS_BDAV)					|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 15000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [VBR](@ref primo::codecs::BitrateMode::VBR)										|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 50																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [High](@ref primo::codecs::H264Profile::High)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 3																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 2																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| TRUE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 20																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 41																				|
                    | [PictureCodingType](@ref primo::avblocks::Param::Encoder::Video::H264::PictureCodingType)					| 0																					|
                    | [NumSlices](@ref primo::avblocks::Param::Encoder::Video::H264::NumSlices)									| 4																					|
                    | [AccessUnitDelimiters](@ref primo::avblocks::Param::Encoder::Video::H264::AccessUnitDelimiters)			| TRUE																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [LPCM](@ref primo::codecs::StreamType::LPCM)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    | [BytesPerFrame (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBytesPerFrame)					| 4																					|
                    */

                    /**
                    Video: H.264, 1280 x 720, 24 fps progressive, aspect ratio 16:9, 15000 kbps<br/>
                    Audio: LPCM, 48000 Hz, 16-bit<br/>
                    Container: BDAV MPEG-2 Transport Stream (m2ts)<br/>
                    */
                    static const char H264_1280x720_24p_PCM[] = "bdav-h264-1280x720-24p-pcm";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|
                    | [StreamSubType](@ref primo::codecs::StreamType)															| [MPEG_TS_BDAV](@ref primo::codecs::StreamSubType::MPEG_TS_BDAV)					|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 15000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [VBR](@ref primo::codecs::BitrateMode::VBR)										|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 24																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [High](@ref primo::codecs::H264Profile::High)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 3																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 2																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| TRUE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 20																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 41																				|
                    | [PictureCodingType](@ref primo::avblocks::Param::Encoder::Video::H264::PictureCodingType)					| 0																					|
                    | [NumSlices](@ref primo::avblocks::Param::Encoder::Video::H264::NumSlices)									| 4																					|
                    | [AccessUnitDelimiters](@ref primo::avblocks::Param::Encoder::Video::H264::AccessUnitDelimiters)			| TRUE																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [LPCM](@ref primo::codecs::StreamType::LPCM)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    | [BytesPerFrame (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBytesPerFrame)					| 4																					|
                    */

                    /**
                    Video: H.264, 1280 x 720, 23.976 fps progressive, aspect ratio 16:9, 15000 kbps<br/>
                    Audio: LPCM, 48000 Hz, 16-bit<br/>
                    Container: BDAV MPEG-2 Transport Stream (m2ts)<br/>
                    */
                    static const char H264_1280x720_2398p_PCM[] = "bdav-h264-1280x720-23.976p-pcm";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|
                    | [StreamSubType](@ref primo::codecs::StreamType)															| [MPEG_TS_BDAV](@ref primo::codecs::StreamSubType::MPEG_TS_BDAV)					|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 15000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [VBR](@ref primo::codecs::BitrateMode::VBR)										|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 23.976																			|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Video Pin %Parameters
                    | %Parameter																								| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [High](@ref primo::codecs::H264Profile::High)										|
                    | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CABAC](@ref primo::codecs::H264EntropyCodingMode::CABAC)							|
                    | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 3																					|
                    | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 2																					|
                    | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| TRUE																				|
                    | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 20																				|
                    | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                    | [Level](@ref primo::avblocks::Param::Encoder::Video::H264::Level)											| 41																				|
                    | [PictureCodingType](@ref primo::avblocks::Param::Encoder::Video::H264::PictureCodingType)					| 0																					|
                    | [NumSlices](@ref primo::avblocks::Param::Encoder::Video::H264::NumSlices)									| 4																					|
                    | [AccessUnitDelimiters](@ref primo::avblocks::Param::Encoder::Video::H264::AccessUnitDelimiters)			| TRUE																				|
                    | [FixedFramerate](@ref primo::avblocks::Param::Encoder::Video::H264::FixedFramerate)						| TRUE																				|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [LPCM](@ref primo::codecs::StreamType::LPCM)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    | [BytesPerFrame (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBytesPerFrame)					| 4																					|
                    */
                }

                /** Presets for High-definition video MPEG-2 Transport Stream */
                namespace HDV
                {
                    /**
                    Video: MPEG-2, 29.97 fps, 1440x1080, aspect ratio 16:9 <br />
                    Audio: MPEG-1 Layer 2, 48000 Hz, stereo, 16bits <br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char MPEG_1080i_30p[] = "hdv.m2t.1080i.30p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG2_Video](@ref primo::codecs::StreamType::MPEG2_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 25000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ColorFormat (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setColorFormat)						| [YUV420](@ref primo::codecs::ColorFormat::YUV420)									|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1440																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [TopFieldFirst](@ref primo::codecs::ScanType::TopFieldFirst)						|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer2](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer2)			|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 384000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    */

                    /**
                    Video: MPEG-2, 25 fps, 1440x1080, aspect ratio 16:9 <br />
                    Audio: MPEG-1 Layer 2, 48000 Hz, stereo, 16bits<br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char MPEG_1080i_25p[] = "hdv.m2t.1080i.25p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG2_Video](@ref primo::codecs::StreamType::MPEG2_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 25000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ColorFormat (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setColorFormat)						| [YUV420](@ref primo::codecs::ColorFormat::YUV420)									|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1440																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 25																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [TopFieldFirst](@ref primo::codecs::ScanType::TopFieldFirst)						|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer2](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer2)			|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 384000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    */

                    /**
                    Video: MPEG-2, 29.97 fps, 1440x1080, aspect ratio 16:9 <br />
                    Audio: MPEG-1 Layer 2, 48000 Hz, stereo, 16bits<br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char MPEG_1080p_30p[] = "hdv.m2t.1080p.30p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG2_Video](@ref primo::codecs::StreamType::MPEG2_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 25000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ColorFormat (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setColorFormat)						| [YUV420](@ref primo::codecs::ColorFormat::YUV420)									|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1440																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer2](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer2)			|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 384000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    */

                    /**
                    Video: MPEG-2, 25 fps, 1440x1080, aspect ratio 16:9 <br />
                    Audio: MPEG-1 Layer 2, 48000 Hz, stereo, 16bits <br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char MPEG_1080p_25p[] = "hdv.m2t.1080p.25p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG2_Video](@ref primo::codecs::StreamType::MPEG2_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 25000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ColorFormat (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setColorFormat)						| [YUV420](@ref primo::codecs::ColorFormat::YUV420)									|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1440																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 25																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer2](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer2)			|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 384000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    */

                    /**
                    Video: MPEG-2, 23.976 fps, 1440x1080, aspect ratio 16:9 <br />
                    Audio: MPEG-1 Layer 2, 48000 Hz, stereo, 16bits<br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char MPEG_1080p_24p[] = "hdv.m2t.1080p.24p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG2_Video](@ref primo::codecs::StreamType::MPEG2_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 25000000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ColorFormat (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setColorFormat)						| [YUV420](@ref primo::codecs::ColorFormat::YUV420)									|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1440																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 1080																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 23.976																			|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer2](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer2)			|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 384000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    */

                    /**
                    Video: MPEG-2, 59.94 fps, 1280x720, aspect 16:9 <br />
                    Audio: MPEG-1 Layer 2, 48000 Hz, stereo, 16bits<br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char MPEG_720p_60p[] = "hdv.m2t.720p.60p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG2_Video](@ref primo::codecs::StreamType::MPEG2_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 18300000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ColorFormat (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setColorFormat)						| [YUV420](@ref primo::codecs::ColorFormat::YUV420)									|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 59.94																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer2](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer2)			|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 384000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    */

                    /**
                    Video: MPEG-2, 50 fps, 1280x720, aspect 16:9 <br />
                    Audio: MPEG-1 Layer 2, 48000 Hz, stereo, 16bits<br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char MPEG_720p_50p[] = "hdv.m2t.720p.50p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG2_Video](@ref primo::codecs::StreamType::MPEG2_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 18300000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ColorFormat (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setColorFormat)						| [YUV420](@ref primo::codecs::ColorFormat::YUV420)									|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 50																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer2](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer2)			|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 384000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    */

                    /**
                    Video: MPEG-2, 29.97 fps, 1280x720, aspect 16:9 <br />
                    Audio: MPEG-1 Layer 2, 48000 Hz, stereo, 16bits <br />
                    Container: TS (MPEG-2 Transport Stream)<br />
                    */
                    static const char MPEG_720p_30p[] = "hdv.m2t.720p.30p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG2_Video](@ref primo::codecs::StreamType::MPEG2_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 18300000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ColorFormat (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setColorFormat)						| [YUV420](@ref primo::codecs::ColorFormat::YUV420)									|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 29.97																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer2](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer2)			|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 384000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    */

                    /**
                    Video: MPEG-2, 25 fps, 1280x720, aspect 16:9 <br />
                    Audio: MPEG-1 Layer 2, 48000 Hz, stereo, 16bits <br />
                    Container: TS (MPEG-2 Transport Stream) <br />
                    */
                    static const char MPEG_720p_25p[] = "hdv.m2t.720p.25p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG2_Video](@ref primo::codecs::StreamType::MPEG2_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 18300000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ColorFormat (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setColorFormat)						| [YUV420](@ref primo::codecs::ColorFormat::YUV420)									|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 25																				|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer2](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer2)			|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 384000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    */

                    /**
                    Video: MPEG-2, 23.976 fps, 1280x720, aspect 16:9 <br />
                    Audio: MPEG-1 Layer 2, 48000 Hz, stereo, 16bits <br />
                    Container: TS (MPEG-2 Transport Stream) <br />
                    */
                    static const char MPEG_720p_24p[] = "hdv.m2t.720p.24p";
                    /*!<
                    ### Socket
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType](@ref primo::codecs::StreamType)																| [MPEG_TS](@ref primo::codecs::StreamType::MPEG_TS)								|

                    ### %Video Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG2_Video](@ref primo::codecs::StreamType::MPEG2_Video)						|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 18300000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ColorFormat (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setColorFormat)						| [YUV420](@ref primo::codecs::ColorFormat::YUV420)									|
                    | [FrameWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameWidth)						| 1280																				|
                    | [FrameHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameHeight)						| 720																				|
                    | [DisplayRatioWidth (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioWidth)			| 16																				|
                    | [DisplayRatioHeight (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setDisplayRatioHeight)		| 9																					|
                    | [FrameRate (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setFrameRate)							| 23.976																			|
                    | [ScanType (VideoStreamInfo)](@ref primo::codecs::VideoStreamInfo::setScanType)							| [Progressive](@ref primo::codecs::ScanType::Progressive)							|

                    ### %Audio Pin
                    | Property																									| Value																				|
                    | ----------------------------------------------------														| :-----------:																		|
                    | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [MPEG_Audio](@ref primo::codecs::StreamType::MPEG_Audio)							|
                    | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [MPEG_Audio_Layer2](@ref primo::codecs::StreamSubType::MPEG_Audio_Layer2)			|
                    | [Bitrate (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrate)										| 384000 bps																		|
                    | [BitrateMode (StreamInfo)](@ref primo::codecs::StreamInfo::setBitrateMode)								| [CBR](@ref primo::codecs::BitrateMode::CBR)										|
                    | [ChannelLayout (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannelLayout)					| [LayoutStereo](@ref primo::codecs::AudioChannelFlags::LayoutStereo)				|
                    | [Channels (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setChannels)							| 2																					|
                    | [SampleRate (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setSampleRate)						| 48000 Hz																			|
                    | [BitsPerSample (AudioStreamInfo)](@ref primo::codecs::AudioStreamInfo::setBitsPerSample)					| 16																				|
                    */
                }

                /** Presets for fastest possible encoding. These presets are designed for real-time video capture applications. */
                namespace Fast
                {
                    /** Fast MP4 presets (audio and video streams in MP4 container) */
                    namespace MP4
                    {
                        /**
                        Video: H.264<br/>
                        Audio: AAC<br/>
                        Container: MP4<br/>

                        %Fast H.264 encoding.
                        */
                        static const char H264_AAC[] = "fast.mp4.h264.aac";
                        /*!<
                        ### Socket
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType](@ref primo::codecs::StreamType)																| [MP4](@ref primo::codecs::StreamType::MP4)										|

                        ### %Video Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [H264](@ref primo::codecs::StreamType::H264)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AVC1](@ref primo::codecs::StreamSubType::AVC1)									|

                        ### %Video Pin %Parameters
                        | %Parameter																								| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [H264Profile](@ref primo::avblocks::Param::Encoder::Video::H264::Profile)									| [Baseline](@ref primo::codecs::H264Profile::Baseline)								|
                        | [EntropyCodingMode](@ref primo::avblocks::Param::Encoder::Video::H264::EntropyCodingMode)					| [CAVLC](@ref primo::codecs::H264EntropyCodingMode::CAVLC)							|
                        | [NumBFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumBFrames)								| 0																					|
                        | [NumRefFrames](@ref primo::avblocks::Param::Encoder::Video::H264::NumRefFrames)							| 1																					|
                        | [Transform8x8](@ref primo::avblocks::Param::Encoder::Video::H264::Transform8x8)							| FALSE																				|
                        | [KeyFrameInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameInterval)					| 15																				|
                        | [KeyFrameIDRInterval](@ref primo::avblocks::Param::Encoder::Video::H264::KeyFrameIDRInterval)				| 1																					|
                        | [QualitySpeed](@ref primo::avblocks::Param::Encoder::Video::H264::QualitySpeed)							| 0																					|
                        | [RateControlMethod](@ref primo::avblocks::Param::Encoder::Video::H264::RateControlMethod)					| [ConstantQuant](@ref primo::codecs::H264RateControlMethod::ConstantQuant)			|
                        | [RateControlQuantI](@ref primo::avblocks::Param::Encoder::Video::H264::RateControlQuantI)					| 26																				|
                        | [RateControlQuantP](@ref primo::avblocks::Param::Encoder::Video::H264::RateControlQuantP)					| 26																				|
                        | [DeblockingFilter](@ref primo::avblocks::Param::Encoder::Video::H264::DeblockingFilter)					| [InSlice](@ref primo::codecs::H264DeblockingFilter::InSlice)						|
                        | [MESplitMode](@ref primo::avblocks::Param::Encoder::Video::H264::MESplitMode)								| [Only16x16](@ref primo::codecs::H264MeSplitMode::Only16x16)						|
                        | [MEMethod](@ref primo::avblocks::Param::Encoder::Video::H264::MEMethod)									| [UMH](@ref primo::codecs::H264MeMethod::UMH)										|

                        ### %Audio Pin
                        | Property																									| Value																				|
                        | ----------------------------------------------------														| :-----------:																		|
                        | [StreamType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamType)									| [AAC](@ref primo::codecs::StreamType::AAC)										|
                        | [StreamSubType (StreamInfo)](@ref primo::codecs::StreamInfo::setStreamSubType)							| [AAC_MP4](@ref primo::codecs::StreamSubType::AAC_MP4)								|
                        */
                    }
                }
            }
        }


    /**
        An input or output point of the Transcoder.

        A MediaSocket object describes either a container (WAV, AVI, MP4, etc.) with one or more streams or 
        simply an elementary stream (MP3, M2V, etc.).

        When MediaSocket describes a container the child collection of \link MediaSocket::pins media pins\endlink
        can contain one or more items (for each contained stream).
        The \link MediaSocket::streamType socket stream type\endlink designates the container type.

        When MediaSocket describes an elementary stream the child collection of \link MediaSocket::pins media pins\endlink
        contains a single item which is the elementary stream itself.
        The \link MediaSocket::streamType socket stream type\endlink is the same as the pin stream type.

        @see Library::createMediaSocket
        @see \ref object_management "Object Management"
    */
    class MediaSocket: public primo::Reference
    {
    public:

        /**
            Creates a deep copy of this object.

            @return	A new MediaSocket object.             
			The caller is responsible for releasing the new object when it is not needed anymore.
            
            @return <b>NULL</b> if there is not enough memory to clone the object.

            @remarks
            When cloned an immutable object becomes mutable.

            @see \ref object_management "Object Management"
        */
        virtual MediaSocket* clone() const = 0;
        
        /**
            Sets the file associated with the media socket.

            @param filename [in] a filename as a Unicode zero terminated string. If <b>NULL</b> the filename is cleared. 
            When not <b>NULL</b> the filename is copied internally and is not required after the method returns. 

            @return TRUE if the file name is successfully set, otherwise FALSE.
        */
        virtual bool_t setFile(const char_t* filename) = 0;
        
        /**
            Returns the file associated with the media socket and previously set via setFile().

            @return A filename as a Unicode zero terminated string. Can be <b>NULL</b> if the file is not set. The default value is <b>NULL</b>.
        */
        virtual const char_t* file() const = 0;
        
        /**
            Sets the user stream associated with the media socket. This is an alternative to setFile().
            
            @param stream [in] A pointer to a Stream object. If <b>NULL</b> the stream object is cleared.
            Internally the Stream object is retained by MediaSocket, so it is safe to release it after this method returns.

            When both a file and a user stream are associated with a socket the transcoder prefers the stream.

            @return TRUE if the stream is successfully set, otherwise FALSE.

            @see \ref object_management "Object Management"
        */
        virtual bool_t setStream(primo::Stream* stream) = 0;
        
        /**
            Returns the associated user stream previously set via setStream().

            @return A Stream object. Can be <b>NULL</b> if the associated stream is not set. The default value is <b>NULL</b>.
        */
        virtual primo::Stream* stream() const = 0;

        /**
            Sets explicitly the format of the associated file or stream.
             
             @param streamType [in] stream type such as AVI, WAV, MP4, etc.

             The Transcoder uses this format and the associated file or stream
             according to following rules:
             
             <table>
             <caption>Using File, Stream and Type in MediaSocket</caption>
             <tr>
                <td>File</td>
                <td>Stream</td>
                <td>Type</td>
                <td>How MediaSocket is treated by Transcoder</td>
             </tr>
             <tr>
                <td> Y </td>
                <td> N </td>
                <td> N </td>
                <td> OK - the socket defines a file whose type is implicitly determined by
                the file extension.</td>
             </tr>
             <tr>
                <td> Y </td>
                <td> N </td>
                <td> Y </td>
                <td> OK - the socket defines a file whose type is explicitly specified by
                the stream type.</td>
             </tr>
             <tr>
                <td> Y,N </td>
                <td> Y </td>
                <td> Y </td>
                <td> OK - the socket defines a user stream whose type is explicitly specified.</td>
             </tr>
             <tr>
                <td> Y,N </td>
                <td> Y </td>
                <td> N </td>
                <td> Error - the socket defines a user stream but its type is unknown.</td>
             </tr>
             <tr>
                <td> N </td>
                <td> N </td>
                <td> Y </td>
                <td> OK - the socket does not have a physical storage.
                it only describes the input/output data.
                This is the required definition for input sockets when the push() method is used and
                also the required definition for output sockets when the pull() method is used.
                </td>
             </tr>
             <tr>
                <td> N </td>
                <td> N </td>
                <td> N </td>
                <td> Error - undefined socket.
                </td>
             </tr>
             </table>

             @return TRUE if the stream type is successfully set, otherwise FALSE.
        */
        virtual bool_t setStreamType(primo::codecs::StreamType::Enum streamType) = 0;

        /**
            Returns the stream type of the associated file or user stream.

            @return stream type. If not set \link primo::codecs::StreamType::Unknown StreamType::Unknown\endlink is returned.
        */
        virtual primo::codecs::StreamType::Enum streamType() const = 0;

        /**
            Sets explicitly the stream subtype of the associated file or user stream.

            @return TRUE if the stream subtype is successfully set, otherwise FALSE.
        */
        virtual bool_t setStreamSubType(primo::codecs::StreamSubType::Enum streamSubType) = 0;

        /**
            Returns the stream subtype of the associated file or user stream.

            @return stream subtype. If not set \link primo::codecs::StreamSubType::Unknown StreamSubType::Unknown\endlink is returned.
        */
        virtual primo::codecs::StreamSubType::Enum streamSubType() const = 0;
        
        /**
            Returns a collection of socket parameters.
            
            These parameters are used when splitting the input or multiplexing the output defined by this media socket.

            @return A ParameterList object. Can be <b>NULL</b> if the collection is not set. The default value is an empty list.

            The caller is not required to release the returned object.
            
            @see \ref object_management "Object Management"
        */
        virtual ParameterList* params() const = 0;

        /**
            Sets the socket parameters.

            @param params [in] A pointer to a ParameterList object. If <b>NULL</b> the stored collection is reset to <b>NULL</b>.
            Internally the ParameterList object is retained by MediaSocket, so it is safe to release it after this method returns.

            @see \ref object_management "Object Management"
        */
        virtual bool_t setParams(ParameterList* params) = 0;

        /**
            Returns a modifiable collection with all the pins (elementary streams) that are defined for this socket.

            @return A collection of MediaPin objects. 
            The default value is an empty collection which can be modified but it cannot be replaced.
            
            The returned collection exists only in the context of MediaSocket.
            The caller cannot influence the lifetime of the returned object.
        */
        virtual MediaPinList* pins() const = 0;

        /**
            Sets the socket time position.

            This property can be used on an input socket to start reading / decoding from a specific point in time.
            The transcoder tries to seek in the input socket to the time position before the transcoding operation starts.

            Seeking is not possible while transcoder is open. %Transcoder need to be closed before setting new time possition.

            @param time [in] Time position in seconds.

            @return TRUE if the time position is successfully set, otherwise FALSE.
        */
        virtual bool_t setTimePosition(double time) = 0;

        /**
            Returns the socket time position.
            
            This property can be used on an input socket to start reading / decoding from a specific point in time.
            The transcoder tries to seek in the input socket to the time position before the transcoding operation starts.

            @return Time position in seconds.
        */
        virtual double timePosition() const = 0;

        /**
            Returns the socket metadata.

            @return A pointer to a Metadata object. Can be <b>NULL</b> if the socket has no metadata.
        */
        virtual primo::codecs::Metadata* metadata() const = 0;

        /**
            Sets the socket metadata.

            When an output socket has metadata the transcoder tries to generate it regardless of the input metadata.
            When an output socket has no metadata (<b>NULL</b>) but an input socket has metadata (not <b>NULL</b>)
            the transcoder uses the input metadata as if it is set for the output.

            @param metadata [in] A pointer to a Metadata object. Can be <b>NULL</b> in which case the metadata is cleared from the socket.

            @return TRUE if the metadata is successfully set, otherwise FALSE.
        */
        virtual bool_t setMetadata(primo::codecs::Metadata* metadata) = 0;

        /**
            Returns whether the object is immutable.
            An immmutable object cannot be modified and all modifying methods return FALSE.

            @return TRUE if the object is immutable, FALSE otherwise.

            @remarks
            An immutable object <b>can</b> be modified by the AVBlocks library.

            @remarks
            Object immutability spreads to all nested objects.
            Therefore it is not possible to add/set an immutable object to a mutable object.

            @remarks
            When cloned an immutable object becomes mutable.
        */
        virtual bool_t immutable() const = 0;

    protected:
        /**
            This object can be destroyed only by the AVBlocks library.
        */
        ~MediaSocket() {}
    };

    /**
        MediaSocketList represents the inputs or outputs of a Transcoder.
        It contains one or more media sockets (MediaSocket objects).
        The collection is modifiable and indexable.
        It exists only in the context of a Transcoder and cannot be created separately from it,
        nor detached.
    */
    class MediaSocketList
    {
    public:
        /**
            Adds a socket to the collection.

            @param item [in] A pointer to a MediaSocket object that should be added to the collection.
            It is allowed to add a <b>NULL</b> value to the collection.
            Internally the MediaSocket object is retained (if not <b>NULL</b>), so it is safe to release it 
            after this method returns.

            @return TRUE if the specified item is added to the collection, otherwise FALSE.

            @see Library::createMediaSocket
            @see \ref object_management "Object Management"
        */
        virtual bool_t add(MediaSocket* item) = 0;

        /**
            Removes a socket from the collection.

            @param itemIndex [in] The index of the socket that should be removed from the collection.
            Indexing starts from 0.

            @return TRUE if the specified item is removed from the collection, otherwise FALSE.

            When a MediaSocket object is removed from the collection it is also released.

            @see \ref object_management "Object Management"
        */
        virtual bool_t remove(int32_t itemIndex) = 0;

        /**
            Removes all sockets from the collection.

            When removed from the collection each MediaSocket object is released.

            @return TRUE if the collection is cleared, otherwise FALSE.

            @see \ref object_management "Object Management"
        */
        virtual bool_t clear() = 0;

        /**
            Returns the number of sockets in the collection.

            @return number of items in the collection.
        */
        virtual int32_t count() const = 0;

        /**
            Returns a media socket specified by index.

            @param itemIndex [in] The index of the socket to be returned. Indexing starts from 0.

            @return A pointer to the MediaSocket object specified by <i>itemIndex</i>. Can be <b>NULL</b>.

            The caller is not required to release the returned object.
            
            @see \ref object_management "Object Management"
        */
        virtual MediaSocket* at(int32_t itemIndex) const = 0;

        /**
            Sets a new socket at the specified index.

            @param itemIndex [in] The index in the collection where the new item shall be set.
            This must be a valid index in the collection. Indexing starts from 0.

            @param item [in] A pointer to the MediaSocket object that shall be set.
            It is allowed to set a <b>NULL</b> value to the collection.
            Internally the new socket is retained (if not <b>NULL</b>), so it is safe to release it 
            after this method returns. The old socket is released (if not <b>NULL</b>).

            @return TRUE The item is set, FALSE otherwise.
            
            @see \ref object_management "Object Management"
        */
        virtual bool_t setAt(int32_t itemIndex, MediaSocket* item) = 0;

        /**
            Inserts a socket at the specified index.

            @param itemIndex [in] The index in the collection where the new item shall be inserted.
            This must be a valid index in the collection. Indexing starts from 0.
            After the new item is inserted in the collection its index is the value of the <i>itemIndex</i> param.

            @param item [in] A pointer to the MediaSocket object that shall be inserted.
            It is allowed to insert a <b>NULL</b> value in the collection.
            Internally the new socket is retained (if not <b>NULL</b>), so it is safe to release it 
            after this method returns.

            @return TRUE The item is set, FALSE otherwise.
            
            @see \ref object_management "Object Management"
        */
        virtual bool_t insert(int32_t itemIndex, MediaSocket* item) = 0;

        /**
            Returns whether the object is immutable.
            An immmutable object cannot be modified and all modifying methods fail to produce a result.

            @return TRUE if the object is immutable, FALSE otherwise.

            @remarks
            An immutable object <b>can</b> be modified by the AVBlocks library.

            @remarks
            Object immutability spreads to all nested objects.
            Therefore it is not possible to add/set an immutable object to a mutable object.
        */
        virtual bool_t immutable() const = 0;

    protected:
        /**
            This object can be destroyed only by the AVBlocks library.
        */
        ~MediaSocketList() {}
    };

    /**
        Defines the status of a transcoder operation.
        
        It is sent with the TranscoderCallback::onStatus notification.
    */
    namespace TranscoderStatus
    {
        /**
            TranscoderStatus::Enum
        */
        enum Enum
        {
            /** Transcoder::run has completed successfully.  */
            Completed = 0,
        };
    }

    /**
        The TranscoderCallback provides a way to receive notifications from the Transcoder and also to control its operation 
        while it is running (Transcoder::run).

        This class is meant to be inherited by the user code and only the required notifications/methods can be overridden.

        When a TranscoderCallback instance is set (Transcoder::setCallback) it is retained by the Transcoder.
        The TranscoderCallback instance is released when the Transcoder itself is being destroyed.
        
        The default implementation of primo::Reference keeps a constant reference count of 1.
        This can be used when the user code is the sole owner of the implemented callback and does not need 
        to manage it through the primo::Reference interface.

        The TranscoderCallback is used only when the transcoding is performed via Transcoder::run.

        @see \ref object_management "Object Management"

    */
    class TranscoderCallback : public primo::Reference
    {
    public:

        /**
            This method is called periodically by the Transcoder to report transcoding progress. See Transcoder::run.
            
            @param currentTime [in] The duration of the media stream that has been processed so far. Time is in seconds.
            
            @param totalTime [in] The total duration of the media stream that will be processed eventually. Time is in seconds.

            For performance reasons this notification is not called for every processed media sample. 
            Currently it is called once for every 0.5 seconds of processed media data.
            This notification is intended for user interface updates.

            The transcoding progress in percents is given by: (currentTime / totalTime * 100).

            The Transcoder does not guarantee that currentTime will ever reach totalTime 
            even when the transcoding completes successfully.
        */
        virtual void onProgress(double currentTime, double totalTime)
        {
        }

        /**
            This method is called by the Transcoder to report the status of current operation.

            @param status [in] Transcoder status.

            @see TranscoderStatus::Enum
        */
        virtual void onStatus(TranscoderStatus::Enum status)
        {
        }

        /**
            This method is called by the Transcoder to give the user code a chance to stop the transcoding process 
            started by Transcoder::run().

            @param currentTime [in] The duration of the media stream that has been processed so far. Time is in seconds.

            @return TRUE if the Transcoder should continue running. FALSE if the Transcoder should stop.

            When instructed the Transcoder stops the transcoding process gracefully so that 
            the media data processed is valid (playable).

            This notification is called as often as possible so that the user code may stop the transcoding process as soon as it is required.
            For this reason there should be minimal processing in this method if it is overridden by the user code, 
            otherwise the overall transcoding may be slowed down.
        */
        virtual bool_t onContinue(double currentTime)
        {
            return 1;
        }

        /**
            This method is called by the Transcoder when an input format has been detected or has changed.
            The user code has a chance to inspect detected inputs, re-configure outputs, connect or disable pins.

            @param inputIndex [in] The index of the input socket that has changed. 
            The socket contains pins that describe the new format.

            Currently this notification is used by the Transcoder when an input socket is configured without pins
            and the input stream is pushed via Transcoder::push().
            The notification is invoked when the input format is detected and after the appropriate pins are added to the input socket.
            The user code may inspect the input pins and can decide which pins to connect (via MediaPin::setConnection()).
            It may also add or remove output sockets and pins.
            The Transcoder is fully initialized only after all inputs have been detected and the user code returns from this method.
        */
        virtual void onInputChange(int32_t inputIndex)
        {
        }

        /**
            The default implementation keeps a constant reference count of 1.
            
            @return 1
        */
        int32_t retain() const		{ return 1; }

        /**
            The default implementation keeps a constant reference count of 1.

            @return 1
        */
        int32_t release() const		{ return 1; }

        /**
            The default implementation keeps a constant reference count of 1.

            @return 1
        */
        int32_t retainCount() const	{ return 1; }
    };

    /**
        A Block is a unit that can produce, consume or transform media samples from/to multiple inputs/outputs
    */
    class Block : public primo::Reference
    {
        public:
            
        /**
        Returns a modifiable collection of MediaSocket objects which
        describe the input data of the block.

        @return A pointer to a collection of input sockets. Cannot be <b>NULL</b>.
        */
        virtual MediaSocketList* inputs() const = 0;

        /**
        Returns a modifiable collection of MediaSocket objects which
        describe the output data of the block.

        @return A pointer to a collection of output sockets. Cannot be <b>NULL</b>.
        */
        virtual MediaSocketList* outputs() const = 0;

        /**
        Initializes the block based on the specified inputs and outputs.

        @return TRU The block is successfully initialized and is ready to process data, otherwise FALSE.
        */
        virtual bool_t open() = 0;

        /**
        Pushes input data to the block.

        @param inputIndex [in] Specifies the index of the input socket whose data is pushed to the block.
        
        @param inputSample [in] A pointer to a MediaSample object that contains the input data.
        
        @return TRUE The block has successfully processed the input data, otherwise FALSE.
        */
        virtual bool_t push(int32_t inputIndex, primo::codecs::MediaSample* inputSample) = 0;

        /**
        Pulls output data from the block.

        @param outputIndex [out] contains the index of the output socket to which the data belongs.

        @param outputData [out] A pointer to a MediaSample object that receives the output data.
        
        @return TRUE The block has successfully generated output, otherwise FALSE.
        */
        virtual bool_t pull(int32_t& outputIndex, primo::codecs::MediaSample* outputData) = 0;
      
        /**
        Closes the block. When closed it can neither accept, nor deliver data.
        */
        virtual void close() = 0;

        /**
        Flushes the data buffered in the block to the output.
        This method can be used only in push mode.

        @return TRUE if the buffered data is successfully flushed, otherwise FALSE.

        @remarks This method also marks the end-of-stream for all inputs.
        No data should be pushed to the block after it is flushed.
        */
        virtual bool_t flush() = 0;

        /**
        Tells the block that there's no more data for the specified input socket.
        This method can be used only in push mode.

        @param inputIndex [in] specifies the index of the input socket for which there's no more data.

        @return TRUE if the operation is successful, otherwise FALSE.
        */
        virtual bool_t endOfStream(int32_t inputIndex) = 0;

        /**
        Returns the error information for the last operation.

        @return A pointer to an ErrorInfo object.
        */
        virtual const primo::error::ErrorInfo* error() const = 0;
    };


    /**
    Analyzes a file or a stream and provides information about its type and properties.
    */
    class MediaInfo: public Block
    {
    public:
        /**
        Returns a collection of StreamInfo objects describing the audio and video streams (tracks) found in the input.

        @return A read-only collection of StreamInfo objects.
        Each StreamInfo object in the collection represents a stream (track) in the analyzed input.

        Normally the caller is not required and should not release the returned object.
        However it is allowed to retain the collection and it will be valid even
        when the MediaInfo instance is destroyed.
        In all cases when the caller retains the returned collection it should release it when it is not needed anymore.

        @deprecated Please use primo::avblocks::MediaInfo::outputs instead.

        @see \ref object_management "Object Management"
        */
        PRIMO_DEPRECATED("primo::avblocks::MediaInfo::streams is deprecated. Please use primo::avblocks::MediaInfo::outputs.")
        virtual primo::codecs::StreamInfoEnum* streams() const = 0;

        /**
        Sets the input that should be analyzed.

        @param filename [in] a filename as an Unicode zero terminated string. If <b>NULL</b> the filename is cleared.
        When not <b>NULL</b> the filename is copied internally and is not required to be valid after the method returns.

        @deprecated Please use primo::avblocks::MediaInfo::inputs instead.
        */
        PRIMO_DEPRECATED("primo::avblocks::MediaInfo::setInputFile is deprecated. Please use primo::avblocks::MediaInfo::inputs.")
        virtual void setInputFile(const char_t* filename) = 0;

        /**
        Returns the input file previously set via setInputFile().

        @return A filename as an Unicode zero terminated string. Can be <b>NULL</b> if the source file is not set.

        @deprecated Please use primo::avblocks::MediaInfo::inputs instead.
        */
        PRIMO_DEPRECATED("primo::avblocks::MediaInfo::inputFile is deprecated. Please use primo::avblocks::MediaInfo::inputs.")
        virtual const char_t* inputFile() const = 0;

        /**
        Sets the input stream that should be analyzed. This is an alternative to setInputFile().

        @param stream [in] A pointer to a \link primo::Stream Stream\endlink object. If <b>NULL</b> the input stream is cleared.
        Internally the \link primo::Stream Stream\endlink object is retained by MediaInfo, so it is safe to release it after this method returns.

        MediaInfo will use the input file when both an input file and input stream are set.

        @deprecated Please use primo::avblocks::MediaInfo::inputs instead.

        @see \ref object_management "Object Management"
        */
        PRIMO_DEPRECATED("primo::avblocks::MediaInfo::setInputStream is deprecated. Please use primo::avblocks::MediaInfo::inputs.")
        virtual void setInputStream(primo::Stream* stream) = 0;

        /**
        Returns the input stream previously set via setInputStream().

        @return A Stream object. Can be <b>NULL</b> if the source stream is not set.

        @deprecated Please use primo::avblocks::MediaInfo::inputs instead.
        */
        PRIMO_DEPRECATED("primo::avblocks::MediaInfo::inputStream is deprecated. Please use primo::avblocks::MediaInfo::inputs.")
        virtual primo::Stream* inputStream() const = 0;

        /**
        Forces a specific stream type.

        @param streamType [in] stream type such as AVI, WAV, MP4, etc.

        If the input is a file and the stream type is not set, the stream type is deduced from the filename extension.
        If the input is a stream, MediaInfo requires the stream type to be specified.
        If the stream type is set to something other than \link primo::codecs::StreamType::Unknown StreamType::Unknown\endlink,
        it takes precedence and is used by MediaInfo regardless of the detected stream type.

        @deprecated Please use primo::avblocks::MediaInfo::inputs instead.
        */
        PRIMO_DEPRECATED("primo::avblocks::MediaInfo::setStreamType is deprecated. Please use primo::avblocks::MediaInfo::inputs.")
        virtual void setStreamType(primo::codecs::StreamType::Enum streamType) = 0;

        /**
        Returns the stream type previously set via setStreamType() or the one detected by MediaInfo.

        @return input stream type. The default value is \link primo::codecs::StreamType::Unknown StreamType::Unknown\endlink.

        @deprecated Please use primo::avblocks::MediaInfo::inputs instead.
        */
        PRIMO_DEPRECATED("primo::avblocks::MediaInfo::streamType is deprecated. Please use primo::avblocks::MediaInfo::inputs.")
        virtual primo::codecs::StreamType::Enum streamType() const = 0;

        /**
        Returns the stream subtype detected by MediaInfo.

        @return input stream subtype. The default value is \link primo::codecs::StreamSubType::Unknown StreamSubType::Unknown\endlink.

        @deprecated Please use primo::avblocks::MediaInfo::inputs instead.
        */
        PRIMO_DEPRECATED("primo::avblocks::MediaInfo::streamSubType is deprecated. Please use primo::avblocks::MediaInfo::inputs.")
        virtual primo::codecs::StreamSubType::Enum streamSubType() const = 0;

        /**
        Analyzes the input specified by setInputFile() or setInputStream() and loads the information in the MediaInfo object.

        @return TRUE if the input is successfully analyzed, otherwise FALSE.

        If this method is successful the information about the detected media streams (tracks) and metadata can be obtained from MediaInfo.

        @deprecated Please use primo::avblocks::MediaInfo::open instead.

        @see MediaInfo::streams
        @see MediaInfo::metadata
        */
        PRIMO_DEPRECATED("primo::avblocks::MediaInfo::load is deprecated. Please use primo::avblocks::MediaInfo::open.")
        virtual bool_t load() = 0;

        /**
        Returns the metadata for the input.

        @return A pointer to a Metadata object. Can be <b>NULL</b> if the input has no metadata.

        Normally the caller is not required and should not release the returned object.
        However it is allowed to retain the Metadata object and it will be valid even
        when the MediaInfo instance is destroyed.
        In all cases when the caller retains the returned object it should release it when it is not needed anymore.

        @deprecated Please use primo::avblocks::MediaInfo::outputs instead.

        @see \ref object_management "Object Management"
        */
        PRIMO_DEPRECATED("primo::avblocks::MediaInfo::metadata is deprecated. Please use primo::avblocks::MediaInfo::outputs.")
        virtual primo::codecs::Metadata* metadata() const = 0;

        /**
        Indicates whether stream info and metadata have been loaded and parsed.
        @see \ref primo::avblocks::MediaInfo::push "MediaInfo::push"
        */
        virtual bool_t isReady() = 0;

        /**
        Pushes input data to the <b>MediaInfo</b>.

        @param inputIndex [in] Specifies the index of the input socket whose data is pushed to the MediaInfo. Must be zero.

        @param inputSample [in] A pointer to a MediaSample object that contains the input data.

        @return TRUE The <b>MediaInfo</b> has successfully processed the input data, otherwise FALSE.

        @remarks In order to use this method the inputs()->at(0)->streamType() must be set and open() must be called.

        @see MediaInfo::isReady

        @note
        <table border="0">
        <tr>
        <td>Supported stream types:</td>
        <td>\link primo::codecs::StreamType::MPEG1_Video StreamType::MPEG1_Video\endlink</td>
        </tr>
        <tr>
        <td></td>
        <td>\link primo::codecs::StreamType::MPEG2_Video StreamType::MPEG2_Video\endlink</td>
        </tr>
        <tr>
        <td></td>
        <td>\link primo::codecs::StreamType::H264 StreamType::H264\endlink</td>
        </tr>
        <tr>
        <td></td>
        <td>\link primo::codecs::StreamType::MPEG_TS StreamType::MPEG_TS\endlink</td>
        </tr>
        </table>
        */
        virtual bool_t push(int32_t inputIndex, primo::codecs::MediaSample* inputSample) = 0;

        /**
        Returns a modifiable collection of MediaSocket objects which describe the input data of the MediaInfo.
        The collection contains one default MediaSocket that must be configured.

        Each socket in the collection represents an input point -
        it can be a container with one or more streams or just an elementary stream.

        @return A pointer to a collection of input sockets. Cannot be <b>NULL</b>.
        */
        virtual MediaSocketList* inputs() const = 0;


        /**
        Returns an immutable collection of MediaSocket objects describing the audio and video streams (tracks) found in the input.

        @return A pointer to a collection of output sockets. Cannot be <b>NULL</b>.
        */
        virtual MediaSocketList* outputs() const = 0;

        /**
        Analyzes the input specified by inputs() and loads the information in the MediaInfo object.

        @return TRUE if the input is successfully analyzed, otherwise FALSE.

        If this method succeeds the information about the detected media streams (tracks) and metadata can be obtained from MediaInfo.
        */
        virtual bool_t open() = 0;

    protected:
        /**
        This object can be destroyed only by the AVBlocks library.
        */
        ~MediaInfo() {}
    };


    /**
        Provides functionality for audio and video encoding, decoding and transforming.
    */
    class Transcoder : public primo::avblocks::Block
    {
        public:
        
        /**
            Returns a modifiable collection of MediaSocket objects which
            describe the input data of the transcoder.

            Each socket in the collection represents an input point -
            it can be a container with one or more streams or just an elementary stream.

            @return A pointer to a collection of input sockets. Cannot be <b>NULL</b>.

            The returned collection exists only in the context of the transcoder.
            The caller cannot influence the lifetime of the returned object.
        */
        virtual MediaSocketList* inputs() const = 0;

        /**
            Returns a modifiable collection of MediaSocket objects which
            describe the output data of the transcoder.

            Each socket in the collection describes an output point -
            it can be a container with one or more streams or just an elementary stream.

            @return A pointer to a collection of output sockets. Cannot be <b>NULL</b>.

            The returned collection exists only in the context of transcoder.
            The caller cannot influence the lifetime of the returned object.
        */
        virtual MediaSocketList* outputs() const = 0;

        /**
            Initializes the transcoder based on the specified inputs and outputs.
            If this method succeeds the transcoder is ready to process data.
            The data flow may be driven by the following methods:
            <ul>
            <li>push() - The caller pushes input data to the transcoder.
            The transcoder writes the output data to a file or user stream.
            </li>
            <li>pull() - The caller pulls output data from the transcoder.
            The transcoder reads the input data from a file or user stream.
            </li>
            <li>push() and pull() - The caller pushes input data to the transcoder.
            It also pulls the output data from the transcoder.
            </li>
            <li>run() - The caller waits for the transcoding to finish.
            The transcoder automatically reads the input data from a file or user stream
            and writes the output to a file or user stream.
            The process runs until the input data is depleted or there's a processing error.
            </li>
            </ul>
            However not all stream types can be used in push or pull mode. See \ref transcoder_io for more info.

            @return <table>
            <tr>
            <td>TRUE</td> <td>The block is successfully initialized and is ready to process data.</td>
            </tr>
            <tr>
            <td>FALSE</td> <td>The block cannot process the specified input and output data.</td>
            </tr>
            </table>
        */
        virtual bool_t open() = 0;

        /**
            Closes the transcoder. When closed it can neither accept, nor deliver data.
            Inputs and outputs are not modified.
        */
        virtual void close() = 0;

        /**
            Flushes the data buffered in the transcoder to the output.
            This method can be used only in push mode.

            @return TRUE if the buffered data is successfully flushed, otherwise FALSE.

            @remarks This method also marks the end-of-stream for all inputs.
            No data should be pushed to the transcoder after it is flushed.
        */
        virtual bool_t flush() = 0;

        /**
            Tells the transcoder that there's no more data for the specified input socket.
            If the last input socket is stopped the transcoder flushes its internal buffers to the output.
            This method can be used only in push mode.

            @param inputIndex [in] specifies the index of the input socket for which there's no more data.

            @return TRUE if the operation is successful, otherwise FALSE.
        */
        virtual bool_t endOfStream(int32_t inputIndex) = 0;

        /**
            Sets the autoConnect property of the transcoder.
            If autoConnect is TRUE the transcoder automatically maps input to output pins when opened.

            @param autoConnect [in] specifies to auto connect input and output pins.
        */
        virtual void setAutoConnect(bool_t autoConnect) = 0;

        /**
            Returns the autoConnect property of the transcoder.

            @return TRUE the transcoder automatically maps input to output pins when opened.
        */
        virtual bool_t autoConnect() const = 0;
       
        /**
            Runs an automatic transcoding.

            @return TRUE The transcoding has succeeded. FALSE There's a transcoding error
            or the run() method cannot be used with the specified inputs and outputs.

            In order to use the run mode all input and output sockets must be associated with files or user streams.
            The method returns when the transcoding is finished because there's no more input data to process or
            when there's an error.

            @note
            The input socket can specify an image file in which case the output video contains a single frame.
            A single frame video (MPEG-2) can be used as a background in DVD-Video authoring.
            For generating a custom video from many images see Transcoder::push.
        */
        virtual bool_t run() = 0;

        // WARNING: The `Supported stream types` table must be under the `push` and `pull` methods below. 
        // The documentation gets ruined if there is a comment between the table and the method
        /**
            Pushes input data to the Transcoder.
            
            @param inputIndex [in] Specifies the index of the input socket whose data is pushed to the Transcoder.
            
            @param inputSample [in] A pointer to a MediaSample object that contains the input data.
            The MediaSample provided is used only within this method and the object is not retained by the Transcoder.
            It is not guaranteed that the whole input will be consumed (read) by the Transcoder after this method has returned.
            It's possible that the Transcoder may consume only part of the provided input data.
            When this method returns successfully 
            the \link primo::codecs::MediaBuffer::dataSize MediaBuffer::dataSize\endlink and
            the \link primo::codecs::MediaBuffer::dataOffset MediaBuffer::dataOffset\endlink
            are updated to reflect how much data is left and where it starts in the buffer.
            This must be taken into consideration if more data should be appended for subsequent pushing.
            
            @return TRUE The Transcoder has successfully processed some or all of the input data, otherwise FALSE.
            
            @remarks In order to use this method the input socket must not have an associated file or user stream.
            If the transcoder operates in the hybrid push/pull mode it will return 
            \ref primo::avblocks::TranscoderError::InputFull "TranscoderError::InputFull" when it cannot accept the input.
            In this case Transcoder::pull() must be called to retrieve the available output.
            
            @note
            The input sample can contain an image like bmp, png or jpeg.
            This makes it possible to create a video from one or more images with any duration.
            The start time of each input sample must be set explicitly and must increase for each successive sample.

            @warning
            This method is not thread-safe (as all other functions in AVBlocks). It cannot be called simultaneously from multiple threads.
            You must implement your own synchronization if the method will be called from multiple threads.
        */
        virtual bool_t push(int32_t inputIndex, primo::codecs::MediaSample* inputSample) = 0;
        /*!<
            |    Supported %Stream Types                                                                |    %MediaSample format                               |
            |:---------------------------                                                               |:---------------------------                          |
            | \link primo::codecs::StreamType::LPCM StreamType::LPCM\endlink                            |                                                      |
            | \link primo::codecs::StreamType::ALAW_PCM StreamType::ALAW_PCM\endlink                    |                                                      |
            | \link primo::codecs::StreamType::MULAW_PCM StreamType::MULAW_PCM\endlink                  |                                                      |
            | \link primo::codecs::StreamType::G726_ADPCM StreamType::G726_ADPCM\endlink                |                                                      |
            | \link primo::codecs::StreamType::UncompressedVideo StreamType::UncompressedVideo\endlink  |                                                      |
            | \link primo::codecs::StreamType::MPEG1_Video StreamType::MPEG1_Video\endlink              |                                                      |
            | \link primo::codecs::StreamType::MPEG2_Video StreamType::MPEG2_Video\endlink              |                                                      |
            | \link primo::codecs::StreamType::MPEG4_Video StreamType::MPEG4_Video\endlink              |                                                      |
            | \link primo::codecs::StreamType::H264 StreamType::H264\endlink                            |                                                      |
            | \link primo::codecs::StreamType::MJPEG StreamType::MJPEG\endlink                          |                                                      |
            | \link primo::codecs::StreamType::MPEG_TS StreamType::MPEG_TS\endlink                      |                                                      |
            | \link primo::codecs::StreamType::BMP StreamType::BMP\endlink                              |                                                      |
            | \link primo::codecs::StreamType::PNG StreamType::PNG\endlink                              |                                                      |
            | \link primo::codecs::StreamType::JPEG StreamType::JPEG\endlink                            | See [Pushing JPEG](@ref using_transcoder_push_jpeg)  |
            | \link primo::codecs::StreamType::TIFF StreamType::TIFF\endlink                            |                                               |
            | \link primo::codecs::StreamType::GIF StreamType::GIF\endlink                              |                                               |
        */

        /**
            Pulls output data from the transcoder.
            
            @param outputIndex [out] contains the index of the output socket to which the data belongs.
            
            @param outputData [out] A pointer to a MediaSample object that receives the output data.
            When the method returns successfully the <i>outputData</i> contains the output sample.

            Each time output data is generated a new MediaBuffer instance is created and attached to <i>outputData</i>.
            The new MediaBuffer instance belongs to the caller and must be released when it is not needed anymore.

            @return TRUE The transcoder has successfully generated output, otherwise FALSE.

            @remarks In order to use the pull mode the output sockets must not have an associated file or user stream.
            If the transcoder operates in the hybrid push/pull mode it will return 
            \ref primo::avblocks::TranscoderError::InputNeeded "TranscoderError::InputNeeded" when there's no available output.
            In this case Transcoder::push() must be called to supply input data to the transcoder.

            @warning
            This method is not thread-safe (as all other functions in AVBlocks). It cannot be called simultaneously from multiple threads.
            You must implement your own synchronization if the method will be called from multiple threads.
        */
        virtual bool_t pull(int32_t& outputIndex, primo::codecs::MediaSample* outputData) = 0;
        /*!<
            |    Supported Stream Types                                                                 |
			|:---------------------------                                                               |
            | \link primo::codecs::StreamType::LPCM StreamType::LPCM\endlink                            | 
            | \link primo::codecs::StreamType::ALAW_PCM StreamType::ALAW_PCM\endlink                    | 
            | \link primo::codecs::StreamType::MULAW_PCM StreamType::MULAW_PCM\endlink                  |
            | \link primo::codecs::StreamType::G726_ADPCM StreamType::G726_ADPCM\endlink                |
            | \link primo::codecs::StreamType::UncompressedVideo StreamType::UncompressedVideo\endlink  |
            | \link primo::codecs::StreamType::MPEG1_Video StreamType::MPEG1_Video\endlink              |
            | \link primo::codecs::StreamType::MPEG2_Video StreamType::MPEG2_Video\endlink              |
            | \link primo::codecs::StreamType::MPEG4_Video StreamType::MPEG4_Video\endlink              |
            | \link primo::codecs::StreamType::H264 StreamType::H264\endlink                            |
            | \link primo::codecs::StreamType::MPEG_TS StreamType::MPEG_TS\endlink                      |
            | \link primo::codecs::StreamType::MPEG_PSI_PACKETS StreamType::MPEG_PSI_PACKETS\endlink    |
            | \link primo::codecs::StreamType::MPEG_TS_PACKETS StreamType::MPEG_TS_PACKETS\endlink      |
            | \link primo::codecs::StreamType::WAVE StreamType::WAVE\endlink                            |
        */

        /**
            Sets a TranscoderCallback instance which is used to receive notifications from the transcoder.
            The TranscoderCallback is used only when the transcoding is performed via Transcoder::run.

            @param callback [in] A pointer to TranscoderCallback object that will be invoked by the transcoder while running.
            It is allowed to set a <b>NULL</b> object.
            
            Internally the TranscoderCallback specified by <i>setCallback</i> is retained (if not <b>NULL</b>).
            The previous TranscoderCallback is released (if not <b>NULL</b>).
            The TranscoderCallback instance is released when the Transcoder itself is being destroyed.

            @note
            If the user code inherits from TranscoderCallback but does not override primo::Reference::release() and
            primo::Reference::retain() it must ensure that the callback instance is valid until it is used by the transcoder.
            This is so because the default implementation of primo::Reference in TranscoderCallback keeps a constant reference count of 1.

            @see TranscoderCallback
        */
        virtual void setCallback(primo::avblocks::TranscoderCallback* callback) = 0;

        /**
            Returns the TranscoderCallback object that is used by the Transcoder to send notifications and events while running.

            @return A pointer to TranscoderCallback object or <b>NULL</b> if the callback is not set.
        */
        virtual primo::avblocks::TranscoderCallback* callback() = 0;
        
        
        /**
            Returns the error information for the last operation.

            @return A pointer to an ErrorInfo object.
        */
        virtual const primo::error::ErrorInfo* error() const = 0;

        /**
            Sets whether demo mode (unlicensed transcoding) should be allowed by the Transcoder.

            @param allowDemoMode [in] specifies whether unlicensed transcoding is allowed.
            
            @remarks The demo mode is enabled by default in the Demo release but not in the OEM release of AVBlocks.
            Therefore in order to test transcoding with the OEM release without a valid license the transcoder demo mode must be enabled explicitly.
        */
        virtual void setAllowDemoMode(bool_t allowDemoMode) = 0;
        
        /**
            Returns whether the Transcoder allows demo mode (unlicensed transcoding).

            @return If TRUE a demo overlay (watermark) is applied when a required codec is not licensed.
            If FALSE the Transcoder returns an error (AVBlocksError::UnlicensedFeature) instead of applying the demo overlay 
            when a required codec is not licensed.
            
            @remarks The default value is TRUE in the Demo release and FALSE in the OEM release.
        */
        virtual bool_t allowDemoMode() const = 0;

    protected:
        
        /**
            This object can be destroyed only by the AVBlocks library. The caller must use Reference::release.

            @see \ref object_management "Object Management"
        */
        ~Transcoder() {}
    };

    /**
        Global hardware configuration
    */
    class HwConfig
    {
    public:

        /**
            Returns whether Intel Quick Sync Video is enabled.

            @return TRUE if Intel Quick Sync Video is enabled, FALSE otherwise
        */
        virtual bool_t intelMedia() const = 0;

        /**
            Enables/disables use of Intel Quick Sync Video.

            @param enable [in] Specifies whether to enable/disable the hardware codec.
        */
        virtual void setIntelMedia(bool_t enable) = 0;

        /**
            Returns whether nVidia NVENC is enabled.

            @return TRUE if nVidia NVENC is enabled, FALSE otherwise
        */
        virtual bool_t nvenc() const = 0;

        /**
            Enables/disables use of nVidia NVENC.

            @param enable [in] Specifies whether to enable/disable the hardware codec.
        */
        virtual void setNvenc(bool_t enable) = 0;

        /**
            Returns whether AMD Video Coding Engine is enabled through OpenVideo.

            @return TRUE if AMD VCE/OpenVideo is enabled, FALSE otherwise
        */
        virtual bool_t amdOpenVideo() const = 0;

        /**
            Enables/disables use of AMD Video Coding Engine through OpenVideo.

            @param enable [in] Specifies whether to enable/disable the hardware codec.
        */
        virtual void setAmdOpenVideo(bool_t enable) = 0;

        /**
            Returns whether AMD Video Coding Engine is enabled through Windows Media Foundation.

            @return TRUE if AMD VCE/MFT is enabled, FALSE otherwise
        */
        virtual bool_t amdMft() const = 0;

        /**
            Enables/disables use of AMD Video Coding Engine through Windows Media Foundation.

            @param enable [in] Specifies whether to enable/disable the hardware codec.
        */
        virtual void setAmdMft(bool_t enable) = 0;

        /**
            Resets hardware configuration settings to the default configuration.
        */
        virtual void resetDefault() = 0;

    protected:

        /**
            This object can be destroyed only by the AVBlocks library.
        */
        ~HwConfig() {}
    };

    /**
        Global configuration
    */
    class Config
    {
    public:
        
        /**
            Returns an object that allows to configure hardware codecs.

            @return A pointer to a global hardware configuration object. The pointer is never <b>NULL</b>.

            @remarks The hardware configuration object is thread-safe.
        */
        virtual HwConfig* hardware() = 0;

    protected:
    
        /**
            This object can be destroyed only by the AVBlocks library.
        */
        ~Config() {}
    };

    #pragma pack(pop)
}} // primo::avblocks


/*
* Exported functions
*/
#ifdef __cplusplus
extern "C" {
#endif

    AVBLOCKS_API bool_t AVBLOCKS_CALL avb_initialize();
    AVBLOCKS_API bool_t AVBLOCKS_CALL avb_is_initialized();
    AVBLOCKS_API void AVBLOCKS_CALL avb_shutdown();
    AVBLOCKS_API int32_t AVBLOCKS_CALL avb_get_major_version();
    AVBLOCKS_API int32_t AVBLOCKS_CALL avb_get_minor_version();
    AVBLOCKS_API int32_t AVBLOCKS_CALL avb_get_patch_version();
    AVBLOCKS_API const char* AVBLOCKS_CALL avb_get_description();
    AVBLOCKS_API int32_t AVBLOCKS_CALL avb_set_license(const char* license);
    AVBLOCKS_API primo::avblocks::Config* AVBLOCKS_CALL avb_config();
    AVBLOCKS_API primo::license::LicenseInfo* AVBLOCKS_CALL avb_create_license_info();
    AVBLOCKS_API primo::avblocks::Transcoder* AVBLOCKS_CALL avb_create_transcoder();
    AVBLOCKS_API primo::error::ErrorInfo* AVBLOCKS_CALL avb_create_error_info();
    AVBLOCKS_API primo::codecs::AudioStreamInfo* AVBLOCKS_CALL avb_create_audio_stream_info();
    AVBLOCKS_API primo::codecs::VideoStreamInfo* AVBLOCKS_CALL avb_create_video_stream_info();
    AVBLOCKS_API primo::codecs::StreamInfo* AVBLOCKS_CALL avb_create_data_stream_info();
    AVBLOCKS_API primo::codecs::MediaSample* AVBLOCKS_CALL avb_create_media_sample();
    AVBLOCKS_API primo::codecs::MediaBuffer* AVBLOCKS_CALL avb_create_media_buffer(int32_t capacity);
    AVBLOCKS_API primo::avblocks::MediaPin* AVBLOCKS_CALL avb_create_media_pin();
    AVBLOCKS_API primo::avblocks::MediaSocket* AVBLOCKS_CALL avb_create_media_socket();
    AVBLOCKS_API primo::avblocks::MediaSocket* AVBLOCKS_CALL avb_create_media_socket_from_media_info(primo::avblocks::MediaInfo* mediaInfo);
    AVBLOCKS_API primo::avblocks::MediaSocket* AVBLOCKS_CALL avb_create_media_socket_from_preset(const char* preset);
    AVBLOCKS_API primo::avblocks::MediaInfo* AVBLOCKS_CALL avb_create_media_info();
    AVBLOCKS_API primo::avblocks::MediaBufferParameter* AVBLOCKS_CALL avb_create_media_buffer_parameter();
    AVBLOCKS_API primo::avblocks::StringParameter* AVBLOCKS_CALL avb_create_string_parameter();
    AVBLOCKS_API primo::avblocks::VideoStreamInfoParameter* AVBLOCKS_CALL avb_create_video_stream_info_parameter();
    AVBLOCKS_API primo::avblocks::IntParameter* AVBLOCKS_CALL avb_create_int_parameter();
    AVBLOCKS_API primo::avblocks::FloatParameter* AVBLOCKS_CALL avb_create_float_parameter();
    AVBLOCKS_API primo::avblocks::ParameterList* AVBLOCKS_CALL avb_create_parameter_list();
    AVBLOCKS_API primo::codecs::Metadata* AVBLOCKS_CALL avb_create_metadata();
    AVBLOCKS_API primo::codecs::MetaAttribute*	AVBLOCKS_CALL avb_create_meta_attribute();
    AVBLOCKS_API primo::codecs::MetaPicture*	AVBLOCKS_CALL avb_create_meta_picture();
    AVBLOCKS_API primo::codecs::Hardware* AVBLOCKS_CALL avb_create_hardware();

#ifdef __cplusplus
}
#endif


namespace primo
{
    namespace avblocks 
    {
    
    /**
        Initializes AVBlocks and manages license information.

        This namespace contains global functions for the AVBlocks library. 
        These are C++ wrappers of the functions in the pure "C" interface exported by the library.
    */
    namespace Library
    {
        /**
            Initializes the AVBlocks library. Must be called once before the library is used,
            usually when the application starts.

            @return TRUE if the library is successfully initialized, otherwise FALSE.
        */
        inline bool_t initialize()
        {
            return avb_initialize();
        }

        /**
            Closes the AVBlocks library and frees the resources that are used (if any).
            Should be called when the library will not be used anymore, e.g. when the application ends.
        */
        inline void shutdown()
        {
            avb_shutdown();
        }

        /**
            Gets library build description.

            @return ANSI string.

            @remarks Contains details about the build including full version,
            architecture (x86/x64), configuration (Release/Debug), license (OEM/Demo) and platform (OS).
        */
        inline const char* description()
        {
            return avb_get_description();
        }

        /**
        Gets library major version.

        @return major version
        */
        inline int32_t majorVersion()
        {
            return avb_get_major_version();
        }

        /**
            Gets library minor version.

            @return minor version
        */
        inline int32_t minorVersion()
        {
            return avb_get_minor_version();
        }

        /**
            Gets library patch version.

            @return patch version

            @remarks The patch version is used for bug fixes and small improvements that do not involve API changes
        */
        inline int32_t patchVersion()
        {
            return avb_get_patch_version();
        }

        /**
            Sets a license string. The supplied license is appended to the library license state.

            @param license A string with license information.
            If the license string is <b>NULL</b> the library license state is cleared.

            @return The status after the license is set.
            The returned value is a combination of flags from the \ref primo::license::LicenseStatusFlags::Enum "LicenseStatusFlags" enum.

            @remarks A Demo build of the library cannot be licensed and always returns \ref primo::license::LicenseStatusFlags::DemoBuild "LicenseStatusFlags::DemoBuild".

            @see primo::license::LicenseStatusFlags::Enum
        */
        inline int32_t setLicense(const char* license)
        {
            return avb_set_license(license);
        }
        
        /**
            Creates a new LicenseInfo instance.

            @return A pointer to LicenseInfo object.

            @see primo::license::LicenseInfo
        */
        inline primo::license::LicenseInfo* createLicenseInfo()
        {
            return avb_create_license_info();
        }

        /**
            Creates a Transcoder.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new Transcoder object.

            @see \ref object_management "Object Management"
            @see Transcoder
        */
        inline Transcoder* createTranscoder()
        {
            return avb_create_transcoder();
        }

        /**
            Creates a new ErrorInfo instance.

            @return A pointer to ErrorInfo object.

            @see primo::error::ErrorInfo
        */
        inline primo::error::ErrorInfo* createErrorInfo()
        {
            return avb_create_error_info();
        }

        /**
            Creates an empty audio stream info.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new AudioStreamInfo object.

            @see \ref object_management "Object Management"
            @see \ref primo::codecs::AudioStreamInfo "AudioStreamInfo"
        */
        inline primo::codecs::AudioStreamInfo* createAudioStreamInfo()
        {
            return avb_create_audio_stream_info();
        }

        /**
            Creates an empty video stream info.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new VideoStreamInfo object.

            @see \ref object_management "Object Management"
            @see \ref primo::codecs::VideoStreamInfo "VideoStreamInfo"
        */
        inline primo::codecs::VideoStreamInfo* createVideoStreamInfo()
        {
            return avb_create_video_stream_info();
        }

        /**
            Creates a stream info for generic data.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new StreamInfo object.

            @remarks The media type is always \link primo::codecs::MediaType::Data MediaType::Data\endlink and cannot be changed.

            @see \ref object_management "Object Management"
        */
        inline primo::codecs::StreamInfo* createDataStreamInfo()
        {
            return avb_create_data_stream_info();
        }

        /**
            Creates an empty media sample.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new MediaSample object.

            @see \ref object_management "Object Management"
            @see \ref primo::codecs::MediaSample "MediaSample"
        */
        inline primo::codecs::MediaSample* createMediaSample()
        {
            return avb_create_media_sample();
        }

        /**
            Creates a media buffer with the specified capacity.
            The caller is responsible for releasing the object when it is not needed anymore.

            @param capacity The required capacity must be greater or equal to 0.

            @return A pointer to the new MediaBuffer object if <i>capacity</i> >= 0, otherwise <b>NULL</b>.

            @remarks The returned object has a capacity that has been specified by the <i>capacity</i> parameter.
            It contains no data, i.e. dataSize() == 0.

            @see \ref object_management "Object Management"
            @see \ref primo::codecs::MediaBuffer "MediaBuffer"
        */
        inline primo::codecs::MediaBuffer* createMediaBuffer(int32_t capacity = 0)
        {
            return avb_create_media_buffer(capacity);
        }

        /**
            Creates a media pin. It is used in input and output sockets of the transcoder.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new MediaPin object.

            @see \ref object_management "Object Management"
            @see MediaPin
            @see MediaSocket
            @see Transcoder
        */
        inline primo::avblocks::MediaPin* createMediaPin()
        {
            return avb_create_media_pin();
        }

        /**
            Creates a media socket. It is used as an input or output point of the transcoder.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new MediaSocket object.

            @see \ref object_management "Object Management"
            @see MediaSocket
            @see MediaPin
            @see Transcoder
        */
        inline primo::avblocks::MediaSocket* createMediaSocket()
        {
            return avb_create_media_socket();
        }

        /**
            Creates a media socket from an AVBlocks \link primo::avblocks::Preset Preset\endlink.

            @param preset [in] A pointer to a preset string. The known presets are listed in the primo::avblocks::Preset namespace.

            @return A pointer to the new MediaSocket object which is configured according to the specified preset.
        */
        inline primo::avblocks::MediaSocket* createMediaSocket(const char* preset)
        {
            return avb_create_media_socket_from_preset(preset);
        }

        /**
            Creates a media socket from a MediaInfo object. This method will call MediaInfo::load, if the MediaInfo has not been loaded.

            @param mediaInfo [in] A pointer to a MediaInfo object with audio/video information. This object is not needed after the method returns.

            @return A pointer to a new <b>MediaSocket</b> object. <br />
            The pointer to a new MediaSocket object contains the information supplied by the <i>mediaInfo</i> parameter.<br />
            The caller is responsible for releasing the returned media socket when it is not needed anymore.

            @return <b>NULL</b> if there was an error. In this case call MediaInfo::error() for detailed error information. <br />

            This method is meant to help in setting correctly the inputs of the Transcoder.
            The intended usage is as follows:
            <ol>
            <li> Use MediaInfo to load information for a file or user stream. </li>
            <li> Create a media socket from the MediaInfo object (Library::createMediaSocket(MediaInfo*)). </li>
            <li> Add the newly created socket to the inputs of the Transcoder (Transcoder::inputs(), MediaSocketList::add()). </li>
            <li> Release the media socket if it will not be used outside of the Transcoder. </li>
            </ol>

            @see MediaInfo
            @see \ref object_management "Object Management"
        */
        inline primo::avblocks::MediaSocket* createMediaSocket(MediaInfo * mediaInfo)
        {
            return avb_create_media_socket_from_media_info(mediaInfo);
        }

        /**
            Creates a media information object.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new MediaInfo object.

            @see \ref object_management "Object Management"
            @see MediaInfo
        */
        inline primo::avblocks::MediaInfo* createMediaInfo()
        {
            return avb_create_media_info();
        }

        /**
            Creates a parameter that holds a MediaBuffer object.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new Parameter object.

            @see \ref object_management "Object Management"
            @see ParameterList
        */
        inline primo::avblocks::MediaBufferParameter* createMediaBufferParameter()
        {
            return avb_create_media_buffer_parameter();
        }

        /**
            Creates a parameter that holds a char_t* string.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new Parameter object.

            @see \ref object_management "Object Management"
            @see ParameterList
        */
        inline primo::avblocks::StringParameter* createStringParameter()
        {
            return avb_create_string_parameter();
        }

        /**
            Creates a parameter that holds a VideoStreamInfoParameter object.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new Parameter object.

            @see \ref object_management "Object Management"
            @see ParameterList
        */
        inline primo::avblocks::VideoStreamInfoParameter* createVideoStreamInfoParameter()
        {
            return avb_create_video_stream_info_parameter();
        }

        /**
            Creates an integer parameter.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new Parameter object.

            @see \ref object_management "Object Management"
            @see ParameterList
        */
        inline primo::avblocks::IntParameter* createIntParameter()
        {
            return avb_create_int_parameter();
        }

        /**
            Creates a float parameter.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new Parameter object.

            @see \ref object_management "Object Management"
            @see ParameterList
        */
        inline primo::avblocks::FloatParameter* createFloatParameter()
        {
            return avb_create_float_parameter();
        }

        /**
            Creates an empty collection of filter parameters.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new ParameterList object.

            @see \ref object_management "Object Management"
            @see ParameterList
        */
        inline primo::avblocks::ParameterList* createParameterList()
        {
            return avb_create_parameter_list();
        }

        /**
            Creates an empty Metadata object.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new Metadata object.

            @see \ref object_management "Object Management"
            @see \ref primo::codecs::Metadata "Metadata"
            @see MediaSocket::setMetadata
            @see MediaSocket::metadata
        */
        inline primo::codecs::Metadata* createMetadata()
        {
            return avb_create_metadata();
        }

        /**
            Creates an empty MetaAttribute object.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new MetaAttribute object.

            @see \ref object_management "Object Management"

            @see \ref primo::codecs::Metadata "Metadata"
            @see \ref primo::codecs::MetaAttribute "Metadata::attributes"
        */
        inline primo::codecs::MetaAttribute* createMetaAttribute()
        {
            return avb_create_meta_attribute();
        }

        /**
            Creates an empty MetaPicture object.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new MetaPicture object.

            @see \ref object_management "Object Management"

            @see \ref primo::codecs::Metadata "Metadata"
            @see \ref primo::codecs::MetaPicture "Metadata::pictures"
        */
        inline primo::codecs::MetaPicture* createMetaPicture()
        {
            return avb_create_meta_picture();
        }

        /**
            Creates a Hardware enumeration object.
            The caller is responsible for releasing the object when it is not needed anymore.

            @return A pointer to the new Hardware object.

            @see \ref object_management "Object Management"
            @see primo::codecs::Hardware
        */
        inline primo::codecs::Hardware* createHardware()
        {
            return avb_create_hardware();
        }

        /**
        Returns the library configuration object.

        @return A pointer to configuration object. This is never <b>NULL</b>.

        @see primo::avblocks::Config
        @see primo::avblocks::HwConfig
        */
        inline primo::avblocks::Config* config()
        {
            return avb_config();
        }
    };
    

}} // primo::avblocks


// Legend:
// FR:	- future release
// TBR:	- to be reviewed
// .NET: - update .NET version
// Java: - update Java version

#endif // ___AVBLOCKS_H___
